@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # deep copying
 
 # Note: deepcopy_internal(::Any, ::ObjectIdDict) is
@@ -5,18 +7,21 @@
 
 deepcopy(x) = deepcopy_internal(x, ObjectIdDict())
 
-deepcopy_internal(x::Union(Symbol,LambdaStaticData,TopNode,QuoteNode,
-                           DataType,UnionType,Task),
+deepcopy_internal(x::Union{Symbol,LambdaStaticData,TopNode,GlobalRef,
+                           DataType,Union,Task},
                   stackdict::ObjectIdDict) = x
 deepcopy_internal(x::Tuple, stackdict::ObjectIdDict) =
-    ntuple(length(x), i->deepcopy_internal(x[i], stackdict))
+    ntuple(i->deepcopy_internal(x[i], stackdict), length(x))
 deepcopy_internal(x::Module, stackdict::ObjectIdDict) = error("deepcopy of Modules not supported")
 
 function deepcopy_internal(x::Function, stackdict::ObjectIdDict)
-    if isa(x.env, Union(MethodTable, Symbol)) || x.env === ()
+    if isa(x.env, Union{MethodTable, Symbol}) || x.env === ()
         return x
     end
-    invoke(deepcopy_internal, (Any, ObjectIdDict), x, stackdict)
+    if haskey(stackdict, x)
+        return stackdict[x]
+    end
+    _deepcopy_t(x, typeof(x), stackdict)
 end
 
 function deepcopy_internal(x, stackdict::ObjectIdDict)
@@ -27,19 +32,17 @@ function deepcopy_internal(x, stackdict::ObjectIdDict)
 end
 
 function _deepcopy_t(x, T::DataType, stackdict::ObjectIdDict)
-    isbits(T) | isempty(T.names) && return x
+    nf = nfields(T)
+    (isbits(T) || nf == 0) && return x
+    y = ccall(:jl_new_struct_uninit, Any, (Any,), T)
     if T.mutable
-        y = ccall(:jl_new_struct_uninit, Any, (Any,), T)
         stackdict[x] = y
-        for i in 1:length(T.names)
-            if isdefined(x,i)
-                y.(i) = deepcopy_internal(x.(i), stackdict)
-            end
+    end
+    for i in 1:nf
+        if isdefined(x,i)
+            ccall(:jl_set_nth_field, Void, (Any, Csize_t, Any), y, i-1,
+                  deepcopy_internal(getfield(x,i), stackdict))
         end
-    else
-        fields = Any[deepcopy_internal(x.(i), stackdict) for i in 1:length(T.names)]
-        y = ccall(:jl_new_structv, Any, (Any, Ptr{Void}, Uint32),
-                  T, pointer(fields), length(fields))
     end
     return y::T
 end
