@@ -1,18 +1,29 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 export
 # Modules
     Collections,
     FFTW,
-    Graphics,
     Meta,
     Operators,
     Pkg,
     Profile,
+    Dates,
     Sys,
     Test,
+    Libc,
+    Libdl,
+    Mmap,
+    LinAlg,
     BLAS,
     LAPACK,
+    Serializer,
+    SparseMatrix,
+    Docs,
+    Markdown,
 
 # Types
+    AbstractChannel,
     AbstractMatrix,
     AbstractSparseArray,
     AbstractSparseMatrix,
@@ -27,14 +38,16 @@ export
     BitArray,
     BitMatrix,
     BitVector,
-    CFILE,
+    BufferStream,
+    CartesianIndex,
+    CartesianRange,
+    Channel,
     Cmd,
     Colon,
     Complex,
     Complex128,
     Complex64,
     Complex32,
-    DArray,
     DenseMatrix,
     DenseVecOrMat,
     DenseVector,
@@ -43,6 +56,7 @@ export
     Dict,
     Dims,
     EachLine,
+    Enum,
     Enumerate,
     Factorization,
     FileMonitor,
@@ -56,12 +70,17 @@ export
     IO,
     IOBuffer,
     IOStream,
+    LinSpace,
     LocalProcess,
-    MathConst,
+    LowerTriangular,
+    Irrational,
     Matrix,
     MergeSort,
+    Nullable,
     ObjectIdDict,
     OrdinalRange,
+    Pair,
+    PartialQuickSort,
     PollingFileWatcher,
     ProcessGroup,
     QuickSort,
@@ -73,13 +92,15 @@ export
     RemoteRef,
     RepString,
     RevString,
-    RopeString,
     RoundFromZero,
     RoundDown,
     RoundingMode,
     RoundNearest,
+    RoundNearestTiesAway,
+    RoundNearestTiesUp,
     RoundToZero,
     RoundUp,
+    SerializationState,
     Set,
     SharedArray,
     SharedMatrix,
@@ -92,24 +113,21 @@ export
     StridedVecOrMat,
     StridedVector,
     SubArray,
-    SubDArray,
-    SubOrDArray,
     SubString,
     Symmetric,
     SymTridiagonal,
     Timer,
-    TmStruct,
-    Triangular,
     Tridiagonal,
     UnitRange,
+    UpperTriangular,
     UTF16String,
     UTF32String,
+    Val,
     VecOrMat,
     Vector,
     VersionNumber,
     WeakKeyDict,
-    WeakRef,
-    Woodbury,
+    WorkerConfig,
     WString,
     Zip,
 
@@ -118,6 +136,7 @@ export
     Cdouble,
     Cfloat,
     Cint,
+    Cintmax_t,
     Clong,
     Clonglong,
     Coff_t,
@@ -127,50 +146,44 @@ export
     Cssize_t,
     Cuchar,
     Cuint,
+    Cuintmax_t,
     Culong,
     Culonglong,
     Cushort,
     Cwchar_t,
+    Cstring,
+    Cwstring,
 
 # Exceptions
     ArgumentError,
     DimensionMismatch,
+    CapturedException,
+    CompositeException,
     EOFError,
     ErrorException,
+    InvalidStateException,
     KeyError,
     LoadError,
+    InitError,
     MethodError,
+    NullException,
     ParseError,
     ProcessExitedException,
+    RemoteException,
     SystemError,
     TypeError,
+    AssertionError,
+    UnicodeError,
 
 # Global constants and variables
     ARGS,
     C_NULL,
     CPU_CORES,
-    DL_LOAD_PATH,
     ENDIAN_BOM,
     ENV,
-    Inf,
-    Inf16,
-    Inf32,
+    JULIA_HOME,
     LOAD_PATH,
-    MS_ASYNC,
-    MS_INVALIDATE,
-    MS_SYNC,
-    NaN,
-    NaN16,
-    NaN32,
     OS_NAME,
-    RTLD_DEEPBIND,
-    RTLD_FIRST,
-    RTLD_GLOBAL,
-    RTLD_LAZY,
-    RTLD_LOCAL,
-    RTLD_NODELETE,
-    RTLD_NOLOAD,
-    RTLD_NOW,
     STDERR,
     STDIN,
     STDOUT,
@@ -178,6 +191,14 @@ export
     WORD_SIZE,
 
 # Mathematical constants
+    Inf,
+    Inf16,
+    Inf32,
+    Inf64,
+    NaN,
+    NaN16,
+    NaN32,
+    NaN64,
     im,
     π, pi,
     e, eu,
@@ -205,6 +226,7 @@ export
     .-,
     .*,
     ./,
+    .%,
     .<,
     .<=,
     .≤,
@@ -227,15 +249,17 @@ export
     >=,
     ≥,
     >>,
-    #.>>,
-    #.<<,
+    .>>,
+    .<<,
     >>>,
     \,
     ^,
     |,
+    |>,
     ~,
     :,
     ÷,
+    =>,
     A_ldiv_B!,
     A_ldiv_Bc,
     A_ldiv_Bt,
@@ -262,6 +286,7 @@ export
     At_mul_Bt!,
     At_rdiv_B,
     At_rdiv_Bt,
+    call,
 
 # scalar math
     @evalpoly,
@@ -289,18 +314,15 @@ export
     atanh,
     big,
     binomial,
-    bool,
     bswap,
     cbrt,
     ceil,
     cis,
     clamp,
+    cld,
     cmp,
     combinations,
     complex,
-    complex128,
-    complex64,
-    complex32,
     conj,
     copysign,
     cos,
@@ -337,35 +359,23 @@ export
     factor,
     factorial,
     fld,
+    fldmod,
     flipsign,
     float,
-    float16,
-    float32,
-    float64,
+    tryparse,
     floor,
+    fma,
     frexp,
     gamma,
     gcd,
     gcdx,
     hex2num,
     hypot,
-    iceil,
-    ifloor,
     imag,
-    inf,
-    int,
-    int128,
-    int16,
-    int32,
-    int64,
-    int8,
-    integer,
     inv,
     invdigamma,
     invmod,
-    iround,
     isapprox,
-    iseltype,
     iseven,
     isfinite,
     isinf,
@@ -377,7 +387,6 @@ export
     isqrt,
     isreal,
     issubnormal,
-    itrunc,
     lcm,
     ldexp,
     leading_ones,
@@ -393,7 +402,7 @@ export
     mod1,
     modf,
     mod2pi,
-    nan,
+    muladd,
     nextfloat,
     nextpow,
     nextpow2,
@@ -407,6 +416,7 @@ export
     prevpow2,
     prevprod,
     primes,
+    primesmask,
     rad2deg,
     rationalize,
     real,
@@ -438,19 +448,16 @@ export
     trailing_zeros,
     trigamma,
     trunc,
+    unsafe_trunc,
     typemax,
     typemin,
-    uint,
-    uint128,
-    uint16,
-    uint32,
-    uint64,
-    uint8,
     unsigned,
     widemul,
     zero,
     √,
     ∛,
+    ≈,
+    ≉,
 
 # specfun
     airy,
@@ -461,6 +468,7 @@ export
     airyprime,
     airyx,
     besselh,
+    besselhx,
     besseli,
     besselix,
     besselj,
@@ -491,11 +499,11 @@ export
     broadcast_function,
     broadcast_getindex,
     broadcast_setindex!,
-    cartesianmap,
     cat,
     cell,
     checkbounds,
     circshift,
+    clamp!,
     colon,
     conj!,
     copy!,
@@ -506,21 +514,22 @@ export
     cumsum,
     cumsum!,
     cumsum_kbn,
+    eachindex,
     extrema,
     fill!,
     fill,
     find,
     findfirst,
+    findlast,
     findin,
     findmax,
     findmin,
     findn,
     findnext,
+    findprev,
     findnz,
     first,
     flipdim,
-    fliplr,
-    flipud,
     gradient,
     hcat,
     hvcat,
@@ -531,10 +540,11 @@ export
     invperm,
     ipermute!,
     ipermutedims,
+    isassigned,
     isperm,
     issorted,
     last,
-    linrange,
+    levicivita,
     linspace,
     logspace,
     mapslices,
@@ -558,6 +568,7 @@ export
     ones,
     parent,
     parentindexes,
+    parity,
     partitions,
     permutations,
     permute!,
@@ -592,7 +603,10 @@ export
     sort!,
     sort,
     sortcols,
+    selectperm,
+    selectperm!,
     sortperm,
+    sortperm!,
     sortrows,
     squeeze,
     step,
@@ -622,6 +636,7 @@ export
     cond,
     condskeel,
     cross,
+    ctranspose!,
     ctranspose,
     det,
     diag,
@@ -636,6 +651,7 @@ export
     eigmin,
     eigs,
     eigvals,
+    eigvals!,
     eigvecs,
     expm,
     eye,
@@ -643,6 +659,7 @@ export
     givens,
     hessfact!,
     hessfact,
+    isdiag,
     ishermitian,
     isposdef!,
     isposdef,
@@ -653,20 +670,23 @@ export
     ldltfact,
     ldltfact!,
     linreg,
+    logabsdet,
     logdet,
+    logm,
     lu,
     lufact!,
     lufact,
     lyap,
     norm,
-    null,
+    nullspace,
+    ordschur!,
+    ordschur,
     peakflops,
     pinv,
     qr,
     qrfact!,
     qrfact,
     rank,
-    rref,
     scale!,
     scale,
     schur,
@@ -676,15 +696,18 @@ export
     svd,
     svdfact!,
     svdfact,
+    svds,
     svdvals!,
     svdvals,
     sylvester,
     trace,
+    transpose!,
     transpose,
     tril!,
     tril,
     triu!,
     triu,
+    vecdot,
     vecnorm,
     ⋅,
     ×,
@@ -703,6 +726,8 @@ export
     sprandn,
     spzeros,
     symperm,
+    rowvals,
+    nzrange,
 
 # bitarrays
     bitpack,
@@ -710,7 +735,9 @@ export
     falses,
     flipbits!,
     rol,
+    rol!,
     ror,
+    ror!,
     trues,
 
 # dequeues
@@ -729,8 +756,6 @@ export
     any!,
     any,
     collect,
-    complement!,
-    complement,
     contains,
     count,
     delete!,
@@ -753,10 +778,14 @@ export
     isempty,
     issubset,
     keys,
+    keytype,
     length,
     map!,
     map,
+    mapfoldl,
+    mapfoldr,
     mapreduce,
+    mapreducedim,
     merge!,
     merge,
     #pop!,
@@ -766,7 +795,7 @@ export
     setdiff,
     setindex!,
     similar,
-    sizehint,
+    sizehint!,
     splice!,
     symdiff!,
     symdiff,
@@ -774,6 +803,7 @@ export
     union,
     unique,
     values,
+    valtype,
     ∈,
     ∉,
     ∋,
@@ -787,44 +817,41 @@ export
 # strings and text output
     ascii,
     base,
-    base64,
-    Base64Pipe,
-    beginswith,
+    base64encode,
+    base64decode,
+    Base64EncodePipe,
+    Base64DecodePipe,
+    startswith,
     bin,
     bits,
     bytes2hex,
     bytestring,
-    char,
     charwidth,
     chomp,
     chop,
     chr2ind,
     dec,
     digits,
+    digits!,
     dump,
     eachmatch,
     endswith,
     escape_string,
-    float32_isvalid,
-    float64_isvalid,
+    graphemes,
     hex,
     hex2bytes,
     ind2chr,
     info,
     is_assigned_char,
-    is_valid_ascii,
-    is_valid_char,
-    is_valid_utf8,
-    is_valid_utf16,
     isalnum,
     isalpha,
     isascii,
-    isblank,
     iscntrl,
     isdigit,
     isgraph,
     islower,
     ismatch,
+    isnumber,
     isprint,
     ispunct,
     isspace,
@@ -842,8 +869,6 @@ export
     nextind,
     normalize_string,
     oct,
-    parsefloat,
-    parseint,
     prevind,
     print,
     print_escaped,
@@ -856,6 +881,7 @@ export
     repeat,
     replace,
     repr,
+    reverseind,
     rpad,
     rsearch,
     rsearchindex,
@@ -886,13 +912,16 @@ export
 # random numbers
     AbstractRNG,
     MersenneTwister,
+    RandomDevice,
     rand!,
     rand,
-    randbool!,
-    randbool,
     randn!,
     randn,
+    randexp!,
+    randexp,
     srand,
+    bitrand,
+    randjump,
 
 # bigfloat & precision
     precision,
@@ -902,12 +931,16 @@ export
     get_rounding,
     set_rounding,
     with_rounding,
+    get_zero_subnormals,
+    set_zero_subnormals,
 
 # statistics
     cor,
     cov,
     hist,
+    hist!,
     hist2d,
+    hist2d!,
     histrange,
     mean!,
     mean,
@@ -958,7 +991,7 @@ export
     rfft,
     xcorr,
 
-#   numerical integration
+# numerical integration
     quadgk,
 
 # iteration
@@ -967,6 +1000,12 @@ export
     next,
     start,
     zip,
+    rest,
+    countfrom,
+    take,
+    drop,
+    cycle,
+    repeated,
 
 # object identity and equality
     copy,
@@ -987,23 +1026,31 @@ export
     Condition,
     consume,
     current_task,
+    istaskstarted,
     istaskdone,
+    lock,
     notify,
     produce,
+    ReentrantLock,
     schedule,
     task_local_storage,
+    unlock,
     yield,
+    yieldto,
 
 # time
     sleep,
-    strftime,
-    strptime,
     tic,
     time,
     time_ns,
     toc,
     toq,
 
+# dates
+    Date,
+    DateTime,
+    now,
+
 # errors
     assert,
     backtrace,
@@ -1015,12 +1062,14 @@ export
 # types
     convert,
     fieldoffsets,
+    fieldnames,
     isleaftype,
     oftype,
     promote,
     promote_rule,
     promote_type,
     subtypes,
+    instances,
     super,
     typeintersect,
     typejoin,
@@ -1039,13 +1088,12 @@ export
     current_module,
     edit,
     code_typed,
+    code_warntype,
     code_lowered,
     code_llvm,
     code_native,
     fullname,
     functionloc,
-    functionlocs,
-    help,
     isconst,
     isgeneric,
     isinteractive,
@@ -1061,22 +1109,23 @@ export
     workspace,
 
 # loading source files
+    __precompile__,
     evalfile,
     include,
     include_string,
+    include_dependency,
     reload,
-    require,
 
 # RTS internals
     finalizer,
+    finalize,
     gc,
-    gc_disable,
     gc_enable,
     precompile,
-    clear_malloc_data,
 
 # misc
     atexit,
+    atreplinit,
     clipboard,
     exit,
     ntuple,
@@ -1100,7 +1149,6 @@ export
     fd,
     fdio,
     flush,
-    flush_cstdio,
     getaddrinfo,
     gethostname,
     getipaddr,
@@ -1113,13 +1161,11 @@ export
     listenany,
     ltoh,
     mark,
-    mmap,
-    mmap_array,
-    mmap_bitarray,
-    msync,
     nb_available,
     ntoh,
     open,
+    pipeline,
+    Pipe,
     PipeBuffer,
     poll_fd,
     poll_file,
@@ -1142,6 +1188,7 @@ export
     redirect_stdin,
     redirect_stdout,
     recv,
+    recvfrom,
     reset,
     seek,
     seekend,
@@ -1150,11 +1197,6 @@ export
     serialize,
     skip,
     skipchars,
-    start_reading,
-    start_timer,
-    start_watching,
-    stop_reading,
-    stop_timer,
     takebuf_array,
     takebuf_string,
     truncate,
@@ -1163,14 +1205,18 @@ export
     write,
     writecsv,
     writedlm,
-    UdpSocket,
+    TCPSocket,
+    UDPSocket,
 
 # multiprocessing
     addprocs,
     ClusterManager,
     fetch,
+    init_worker,
     interrupt,
     isready,
+    launch,
+    manage,
     myid,
     nprocs,
     nworkers,
@@ -1202,21 +1248,13 @@ export
     popdisplay,
     pushdisplay,
     redisplay,
-
-# distributed arrays
-    dfill,
-    distribute,
-    dones,
-    drand,
-    drandn,
-    dzeros,
-    localpart,
-    localindexes,
-    procs,
+    HTML,
+    Text,
 
 # shared arrays
     sdata,
     indexpids,
+    localindexes,
 
 # paths and file names
     abspath,
@@ -1229,6 +1267,7 @@ export
     joinpath,
     normpath,
     realpath,
+    relpath,
     splitdir,
     splitdrive,
     splitext,
@@ -1249,6 +1288,7 @@ export
     isfifo,
     isfile,
     islink,
+    ismount,
     ispath,
     isreadable,
     issetgid,
@@ -1265,6 +1305,7 @@ export
     mv,
     operm,
     pwd,
+    readlink,
     rm,
     stat,
     symlink,
@@ -1285,57 +1326,69 @@ export
     setenv,
     spawn,
     success,
+    withenv,
 
 # C interface
-    c_free,
-    c_malloc,
-    c_calloc,
-    c_realloc,
     cfunction,
     cglobal,
     disable_sigint,
-    dlclose,
-    dlopen,
-    dlopen_e,
-    dlsym,
-    dlsym_e,
-    errno,
-    find_library,
     pointer,
     pointer_from_objref,
     pointer_to_array,
+    pointer_to_string,
     reenable_sigint,
-    strerror,
     unsafe_copy!,
     unsafe_load,
     unsafe_pointer_to_objref,
     unsafe_store!,
 
+# nullable types
+    isnull,
+
 # Macros
+    # parser internal
     @__FILE__,
-    @b_str,
-    @r_str,
-    @r_mstr,
-    @v_str,
     @int128_str,
     @uint128_str,
-    @bigint_str,
-    @mstr,
-    @unexpected,
-    @assert,
-    @cmd,
+    @big_str,
+    @cmd,    # `commands`
+
+    # notation for certain types
+    @b_str,  # byte vector
+    @r_str,  # regex
+    @s_str,  # regex substitution string
+    @v_str,  # version number
+
+    # documentation
+    @text_str,
+    @html_str,
+    @doc,
+    @doc_str,
+
+    # output
+    @show,
+    @printf,
+    @sprintf,
+
+    # profiling
     @time,
     @timed,
+    @timev,
     @elapsed,
     @allocated,
     @profile,
+
+    # reflection
     @which,
     @edit,
     @less,
     @code_typed,
+    @code_warntype,
     @code_lowered,
     @code_llvm,
     @code_native,
+
+    # platform-conditional code
     @windows,
     @unix,
     @osx,
@@ -1344,29 +1397,38 @@ export
     @unix_only,
     @osx_only,
     @linux_only,
-    @windowsxp_only,
-    @non_windowsxp_only,
+
+    # tasks
     @schedule,
     @sync,
     @async,
+    @task,
+
+    # multiprocessing
     @spawn,
     @spawnat,
     @fetch,
     @fetchfrom,
     @everywhere,
     @parallel,
+
+    # metaprogramming utilities
+    @generated,
     @gensym,
     @eval,
-    @task,
-    @thunk,
     @vectorize_1arg,
     @vectorize_2arg,
-    @show,
-    @printf,
-    @sprintf,
     @deprecate,
+
+    # performance annotations
     @boundscheck,
     @inbounds,
+    @fastmath,
     @simd,
+    @inline,
+    @noinline,
+
+    @assert,
+    @enum,
     @label,
     @goto
