@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module GMP
 
 export BigInt
@@ -5,23 +7,40 @@ export BigInt
 import Base: *, +, -, /, <, <<, >>, >>>, <=, ==, >, >=, ^, (~), (&), (|), ($),
              binomial, cmp, convert, div, divrem, factorial, fld, gcd, gcdx, lcm, mod,
              ndigits, promote_rule, rem, show, isqrt, string, isprime, powermod,
-             widemul, sum, trailing_zeros, trailing_ones, count_ones, base, parseint,
-             serialize, deserialize, bin, oct, dec, hex, isequal, invmod,
-             prevpow2, nextpow2, ndigits0z, widen
+             sum, trailing_zeros, trailing_ones, count_ones, base, tryparse_internal,
+             bin, oct, dec, hex, isequal, invmod, prevpow2, nextpow2, ndigits0z, widen, signed, unsafe_trunc, trunc
 
 if Clong == Int32
-    typealias ClongMax Union(Int8, Int16, Int32)
-    typealias CulongMax Union(Uint8, Uint16, Uint32)
+    typealias ClongMax Union{Int8, Int16, Int32}
+    typealias CulongMax Union{UInt8, UInt16, UInt32}
 else
-    typealias ClongMax Union(Int8, Int16, Int32, Int64)
-    typealias CulongMax Union(Uint8, Uint16, Uint32, Uint64)
+    typealias ClongMax Union{Int8, Int16, Int32, Int64}
+    typealias CulongMax Union{UInt8, UInt16, UInt32, UInt64}
 end
-typealias CdoubleMax Union(Float16, Float32, Float64)
+typealias CdoubleMax Union{Float16, Float32, Float64}
+
+gmp_version() = VersionNumber(bytestring(unsafe_load(cglobal((:__gmp_version, :libgmp), Ptr{Cchar}))))
+gmp_bits_per_limb() = Int(unsafe_load(cglobal((:__gmp_bits_per_limb, :libgmp), Cint)))
+
+const GMP_VERSION = gmp_version()
+const GMP_BITS_PER_LIMB = gmp_bits_per_limb()
+
+# GMP's mp_limb_t is by default a typedef of `unsigned long`, but can also be configured to be either
+# `unsigned int` or `unsigned long long int`. The correct unsigned type is here named Limb, and must
+# be used whenever mp_limb_t is in the signature of ccall'ed GMP functions.
+if GMP_BITS_PER_LIMB == 32
+    typealias Limb UInt32
+elseif GMP_BITS_PER_LIMB == 64
+    typealias Limb UInt64
+else
+    error("GMP: cannot determine the type mp_limb_t (__gmp_bits_per_limb == $GMP_BITS_PER_LIMB)")
+end
+
 
 type BigInt <: Integer
     alloc::Cint
     size::Cint
-    d::Ptr{Culong}
+    d::Ptr{Limb}
     function BigInt()
         b = new(zero(Cint), zero(Cint), C_NULL)
         ccall((:__gmpz_init,:libgmp), Void, (Ptr{BigInt},), &b)
@@ -34,56 +53,94 @@ _gmp_clear_func = C_NULL
 _mpfr_clear_func = C_NULL
 
 function __init__()
-    global _gmp_clear_func = cglobal((:__gmpz_clear, :libgmp))
-    global _mpfr_clear_func = cglobal((:mpfr_clear, :libmpfr))
-    ccall((:__gmp_set_memory_functions, :libgmp), Void,
-          (Ptr{Void},Ptr{Void},Ptr{Void}),
-          cglobal(:jl_gc_counted_malloc),
-          cglobal(:jl_gc_counted_realloc_with_old_size),
-          cglobal(:jl_gc_counted_free))
+    try
+        if gmp_version().major != GMP_VERSION.major || gmp_bits_per_limb() != GMP_BITS_PER_LIMB
+            error(string("The dynamically loaded GMP library (version $(gmp_version()) with __gmp_bits_per_limb == $(gmp_bits_per_limb()))\n",
+                         "does not correspond to the compile time version (version $GMP_VERSION with __gmp_bits_per_limb == $GMP_BITS_PER_LIMB).\n",
+                         "Please rebuild Julia."))
+        end
+
+        global _gmp_clear_func = cglobal((:__gmpz_clear, :libgmp))
+        global _mpfr_clear_func = cglobal((:mpfr_clear, :libmpfr))
+        ccall((:__gmp_set_memory_functions, :libgmp), Void,
+              (Ptr{Void},Ptr{Void},Ptr{Void}),
+              cglobal(:jl_gc_counted_malloc),
+              cglobal(:jl_gc_counted_realloc_with_old_size),
+              cglobal(:jl_gc_counted_free))
+    catch ex
+        Base.showerror_nostdio(ex,
+            "WARNING: Error during initialization of module GMP")
+    end
 end
 
 widen(::Type{Int128})  = BigInt
-widen(::Type{Uint128}) = BigInt
+widen(::Type{UInt128}) = BigInt
 widen(::Type{BigInt})  = BigInt
 
-BigInt(x::BigInt) = x
-BigInt(s::String) = parseint(BigInt,s)
+signed(x::BigInt) = x
 
-function Base.parseint_nocheck(::Type{BigInt}, s::String, base::Int)
-    s = bytestring(s)
-    sgn, base, i = Base.parseint_preamble(true,s,base)
+convert(::Type{BigInt}, x::BigInt) = x
+
+function tryparse_internal(::Type{BigInt}, s::AbstractString, startpos::Int, endpos::Int, base::Int, raise::Bool)
+    _n = Nullable{BigInt}()
+
+    # don't make a copy in the common case where we are parsing a whole bytestring
+    bstr = startpos == start(s) && endpos == endof(s) ? bytestring(s) : bytestring(SubString(s,startpos,endpos))
+
+    sgn, base, i = Base.parseint_preamble(true,base,bstr,start(bstr),endof(bstr))
+    if i == 0
+        raise && throw(ArgumentError("premature end of integer: $(repr(bstr))"))
+        return _n
+    end
     z = BigInt()
-    err = ccall((:__gmpz_set_str, :libgmp),
-               Int32, (Ptr{BigInt}, Ptr{Uint8}, Int32),
-               &z, convert(Ptr{Uint8},SubString(s,i)), base)
-    err == 0 || error("invalid big integer: $(repr(s))")
-    return sgn < 0 ? -z : z
+    if Base.containsnul(bstr)
+        err = -1 # embedded NUL char (not handled correctly by GMP)
+    else
+        err = ccall((:__gmpz_set_str, :libgmp),
+                    Int32, (Ptr{BigInt}, Ptr{UInt8}, Int32),
+                    &z, pointer(bstr)+(i-start(bstr)), base)
+    end
+    if err != 0
+        raise && throw(ArgumentError("invalid BigInt: $(repr(bstr))"))
+        return _n
+    end
+    Nullable(sgn < 0 ? -z : z)
 end
 
-function BigInt(x::Union(Clong,Int32))
+function convert(::Type{BigInt}, x::Union{Clong,Int32})
     z = BigInt()
     ccall((:__gmpz_set_si, :libgmp), Void, (Ptr{BigInt}, Clong), &z, x)
     return z
 end
-function BigInt(x::Union(Culong,Uint32))
+function convert(::Type{BigInt}, x::Union{Culong,UInt32})
     z = BigInt()
     ccall((:__gmpz_set_ui, :libgmp), Void, (Ptr{BigInt}, Culong), &z, x)
     return z
 end
 
-BigInt(x::Bool) = BigInt(uint(x))
+convert(::Type{BigInt}, x::Bool) = BigInt(UInt(x))
 
-function BigInt(x::Float64)
-    !isinteger(x) && throw(InexactError())
+
+function unsafe_trunc(::Type{BigInt}, x::Union{Float32,Float64})
     z = BigInt()
     ccall((:__gmpz_set_d, :libgmp), Void, (Ptr{BigInt}, Cdouble), &z, x)
     return z
 end
 
-BigInt(x::Union(Float16,Float32)) = BigInt(float64(x))
+function convert(::Type{BigInt}, x::Union{Float32,Float64})
+    isinteger(x) || throw(InexactError())
+    unsafe_trunc(BigInt,x)
+end
 
-function BigInt(x::Integer)
+function trunc(::Type{BigInt}, x::Union{Float32,Float64})
+    isfinite(x) || throw(InexactError())
+    unsafe_trunc(BigInt,x)
+end
+
+convert(::Type{BigInt}, x::Float16) = BigInt(Float64(x))
+convert(::Type{BigInt}, x::Float32) = BigInt(Float64(x))
+
+function convert(::Type{BigInt}, x::Integer)
     if x < 0
         if typemin(Clong) <= x
             return BigInt(convert(Clong,x))
@@ -91,7 +148,7 @@ function BigInt(x::Integer)
         b = BigInt(0)
         shift = 0
         while x < -1
-            b += BigInt(~uint32(x&0xffffffff))<<shift
+            b += BigInt(~UInt32(x&0xffffffff))<<shift
             x >>= 32
             shift += 32
         end
@@ -103,7 +160,7 @@ function BigInt(x::Integer)
         b = BigInt(0)
         shift = 0
         while x > 0
-            b += BigInt(uint32(x&0xffffffff))<<shift
+            b += BigInt(UInt32(x&0xffffffff))<<shift
             x >>>= 32
             shift += 32
         end
@@ -111,79 +168,83 @@ function BigInt(x::Integer)
     end
 end
 
-convert(::Type{BigInt}, x::Integer) = BigInt(x)
-convert(::Type{BigInt}, x::Float16) = BigInt(x)
-convert(::Type{BigInt}, x::FloatingPoint) = BigInt(x)
 
-function convert(::Type{Int64}, x::BigInt)
-    lo = int64(convert(Culong, x & typemax(Uint32)))
-    hi = int64(convert(Clong, x >> 32))
-    hi << 32 | lo
+rem(x::BigInt, ::Type{Bool}) = ((x&1)!=0)
+function rem{T<:Union{Unsigned,Signed}}(x::BigInt, ::Type{T})
+    u = zero(T)
+    for l = 1:min(abs(x.size), cld(sizeof(T),sizeof(Limb)))
+        u += (unsafe_load(x.d,l)%T) << ((sizeof(Limb)<<3)*(l-1))
+    end
+    x.size < 0 ? -u : u
 end
-convert(::Type{Int32}, n::BigInt) = int32(convert(Clong, n))
-convert(::Type{Int16}, n::BigInt) = int16(convert(Clong, n))
-convert(::Type{Int8}, n::BigInt) = int8(convert(Clong, n))
 
-function convert(::Type{Clong}, n::BigInt)
-    fits = ccall((:__gmpz_fits_slong_p, :libgmp), Int32, (Ptr{BigInt},), &n) != 0
-    if fits
-        ccall((:__gmpz_get_si, :libgmp), Clong, (Ptr{BigInt},), &n)
+function convert{T<:Unsigned}(::Type{T}, x::BigInt)
+    if sizeof(T) < sizeof(Limb)
+        convert(T, convert(Limb,x))
     else
-        throw(InexactError())
+        0 <= x.size <= cld(sizeof(T),sizeof(Limb)) || throw(InexactError())
+        x % T
     end
 end
 
-function convert(::Type{Uint64}, x::BigInt)
-    lo = uint64(convert(Culong, x & typemax(Uint32)))
-    hi = uint64(convert(Culong, x >> 32))
-    hi << 32 | lo
-end
-convert(::Type{Uint32}, x::BigInt) = uint32(convert(Culong, x))
-convert(::Type{Uint16}, x::BigInt) = uint16(convert(Culong, x))
-convert(::Type{Uint8}, x::BigInt) = uint8(convert(Culong, x))
-
-function convert(::Type{Culong}, n::BigInt)
-    fits = ccall((:__gmpz_fits_ulong_p, :libgmp), Int32, (Ptr{BigInt},), &n) != 0
-    if fits
-        ccall((:__gmpz_get_ui, :libgmp), Culong, (Ptr{BigInt},), &n)
+function convert{T<:Signed}(::Type{T}, x::BigInt)
+    n = abs(x.size)
+    if sizeof(T) < sizeof(Limb)
+        SLimb = typeof(Signed(one(Limb)))
+        convert(T, convert(SLimb, x))
     else
-        throw(InexactError())
+        0 <= n <= cld(sizeof(T),sizeof(Limb)) || throw(InexactError())
+        y = x % T
+        (x.size > 0) $ (y > 0) && throw(InexactError()) # catch overflow
+        y
     end
 end
 
-if sizeof(Int32) == sizeof(Clong)
-    function convert(::Type{Uint128}, x::BigInt)
-        uint128(uint(x>>>96))<<96 +
-        uint128(uint((x>>>64) & typemax(Uint32)))<<64 +
-        uint128(uint((x>>>32) & typemax(Uint32)))<<32 +
-        uint128(uint(x & typemax(Uint32)))
-    end
-end
-if sizeof(Int64) == sizeof(Clong)
-    function convert(::Type{Uint128}, x::BigInt)
-        uint128(uint(x>>>64))<<64 +
-        uint128(uint(x & typemax(Uint64)))
-    end
-end
-convert(::Type{Int128}, x::BigInt) = copysign(int128(uint128(abs(x))),x)
 
-function convert(::Type{Float64}, n::BigInt)
-    # TODO: this should round to nearest but instead rounds to zero
+function call(::Type{Float64}, n::BigInt, ::RoundingMode{:ToZero})
     ccall((:__gmpz_get_d, :libgmp), Float64, (Ptr{BigInt},), &n)
 end
-convert(::Type{Float32}, n::BigInt) = float32(float64(n))
-convert(::Type{Float16}, n::BigInt) = float16(float64(n))
 
-promote_rule{T<:Integer}(::Type{BigInt}, ::Type{T}) = BigInt
-
-# serialization
-
-function serialize(s, n::BigInt)
-    Base.serialize_type(s, BigInt)
-    serialize(s, base(62,n))
+function call{T<:Union{Float16,Float32}}(::Type{T}, n::BigInt, ::RoundingMode{:ToZero})
+    T(Float64(n,RoundToZero),RoundToZero)
 end
 
-deserialize(s, ::Type{BigInt}) = Base.parseint_nocheck(BigInt, deserialize(s), 62)
+function call{T<:CdoubleMax}(::Type{T}, n::BigInt, ::RoundingMode{:Down})
+    x = T(n,RoundToZero)
+    x > n ? prevfloat(x) : x
+end
+function call{T<:CdoubleMax}(::Type{T}, n::BigInt, ::RoundingMode{:Up})
+    x = T(n,RoundToZero)
+    x < n ? nextfloat(x) : x
+end
+
+function call{T<:CdoubleMax}(::Type{T}, n::BigInt, ::RoundingMode{:Nearest})
+    x = T(n,RoundToZero)
+    if maxintfloat(T) <= abs(x) < T(Inf)
+        r = n-BigInt(x)
+        h = eps(x)/2
+        if iseven(reinterpret(Unsigned,x)) # check if last bit is odd/even
+            if r < -h
+                return prevfloat(x)
+            elseif r > h
+                return nextfloat(x)
+            end
+        else
+            if r <= -h
+                return prevfloat(x)
+            elseif r >= h
+                return nextfloat(x)
+            end
+        end
+    end
+    x
+end
+
+convert(::Type{Float64}, n::BigInt) = Float64(n,RoundNearest)
+convert(::Type{Float32}, n::BigInt) = Float32(n,RoundNearest)
+convert(::Type{Float16}, n::BigInt) = Float16(n,RoundNearest)
+
+promote_rule{T<:Integer}(::Type{BigInt}, ::Type{T}) = BigInt
 
 # Binary ops
 for (fJ, fC) in ((:+, :add), (:-,:sub), (:*, :mul),
@@ -256,10 +317,10 @@ function -(c::CulongMax, x::BigInt)
     ccall((:__gmpz_ui_sub, :libgmp), Void, (Ptr{BigInt}, Culong, Ptr{BigInt}), &z, c, &x)
     return z
 end
-+(x::BigInt, c::ClongMax) = c < 0 ? -(x, convert(Culong, -c)) : x + convert(Culong, c)
-+(c::ClongMax, x::BigInt) = c < 0 ? -(x, convert(Culong, -c)) : x + convert(Culong, c)
--(x::BigInt, c::ClongMax) = c < 0 ? +(x, convert(Culong, -c)) : -(x, convert(Culong, c))
--(c::ClongMax, x::BigInt) = c < 0 ? -(x + convert(Culong, -c)) : -(convert(Culong, c), x)
++(x::BigInt, c::ClongMax) = c < 0 ? -(x, -(c % Culong)) : x + convert(Culong, c)
++(c::ClongMax, x::BigInt) = c < 0 ? -(x, -(c % Culong)) : x + convert(Culong, c)
+-(x::BigInt, c::ClongMax) = c < 0 ? +(x, -(c % Culong)) : -(x, convert(Culong, c))
+-(c::ClongMax, x::BigInt) = c < 0 ? -(x + -(c % Culong)) : -(convert(Culong, c), x)
 
 function *(x::BigInt, c::CulongMax)
     z = BigInt()
@@ -285,7 +346,7 @@ for (fJ, fC) in ((:-, :neg), (:~, :com))
     end
 end
 
-function <<(x::BigInt, c::Int32)
+function <<(x::BigInt, c::Int)
     c < 0 && throw(DomainError())
     c == 0 && return x
     z = BigInt()
@@ -293,7 +354,7 @@ function <<(x::BigInt, c::Int32)
     return z
 end
 
-function >>(x::BigInt, c::Int32)
+function >>(x::BigInt, c::Int)
     c < 0 && throw(DomainError())
     c == 0 && return x
     z = BigInt()
@@ -301,12 +362,12 @@ function >>(x::BigInt, c::Int32)
     return z
 end
 
->>>(x::BigInt, c::Int32) = x >> c
+>>>(x::BigInt, c::Int) = x >> c
 
-trailing_zeros(x::BigInt) = int(ccall((:__gmpz_scan1, :libgmp), Culong, (Ptr{BigInt}, Culong), &x, 0))
-trailing_ones(x::BigInt) = int(ccall((:__gmpz_scan0, :libgmp), Culong, (Ptr{BigInt}, Culong), &x, 0))
+trailing_zeros(x::BigInt) = Int(ccall((:__gmpz_scan1, :libgmp), Culong, (Ptr{BigInt}, Culong), &x, 0))
+trailing_ones(x::BigInt) = Int(ccall((:__gmpz_scan0, :libgmp), Culong, (Ptr{BigInt}, Culong), &x, 0))
 
-count_ones(x::BigInt) = int(ccall((:__gmpz_popcount, :libgmp), Culong, (Ptr{BigInt},), &x))
+count_ones(x::BigInt) = Int(ccall((:__gmpz_popcount, :libgmp), Culong, (Ptr{BigInt},), &x))
 
 function divrem(x::BigInt, y::BigInt)
     z1 = BigInt()
@@ -339,7 +400,7 @@ function isqrt(x::BigInt)
     return z
 end
 
-function ^(x::BigInt, y::Uint)
+function ^(x::BigInt, y::Culong)
     z = BigInt()
     ccall((:__gmpz_pow_ui, :libgmp), Void, (Ptr{BigInt}, Ptr{BigInt}, Culong), &z, &x, y)
     return z
@@ -349,8 +410,20 @@ function bigint_pow(x::BigInt, y::Integer)
     if y<0; throw(DomainError()); end
     if x== 1; return x; end
     if x==-1; return isodd(y) ? x : -x; end
-    if y>typemax(Uint); throw(DomainError()); end
-    return x^uint(y)
+    if y>typemax(Culong)
+       x==0 && return x
+
+       #At this point, x is not 1, 0 or -1 and it is not possible to use
+       #gmpz_pow_ui to compute the answer. Note that the magnitude of the
+       #answer is:
+       #- at least 2^(2^32-1) ≈ 10^(1.3e9) (if Culong === UInt32).
+       #- at least 2^(2^64-1) ≈ 10^(5.5e18) (if Culong === UInt64).
+       #
+       #Assume that the answer will definitely overflow.
+
+       throw(OverflowError())
+    end
+    return x^convert(Culong, y)
 end
 
 ^(x::BigInt , y::BigInt ) = bigint_pow(x, y)
@@ -407,12 +480,12 @@ function factorial(x::BigInt)
     return z
 end
 
-function binomial(n::BigInt, k::Uint)
+function binomial(n::BigInt, k::UInt)
     z = BigInt()
     ccall((:__gmpz_bin_ui, :libgmp), Void, (Ptr{BigInt}, Ptr{BigInt}, Culong), &z, &n, k)
     return z
 end
-binomial(n::BigInt, k::Integer) = k < 0 ? throw(DomainError()) : binomial(n, uint(k))
+binomial(n::BigInt, k::Integer) = k < 0 ? BigInt(0) : binomial(n, UInt(k))
 
 ==(x::BigInt, y::BigInt) = cmp(x,y) == 0
 ==(x::BigInt, i::Integer) = cmp(x,i) == 0
@@ -441,23 +514,23 @@ dec(n::BigInt) = base(10, n)
 hex(n::BigInt) = base(16, n)
 
 function base(b::Integer, n::BigInt)
-    2 <= b <= 62 || error("invalid base: $b")
-    p = ccall((:__gmpz_get_str,:libgmp), Ptr{Uint8}, (Ptr{Uint8}, Cint, Ptr{BigInt}), C_NULL, b, &n)
-    len = int(ccall(:strlen, Csize_t, (Ptr{Uint8},), p))
+    2 <= b <= 62 || throw(ArgumentError("base must be 2 ≤ base ≤ 62, got $b"))
+    p = ccall((:__gmpz_get_str,:libgmp), Ptr{UInt8}, (Ptr{UInt8}, Cint, Ptr{BigInt}), C_NULL, b, &n)
+    len = Int(ccall(:strlen, Csize_t, (Ptr{UInt8},), p))
     ASCIIString(pointer_to_array(p,len,true))
 end
 
 function ndigits0z(x::BigInt, b::Integer=10)
     b < 2 && throw(DomainError())
     if ispow2(b)
-        int(ccall((:__gmpz_sizeinbase,:libgmp), Culong, (Ptr{BigInt}, Int32), &x, b))
+        Int(ccall((:__gmpz_sizeinbase,:libgmp), Culong, (Ptr{BigInt}, Int32), &x, b))
     else
         # non-base 2 mpz_sizeinbase might return an answer 1 too big
         # use property that log(b, x) < ndigits(x, b) <= log(b, x) + 1
-        n = int(ccall((:__gmpz_sizeinbase,:libgmp), Culong, (Ptr{BigInt}, Int32), &x, 2))
+        n = Int(ccall((:__gmpz_sizeinbase,:libgmp), Culong, (Ptr{BigInt}, Int32), &x, 2))
         lb = log2(b) # assumed accurate to <1ulp (true for openlibm)
         q,r = divrem(n,lb)
-        iq = int(q)
+        iq = Int(q)
         maxerr = q*eps(lb) # maximum error in remainder
         if r-1.0 < maxerr
             abs(x) >= big(b)^iq ? iq+1 : iq
@@ -472,10 +545,11 @@ ndigits(x::BigInt, b::Integer=10) = x.size == 0 ? 1 : ndigits0z(x,b)
 
 isprime(x::BigInt, reps=25) = ccall((:__gmpz_probab_prime_p,:libgmp), Cint, (Ptr{BigInt}, Cint), &x, reps) > 0
 
-widemul(x::Int128, y::Uint128)  = BigInt(x)*BigInt(y)
-widemul(x::Uint128, y::Int128)  = BigInt(x)*BigInt(y)
-
 prevpow2(x::BigInt) = x.size < 0 ? -prevpow2(-x) : (x <= 2 ? x : one(BigInt) << (ndigits(x, 2)-1))
 nextpow2(x::BigInt) = x.size < 0 ? -nextpow2(-x) : (x <= 2 ? x : one(BigInt) << ndigits(x-1, 2))
 
+Base.checked_add(a::BigInt, b::BigInt) = a + b
+Base.checked_sub(a::BigInt, b::BigInt) = a - b
+Base.checked_mul(a::BigInt, b::BigInt) = a * b
+
 end # module
