@@ -1,26 +1,45 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 type IntSet
-    bits::Array{Uint32,1}
+    bits::Array{UInt32,1}
     limit::Int
     fill1s::Bool
 
-    IntSet() = new(zeros(Uint32,256>>>5), 256, false)
+    IntSet() = new(zeros(UInt32,256>>>5), 256, false)
 end
-
 IntSet(itr) = (s=IntSet(); for a in itr; push!(s,a); end; s)
 
+eltype(::Type{IntSet}) = Int64
 similar(s::IntSet) = IntSet()
 
+function show(io::IO, s::IntSet)
+    print(io, "IntSet([")
+    first = true
+    for n in s
+        if n > s.limit
+            break
+        end
+        if !first
+            print(io, ", ")
+        end
+        print(io, n)
+        first = false
+    end
+    if s.fill1s
+        print(io, ", ..., ", typemax(Int)-1)
+    end
+    print(io, "])")
+end
+
 copy(s::IntSet) = union!(IntSet(), s)
 
-eltype(s::IntSet) = Int64
-
-function sizehint(s::IntSet, top::Integer)
+function sizehint!(s::IntSet, top::Integer)
     if top >= s.limit
         lim = ((top+31) & -32)>>>5
         olsz = length(s.bits)
         if olsz < lim
             resize!(s.bits, lim)
-            fill = s.fill1s ? uint32(-1) : uint32(0)
+            fill = s.fill1s ? UInt32(-1) : UInt32(0)
             for i=(olsz+1):lim; s.bits[i] = fill; end
         end
         s.limit = top
@@ -33,13 +52,17 @@ function push!(s::IntSet, n::Integer)
         if s.fill1s
             return s
         else
-            lim = int(n + div(n,2))
-            sizehint(s, lim)
+            lim = Int(n + div(n,2))
+            sizehint!(s, lim)
+        end
+    elseif n <= 0
+        if n < 0
+            throw(ArgumentError("IntSet elements cannot be negative"))
+        else
+            depwarn("storing zero in IntSets is deprecated", :push!)
         end
-    elseif n < 0
-        throw(ArgumentError("IntSet elements cannot be negative"))
     end
-    s.bits[n>>5 + 1] |= (uint32(1)<<(n&31))
+    s.bits[n>>5 + 1] |= (UInt32(1)<<(n&31))
     return s
 end
 
@@ -53,13 +76,20 @@ end
 function pop!(s::IntSet, n::Integer, deflt)
     if n >= s.limit
         if s.fill1s
-            lim = int(n + div(n,2))
-            sizehint(s, lim)
+            lim = Int(n + div(n,2))
+            sizehint!(s, lim)
         else
             return deflt
         end
     end
-    mask = uint32(1)<<(n&31)
+    if n <= 0
+        if n < 0
+            return deflt
+        else
+            depwarn("stored zeros in IntSet is deprecated", :pop!)
+        end
+    end
+    mask = UInt32(1)<<(n&31)
     idx = n>>5 + 1
     b = s.bits[idx]
     if (b&mask)==0; return deflt; end
@@ -94,21 +124,31 @@ symdiff(s1::IntSet, s2::IntSet) =
     (s1.limit >= s2.limit ? symdiff!(copy(s1), s2) : symdiff!(copy(s2), s1))
 
 function empty!(s::IntSet)
-    s.bits[:] = 0
+    fill!(s.bits, 0)
     return s
 end
 
+"""
+    symdiff!(s, n)
+
+The set `s` is destructively modified to toggle the inclusion of integer `n`.
+"""
 function symdiff!(s::IntSet, n::Integer)
     if n >= s.limit
-        lim = int(n + dim(n,2))
-        sizehint(s, lim)
+        lim = Int(n + div(n,2))
+        sizehint!(s, lim)
     elseif n < 0
         throw(ArgumentError("IntSet elements cannot be negative"))
     end
-    s.bits[n>>5 + 1] $= (uint32(1)<<(n&31))
+    s.bits[n>>5 + 1] $= (UInt32(1)<<(n&31))
     return s
 end
 
+"""
+    symdiff!(s, itr)
+
+For each element in `itr`, destructively toggle its inclusion in set `s`.
+"""
 function symdiff!(s::IntSet, ns)
    for n in ns
        symdiff!(s, n)
@@ -117,40 +157,47 @@ function symdiff!(s::IntSet, ns)
 end
 
 function copy!(to::IntSet, from::IntSet)
-    empty!(to)
-    union!(to, from)
+    if is(to, from)
+        return to
+    else
+        empty!(to)
+        return union!(to, from)
+    end
 end
 
 function in(n::Integer, s::IntSet)
     if n >= s.limit
         # max IntSet length is typemax(Int), so highest possible element is
         # typemax(Int)-1
-        s.fill1s && n >= 0 && n < typemax(Int)
-    elseif n < 0
-        return false
-    else
-        (s.bits[n>>5 + 1] & (uint32(1)<<(n&31))) != 0
+        return s.fill1s && n >= 0 && n < typemax(Int)
+    elseif n <= 0
+        if n < 0
+            return false
+        else
+            depwarn("stored zeros in IntSet is deprecated", :in)
+        end
     end
+    (s.bits[n>>5 + 1] & (UInt32(1)<<(n&31))) != 0
 end
 
-start(s::IntSet) = int64(0)
+start(s::IntSet) = Int64(0)
 done(s::IntSet, i) = (!s.fill1s && next(s,i)[1] >= s.limit) || i == typemax(Int)
 function next(s::IntSet, i)
     if i >= s.limit
-        n = int64(i)
+        n = Int64(i)
     else
-        n = int64(ccall(:bitvector_next, Uint64, (Ptr{Uint32}, Uint64, Uint64), s.bits, i, s.limit))
+        n = Int64(ccall(:bitvector_next, UInt64, (Ptr{UInt32}, UInt64, UInt64), s.bits, i, s.limit))
     end
     (n, n+1)
 end
 
 isempty(s::IntSet) =
-    !s.fill1s && ccall(:bitvector_any1, Uint32, (Ptr{Uint32}, Uint64, Uint64), s.bits, 0, s.limit)==0
+    !s.fill1s && ccall(:bitvector_any1, UInt32, (Ptr{UInt32}, UInt64, UInt64), s.bits, 0, s.limit)==0
 
 function first(s::IntSet)
     n = next(s,0)[1]
     if n >= s.limit
-        error("set must be non-empty")
+        throw(ArgumentError("set must be non-empty"))
     end
     return n
 end
@@ -166,37 +213,17 @@ function last(s::IntSet)
             end
         end
     end
-    error("set has no last element")
+    throw(ArgumentError("set has no last element"))
 end
 
-length(s::IntSet) = int(ccall(:bitvector_count, Uint64, (Ptr{Uint32}, Uint64, Uint64), s.bits, 0, s.limit)) +
+length(s::IntSet) = Int(ccall(:bitvector_count, UInt64, (Ptr{UInt32}, UInt64, UInt64), s.bits, 0, s.limit)) +
     (s.fill1s ? typemax(Int) - s.limit : 0)
 
-function show(io::IO, s::IntSet)
-    print(io, "IntSet([")
-    first = true
-    for n in s
-        if n > s.limit
-            break
-        end
-        if !first
-            print(io, ", ")
-        end
-        print(io, n)
-        first = false
-    end
-    if s.fill1s
-        print(io, ", ..., ", typemax(Int)-1, ")")
-    else
-        print(io, "])")
-    end
-end
-
 
 # Math functions
 function union!(s::IntSet, s2::IntSet)
     if s2.limit > s.limit
-        sizehint(s, s2.limit)
+        sizehint!(s, s2.limit)
     end
     lim = length(s2.bits)
     for n = 1:lim
@@ -204,7 +231,7 @@ function union!(s::IntSet, s2::IntSet)
     end
     if s2.fill1s
         for n=lim+1:length(s.bits)
-            s.bits[n] = uint32(-1)
+            s.bits[n] = UInt32(-1)
         end
     end
     s.fill1s |= s2.fill1s
@@ -217,7 +244,7 @@ union(s1::IntSet, ss::IntSet...) = union(s1, union(ss...))
 
 function intersect!(s::IntSet, s2::IntSet)
     if s2.limit > s.limit
-        sizehint(s, s2.limit)
+        sizehint!(s, s2.limit)
     end
     lim = length(s2.bits)
     for n = 1:lim
@@ -225,7 +252,7 @@ function intersect!(s::IntSet, s2::IntSet)
     end
     if !s2.fill1s
         for n=lim+1:length(s.bits)
-            s.bits[n] = uint32(0)
+            s.bits[n] = UInt32(0)
         end
     end
     s.fill1s &= s2.fill1s
@@ -237,19 +264,14 @@ intersect(s1::IntSet, s2::IntSet) =
     (s1.limit >= s2.limit ? intersect!(copy(s1), s2) : intersect!(copy(s2), s1))
 intersect(s1::IntSet, ss::IntSet...) = intersect(s1, intersect(ss...))
 
-function complement!(s::IntSet)
-    for n = 1:length(s.bits)
-        s.bits[n] = ~s.bits[n]
-    end
-    s.fill1s = !s.fill1s
-    s
-end
-
-complement(s::IntSet) = complement!(copy(s))
+"""
+    symdiff!(s1, s2)
 
+Construct the symmetric difference of sets `s1` and `s2`, storing the result in `s1`.
+"""
 function symdiff!(s::IntSet, s2::IntSet)
     if s2.limit > s.limit
-        sizehint(s, s2.limit)
+        sizehint!(s, s2.limit)
     end
     lim = length(s2.bits)
     for n = 1:lim
@@ -275,7 +297,7 @@ function ==(s1::IntSet, s2::IntSet)
             return false
         end
     end
-    filln = s1.fill1s ? uint32(-1) : uint32(0)
+    filln = s1.fill1s ? reinterpret(UInt32, Int32(-1)) : UInt32(0)
     if lim1 > lim2
         for i = lim2:lim1
             if s1.bits[i] != filln
@@ -292,6 +314,19 @@ function ==(s1::IntSet, s2::IntSet)
     return true
 end
 
+const hashis_seed = UInt === UInt64 ? 0x88989f1fc7dea67d : 0xc7dea67d
+function hash(s::IntSet, h::UInt)
+    h += hashis_seed
+    h += hash(s.fill1s)
+    filln = s.fill1s ? ~zero(eltype(s.bits)) : zero(eltype(s.bits))
+    for x in s.bits
+        if x != filln
+            h = hash(x, h)
+        end
+    end
+    return h
+end
+
 issubset(a::IntSet, b::IntSet) = isequal(a, intersect(a,b))
 <(a::IntSet, b::IntSet) = (a<=b) && !isequal(a,b)
 <=(a::IntSet, b::IntSet) = issubset(a, b)
