@@ -1,13 +1,21 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # matmul.jl: Everything to do with dense matrix multiplication
 
 arithtype(T) = T
 arithtype(::Type{Bool}) = Int
 
 # multiply by diagonal matrix as vector
-function scale!(C::Matrix, A::Matrix, b::Vector)
+function scale!(C::AbstractMatrix, A::AbstractMatrix, b::AbstractVector)
     m, n = size(A)
-    n==length(b) || throw(DimensionMismatch(""))
-    for j = 1:n
+    p, q = size(C)
+    if size(A) != size(C)
+        throw(DimensionMismatch("size of A, $(size(A)), does not match size of C, $(size(C))"))
+    end
+    if n != length(b)
+        throw(DimensionMismatch("second dimension of A, $n, does not match length of b, $(length(b))"))
+    end
+    @inbounds for j = 1:n
         bj = b[j]
         for i = 1:m
             C[i,j] = A[i,j]*bj
@@ -16,51 +24,58 @@ function scale!(C::Matrix, A::Matrix, b::Vector)
     C
 end
 
-function scale!(C::Matrix, b::Vector, A::Matrix)
+function scale!(C::AbstractMatrix, b::AbstractVector, A::AbstractMatrix)
     m, n = size(A)
-    m==length(b) || throw(DimensionMismatch(""))
-    for j=1:n, i=1:m
+    p, q = size(C)
+    if size(A) != size(C)
+        throw(DimensionMismatch("size of A, $(size(A)), does not match size of C, $(size(C))"))
+    end
+    if m != length(b)
+        throw(DimensionMismatch("first dimension of A, $m, does not match length of b, $(length(b))"))
+    end
+    @inbounds for j = 1:n, i = 1:m
         C[i,j] = A[i,j]*b[i]
     end
     C
 end
-scale(A::Matrix, b::Vector) = scale!(similar(A, promote_type(eltype(A),eltype(b))), A, b)
-scale(b::Vector, A::Matrix) = scale!(similar(b, promote_type(eltype(A),eltype(b)), size(A)), b, A)
+
+scale(A::AbstractMatrix, b::AbstractVector) = scale!(similar(A, promote_type(eltype(A),eltype(b))), A, b)
+scale(b::AbstractVector, A::AbstractMatrix) = scale!(similar(b, promote_type(eltype(b),eltype(A)), size(A)), b, A)
 
 # Dot products
 
-dot{T<:BlasReal}(x::Vector{T}, y::Vector{T}) = BLAS.dot(x, y)
-dot{T<:BlasComplex}(x::Vector{T}, y::Vector{T}) = BLAS.dotc(x, y)
-function dot{T<:BlasReal, TI<:Integer}(x::Vector{T}, rx::Union(UnitRange{TI},Range{TI}), y::Vector{T}, ry::Union(UnitRange{TI},Range{TI}))
-    length(rx)==length(ry) || throw(DimensionMismatch(""))
-    if minimum(rx) < 1 || maximum(rx) > length(x) || minimum(ry) < 1 || maximum(ry) > length(y)
-        throw(BoundsError())
+vecdot{T<:BlasReal}(x::Union{DenseArray{T},StridedVector{T}}, y::Union{DenseArray{T},StridedVector{T}}) = BLAS.dot(x, y)
+vecdot{T<:BlasComplex}(x::Union{DenseArray{T},StridedVector{T}}, y::Union{DenseArray{T},StridedVector{T}}) = BLAS.dotc(x, y)
+
+function dot{T<:BlasReal, TI<:Integer}(x::Vector{T}, rx::Union{UnitRange{TI},Range{TI}}, y::Vector{T}, ry::Union{UnitRange{TI},Range{TI}})
+    if length(rx) != length(ry)
+        throw(DimensionMismatch("length of rx, $(length(rx)), does not equal length of ry, $(length(ry))"))
+    end
+    if minimum(rx) < 1 || maximum(rx) > length(x)
+        throw(BoundsError(x, rx))
+    end
+    if minimum(ry) < 1 || maximum(ry) > length(y)
+        throw(BoundsError(y, ry))
     end
     BLAS.dot(length(rx), pointer(x)+(first(rx)-1)*sizeof(T), step(rx), pointer(y)+(first(ry)-1)*sizeof(T), step(ry))
 end
-function dot{T<:BlasComplex, TI<:Integer}(x::Vector{T}, rx::Union(UnitRange{TI},Range{TI}), y::Vector{T}, ry::Union(UnitRange{TI},Range{TI}))
-    length(rx)==length(ry) || throw(DimensionMismatch(""))
-    if minimum(rx) < 1 || maximum(rx) > length(x) || minimum(ry) < 1 || maximum(ry) > length(y)
-        throw(BoundsError())
+
+function dot{T<:BlasComplex, TI<:Integer}(x::Vector{T}, rx::Union{UnitRange{TI},Range{TI}}, y::Vector{T}, ry::Union{UnitRange{TI},Range{TI}})
+    if length(rx) != length(ry)
+        throw(DimensionMismatch("length of rx, $(length(rx)), does not equal length of ry, $(length(ry))"))
+    end
+    if minimum(rx) < 1 || maximum(rx) > length(x)
+        throw(BoundsError(x, rx))
+    end
+    if minimum(ry) < 1 || maximum(ry) > length(y)
+        throw(BoundsError(y, ry))
     end
     BLAS.dotc(length(rx), pointer(x)+(first(rx)-1)*sizeof(T), step(rx), pointer(y)+(first(ry)-1)*sizeof(T), step(ry))
 end
-function dot(x::AbstractVector, y::AbstractVector)
-    lx = length(x)
-    lx==length(y) || throw(DimensionMismatch(""))
-    if lx == 0
-        return zero(eltype(x))*zero(eltype(y))
-    end
-    s = conj(x[1])*y[1]
-    @inbounds for i = 2:lx
-        s += conj(x[i])*y[i]
-    end
-    s
-end
-dot(x::Number, y::Number) = conj(x) * y
-Ac_mul_B(x::Vector, y::Vector) = [dot(x, y)]
-At_mul_B{T<:Real}(x::Vector{T}, y::Vector{T}) = [dot(x, y)]
-At_mul_B{T<:BlasComplex}(x::Vector{T}, y::Vector{T}) = [BLAS.dotu(x, y)]
+
+Ac_mul_B(x::AbstractVector, y::AbstractVector) = [dot(x, y)]
+At_mul_B{T<:Real}(x::AbstractVector{T}, y::AbstractVector{T}) = [dot(x, y)]
+At_mul_B{T<:BlasComplex}(x::StridedVector{T}, y::StridedVector{T}) = [BLAS.dotu(x, y)]
 
 # Matrix-vector multiplication
 function (*){T<:BlasFloat,S}(A::StridedMatrix{T}, x::StridedVector{S})
@@ -112,9 +127,9 @@ Ac_mul_B!(y::StridedVector, A::StridedVecOrMat, x::StridedVector) = generic_matv
 
 # Matrix-matrix multiplication
 
-function (*){T,S}(A::StridedMatrix{T}, B::StridedMatrix{S})
-    TS = promote_type(arithtype(T),arithtype(S))
-    A_mul_B!(similar(B,TS,(size(A,1),size(B,2))),A,B)
+function (*){T,S}(A::AbstractMatrix{T}, B::StridedMatrix{S})
+    TS = promote_type(arithtype(T), arithtype(S))
+    A_mul_B!(similar(B, TS, (size(A,1), size(B,2))), A, B)
 end
 A_mul_B!{T<:BlasFloat}(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) = gemm_wrapper!(C, 'N', 'N', A, B)
 for elty in (Float32,Float64)
@@ -129,16 +144,16 @@ for elty in (Float32,Float64)
 end
 A_mul_B!(C::StridedMatrix, A::StridedVecOrMat, B::StridedVecOrMat) = generic_matmatmul!(C, 'N', 'N', A, B)
 
-function At_mul_B{T,S}(A::StridedMatrix{T}, B::StridedMatrix{S})
-    TS = promote_type(arithtype(T),arithtype(S))
-    At_mul_B!(similar(B,TS,(size(A,2),size(B,2))),A,B)
+function At_mul_B{T,S}(A::AbstractMatrix{T}, B::StridedMatrix{S})
+    TS = promote_type(arithtype(T), arithtype(S))
+    At_mul_B!(similar(B, TS, (size(A,2), size(B,2))), A, B)
 end
 At_mul_B!{T<:BlasFloat}(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) = is(A,B) ? syrk_wrapper!(C, 'T', A) : gemm_wrapper!(C, 'T', 'N', A, B)
 At_mul_B!(C::StridedMatrix, A::StridedVecOrMat, B::StridedVecOrMat) = generic_matmatmul!(C, 'T', 'N', A, B)
 
-function A_mul_Bt{T,S}(A::StridedMatrix{T}, B::StridedMatrix{S})
-    TS = promote_type(arithtype(T),arithtype(S))
-    A_mul_Bt!(similar(B,TS,(size(A,1),size(B,1))),A,B)
+function A_mul_Bt{T,S}(A::AbstractMatrix{T}, B::StridedMatrix{S})
+    TS = promote_type(arithtype(T), arithtype(S))
+    A_mul_Bt!(similar(B, TS, (size(A,1), size(B,1))), A, B)
 end
 A_mul_Bt!{T<:BlasFloat}(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) = is(A,B) ? syrk_wrapper!(C, 'N', A) : gemm_wrapper!(C, 'N', 'T', A, B)
 for elty in (Float32,Float64)
@@ -163,8 +178,8 @@ At_mul_Bt!(C::StridedMatrix, A::StridedVecOrMat, B::StridedVecOrMat) = generic_m
 Ac_mul_B{T<:BlasReal}(A::StridedMatrix{T}, B::StridedMatrix{T}) = At_mul_B(A, B)
 Ac_mul_B!{T<:BlasReal}(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) = At_mul_B!(C, A, B)
 function Ac_mul_B{T,S}(A::StridedMatrix{T}, B::StridedMatrix{S})
-    TS = promote_type(arithtype(T),arithtype(S))
-    Ac_mul_B!(similar(B,TS,(size(A,2),size(B,2))),A,B)
+    TS = promote_type(arithtype(T), arithtype(S))
+    Ac_mul_B!(similar(B, TS, (size(A,2), size(B,2))), A, B)
 end
 Ac_mul_B!{T<:BlasComplex}(C::StridedMatrix{T}, A::StridedVecOrMat{T}, B::StridedVecOrMat{T}) = is(A,B) ? herk_wrapper!(C,'C',A) : gemm_wrapper!(C,'C', 'N', A, B)
 Ac_mul_B!(C::StridedMatrix, A::StridedVecOrMat, B::StridedVecOrMat) = generic_matmatmul!(C, 'C', 'N', A, B)
@@ -197,18 +212,24 @@ function copytri!(A::StridedMatrix, uplo::Char, conjugate::Bool=false)
         for i = 1:(n-1), j = (i+1):n
             A[i,j] = conjugate ? conj(A[j,i]) : A[j,i]
         end
-    else
-        throw(ArgumentError("second argument must be 'U' or 'L'"))
     end
     A
 end
 
 function gemv!{T<:BlasFloat}(y::StridedVector{T}, tA::Char, A::StridedVecOrMat{T}, x::StridedVector{T})
     mA, nA = lapack_size(tA, A)
-    nA == length(x) || throw(DimensionMismatch(""))
-    mA == length(y) || throw(DimensionMismatch(""))
-    mA == 0 && return y
-    nA == 0 && return fill!(y,0)
+    if nA != length(x)
+        throw(DimensionMismatch("second dimension of A, $nA, does not match length of x, $(length(x))"))
+    end
+    if mA != length(y)
+        throw(DimensionMismatch("first dimension of A, $mA, does not match length of y, $(length(y))"))
+    end
+    if mA == 0
+        return y
+    end
+    if nA == 0
+        return fill!(y,0)
+    end
     stride(A, 1) == 1 && stride(A, 2) >= size(A, 1) && return BLAS.gemv!(tA, one(T), A, x, zero(T), y)
     return generic_matvecmul!(y, tA, A, x)
 end
@@ -222,16 +243,26 @@ function syrk_wrapper!{T<:BlasFloat}(C::StridedMatrix{T}, tA::Char, A::StridedVe
         (mA, nA) = size(A,1), size(A,2)
         tAt = 'T'
     end
-    nC == mA || throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
-    if mA == 0 || nA == 0; return fill!(C,0); end
-    if mA == 2 && nA == 2; return matmul2x2!(C,tA,tAt,A,A); end
-    if mA == 3 && nA == 3; return matmul3x3!(C,tA,tAt,A,A); end
+    if nC != mA
+        throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
+    end
+    if mA == 0 || nA == 0
+        return fill!(C,0)
+    end
+    if mA == 2 && nA == 2
+        return matmul2x2!(C,tA,tAt,A,A)
+    end
+    if mA == 3 && nA == 3
+        return matmul3x3!(C,tA,tAt,A,A)
+    end
 
-    stride(A, 1) == 1 && stride(A, 2) >= size(A, 1) && return copytri!(BLAS.syrk!('U', tA, one(T), A, zero(T), C), 'U')
+    if stride(A, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(C, 2) >= size(C, 1)
+        return copytri!(BLAS.syrk!('U', tA, one(T), A, zero(T), C), 'U')
+    end
     return generic_matmatmul!(C, tA, tAt, A, A)
 end
 
-function herk_wrapper!{T<:BlasFloat}(C::StridedMatrix{T}, tA::Char, A::StridedVecOrMat{T})
+function herk_wrapper!{T<:BlasReal}(C::Union{StridedMatrix{T}, StridedMatrix{Complex{T}}}, tA::Char, A::Union{StridedVecOrMat{T}, StridedVecOrMat{Complex{T}}})
     nC = chksquare(C)
     if tA == 'C'
         (nA, mA) = size(A,1), size(A,2)
@@ -240,15 +271,25 @@ function herk_wrapper!{T<:BlasFloat}(C::StridedMatrix{T}, tA::Char, A::StridedVe
         (mA, nA) = size(A,1), size(A,2)
         tAt = 'C'
     end
-    nC == mA || throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
-    if mA == 0 || nA == 0; return fill!(C,0); end
-    if mA == 2 && nA == 2; return matmul2x2!(C,tA,tAt,A,A); end
-    if mA == 3 && nA == 3; return matmul3x3!(C,tA,tAt,A,A); end
+    if nC != mA
+        throw(DimensionMismatch("output matrix has size: $(nC), but should have size $(mA)"))
+    end
+    if mA == 0 || nA == 0
+        return fill!(C,0)
+    end
+    if mA == 2 && nA == 2
+        return matmul2x2!(C,tA,tAt,A,A)
+    end
+    if mA == 3 && nA == 3
+        return matmul3x3!(C,tA,tAt,A,A)
+    end
 
     # Result array does not need to be initialized as long as beta==0
     #    C = Array(T, mA, mA)
 
-    stride(A, 1) == 1 && stride(A, 2) >= size(A, 1) && return copytri!(BLAS.herk!('U', tA, one(T), A, zero(T), C), 'U', true)
+    if stride(A, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(C, 2) >= size(C, 1)
+        return copytri!(BLAS.herk!('U', tA, one(T), A, zero(T), C), 'U', true)
+    end
     return generic_matmatmul!(C,tA, tAt, A, A)
 end
 
@@ -267,16 +308,26 @@ function gemm_wrapper!{T<:BlasFloat}(C::StridedVecOrMat{T}, tA::Char, tB::Char,
     mA, nA = lapack_size(tA, A)
     mB, nB = lapack_size(tB, B)
 
-    nA==mB || throw(DimensionMismatch("*"))
+    if nA != mB
+        throw(DimensionMismatch("A has dimensions ($mA,$nA) but B has dimensions ($mB,$nB)"))
+    end
 
     if mA == 0 || nA == 0 || nB == 0
-        size(C) == (mA, nB) || throw(DimensionMismatch(""))
+        if size(C) != (mA, nB)
+            throw(DimensionMismatch("C has dimensions $(size(C)), should have ($mA,$nB)"))
+        end
         return fill!(C,0)
     end
-    if mA == 2 && nA == 2 && nB == 2; return matmul2x2!(C,tA,tB,A,B); end
-    if mA == 3 && nA == 3 && nB == 3; return matmul3x3!(C,tA,tB,A,B); end
+    if mA == 2 && nA == 2 && nB == 2
+        return matmul2x2!(C,tA,tB,A,B)
+    end
+    if mA == 3 && nA == 3 && nB == 3
+        return matmul3x3!(C,tA,tB,A,B)
+    end
 
-    stride(A, 1) == stride(B, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(B, 2) >= size(B, 1) && return BLAS.gemm!(tA, tB, one(T), A, B, zero(T), C)
+    if stride(A, 1) == stride(B, 1) == stride(C, 1) == 1 && stride(A, 2) >= size(A, 1) && stride(B, 2) >= size(B, 1) && stride(C, 2) >= size(C, 1)
+        return BLAS.gemm!(tA, tB, one(T), A, B, zero(T), C)
+    end
     generic_matmatmul!(C, tA, tB, A, B)
 end
 
@@ -292,6 +343,7 @@ function copy!{R,S}(B::AbstractVecOrMat{R}, ir_dest::UnitRange{Int}, jr_dest::Un
         Base.copy_transpose!(B, ir_dest, jr_dest, M, jr_src, ir_src)
         tM == 'C' && conj!(B)
     end
+    B
 end
 
 function copy_transpose!{R,S}(B::AbstractMatrix{R}, ir_dest::UnitRange{Int}, jr_dest::UnitRange{Int}, tM::Char, M::AbstractVecOrMat{S}, ir_src::UnitRange{Int}, jr_src::UnitRange{Int})
@@ -301,6 +353,7 @@ function copy_transpose!{R,S}(B::AbstractMatrix{R}, ir_dest::UnitRange{Int}, jr_
         copy!(B, ir_dest, jr_dest, M, jr_src, ir_src)
         tM == 'C' && conj!(B)
     end
+    B
 end
 
 # TODO: It will be faster for large matrices to convert to float,
@@ -312,16 +365,23 @@ end
 function generic_matvecmul!{T,S,R}(C::AbstractVector{R}, tA, A::AbstractVecOrMat{T}, B::AbstractVector{S})
     mB = length(B)
     mA, nA = lapack_size(tA, A)
-    mB==nA || throw(DimensionMismatch("*"))
-    mA==length(C) || throw(DimensionMismatch("*"))
-    z = zero(R)
+    if mB != nA
+        throw(DimensionMismatch("matrix A has dimensions ($mA,$nA), vector B has length $mB"))
+    end
+    if mA != length(C)
+        throw(DimensionMismatch("result C has length $(length(C)), needs length $mA"))
+    end
 
     Astride = size(A, 1)
 
     if tA == 'T'  # fastest case
         for k = 1:mA
             aoffs = (k-1)*Astride
-            s = z
+            if mB == 0
+                s = zero(R)
+            else
+                s = zero(A[aoffs + 1]*B[1] + A[aoffs + 1]*B[1])
+            end
             for i = 1:nA
                 s += A[aoffs+i].'B[i]
             end
@@ -330,19 +390,29 @@ function generic_matvecmul!{T,S,R}(C::AbstractVector{R}, tA, A::AbstractVecOrMat
     elseif tA == 'C'
         for k = 1:mA
             aoffs = (k-1)*Astride
-            s = z
+            if mB == 0
+                s = zero(R)
+            else
+                s = zero(A[aoffs + 1]*B[1] + A[aoffs + 1]*B[1])
+            end
             for i = 1:nA
-                s += A[aoffs+i]'B[i]
+                s += A[aoffs + i]'B[i]
             end
             C[k] = s
         end
     else # tA == 'N'
-        fill!(C, z)
+        for i = 1:mA
+            if mB == 0
+                C[i] = zero(R)
+            else
+                C[i] = zero(A[i]*B[1] + A[i]*B[1])
+            end
+        end
         for k = 1:mB
             aoffs = (k-1)*Astride
             b = B[k]
             for i = 1:mA
-                C[i] += A[aoffs+i] * b
+                C[i] += A[aoffs + i] * b
             end
         end
     end
@@ -357,25 +427,45 @@ function generic_matmatmul{T,S}(tA, tB, A::AbstractVecOrMat{T}, B::AbstractMatri
 end
 
 const tilebufsize = 10800  # Approximately 32k/3
-const Abuf = Array(Uint8, tilebufsize)
-const Bbuf = Array(Uint8, tilebufsize)
-const Cbuf = Array(Uint8, tilebufsize)
+const Abuf = Array(UInt8, tilebufsize)
+const Bbuf = Array(UInt8, tilebufsize)
+const Cbuf = Array(UInt8, tilebufsize)
 
-function generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat{T}, B::AbstractVecOrMat{S})
+function generic_matmatmul!{T,S,R}(C::AbstractMatrix{R}, tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S})
     mA, nA = lapack_size(tA, A)
     mB, nB = lapack_size(tB, B)
-    mB==nA || throw(DimensionMismatch("*"))
-    if size(C,1) != mA || size(C,2) != nB; throw(DimensionMismatch("*")); end
+    mC, nC = size(C)
 
-    if mA == nA == nB == 2; return matmul2x2!(C, tA, tB, A, B); end
-    if mA == nA == nB == 3; return matmul3x3!(C, tA, tB, A, B); end
+    if mA == nA == mB == nB == mC == nC == 2
+        return matmul2x2!(C, tA, tB, A, B)
+    end
+    if mA == nA == mB == nB == mC == nC == 3
+        return matmul3x3!(C, tA, tB, A, B)
+    end
+    _generic_matmatmul!(C, tA, tB, A, B)
+end
 
+generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat{T}, B::AbstractVecOrMat{S}) = _generic_matmatmul!(C, tA, tB, A, B)
+
+function _generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVecOrMat{T}, B::AbstractVecOrMat{S})
+    mA, nA = lapack_size(tA, A)
+    mB, nB = lapack_size(tB, B)
+    if mB != nA
+        throw(DimensionMismatch("matrix A has dimensions ($mA,$nA), matrix B has dimensions ($mB,$nB)"))
+    end
+    if size(C,1) != mA || size(C,2) != nB
+        throw(DimensionMismatch("result C has dimensions $(size(C)), needs ($mA,$nB)"))
+    end
+
+    tile_size = 0
+    if isbits(R) && isbits(T) && isbits(S) && (tA == 'N' || tB != 'N')
+        tile_size = floor(Int,sqrt(tilebufsize/max(sizeof(R),sizeof(S),sizeof(T))))
+    end
     @inbounds begin
-    if isbits(R)
-        tile_size = int(ifloor(sqrt(tilebufsize/sizeof(R))))
+    if tile_size > 0
         sz = (tile_size, tile_size)
-        Atile = pointer_to_array(convert(Ptr{R}, pointer(Abuf)), sz)
-        Btile = pointer_to_array(convert(Ptr{R}, pointer(Bbuf)), sz)
+        Atile = pointer_to_array(convert(Ptr{T}, pointer(Abuf)), sz)
+        Btile = pointer_to_array(convert(Ptr{S}, pointer(Bbuf)), sz)
 
         z = zero(R)
 
@@ -425,27 +515,40 @@ function generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVe
         end
     else
         # Multiplication for non-plain-data uses the naive algorithm
+
         if tA == 'N'
             if tB == 'N'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[i, 1]*B[1, j]
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[i, 1]*B[1, j] + A[i, 1]*B[1, j])
+                    end
+                    for k = 1:nA
                         Ctmp += A[i, k]*B[k, j]
                     end
                     C[i,j] = Ctmp
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[i, 1]*B[j, 1].'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[i, 1]*B[j, 1] + A[i, 1]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[i, k]*B[j, k].'
                     end
                     C[i,j] = Ctmp
                 end
             else
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[i, 1]*B[j, 1]'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[i, 1]*B[j, 1] + A[i, 1]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[i, k]*B[j, k]'
                     end
                     C[i,j] = Ctmp
@@ -454,24 +557,36 @@ function generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVe
         elseif tA == 'T'
             if tB == 'N'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i].'B[1, j]
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[1, j] + A[1, i]*B[1, j])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i].'B[k, j]
                     end
                     C[i,j] = Ctmp
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i].'B[j, 1].'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[j, 1] + A[1, i]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i].'B[j, k].'
                     end
                     C[i,j] = Ctmp
                 end
             else
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i].'B[j, 1]'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[j, 1] + A[1, i]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i].'B[j, k]'
                     end
                     C[i,j] = Ctmp
@@ -480,24 +595,36 @@ function generic_matmatmul!{T,S,R}(C::AbstractVecOrMat{R}, tA, tB, A::AbstractVe
         else
             if tB == 'N'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i]'B[1, j]
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[1, j] + A[1, i]*B[1, j])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i]'B[k, j]
                     end
                     C[i,j] = Ctmp
                 end
             elseif tB == 'T'
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i]'B[j, 1].'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[j, 1] + A[1, i]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i]'B[j, k].'
                     end
                     C[i,j] = Ctmp
                 end
             else
                 for i = 1:mA, j = 1:nB
-                    Ctmp = A[1, i]'B[j, 1]'
-                    for k = 2:nA
+                    if length(A) == 0 || length(B) == 0
+                        Ctmp = zero(R)
+                    else
+                        Ctmp = zero(A[1, i]*B[j, 1] + A[1, i]*B[j, 1])
+                    end
+                    for k = 1:nA
                         Ctmp += A[k, i]'B[j, k]'
                     end
                     C[i,j] = Ctmp
@@ -516,6 +643,7 @@ function matmul2x2{T,S}(tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S})
 end
 
 function matmul2x2!{T,S,R}(C::AbstractMatrix{R}, tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S})
+    @inbounds begin
     if tA == 'T'
         A11 = transpose(A[1,1]); A12 = transpose(A[2,1]); A21 = transpose(A[1,2]); A22 = transpose(A[2,2])
     elseif tA == 'C'
@@ -534,6 +662,7 @@ function matmul2x2!{T,S,R}(C::AbstractMatrix{R}, tA, tB, A::AbstractMatrix{T}, B
     C[1,2] = A11*B12 + A12*B22
     C[2,1] = A21*B11 + A22*B21
     C[2,2] = A21*B12 + A22*B22
+    end # inbounds
     C
 end
 
@@ -543,6 +672,7 @@ function matmul3x3{T,S}(tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S})
 end
 
 function matmul3x3!{T,S,R}(C::AbstractMatrix{R}, tA, tB, A::AbstractMatrix{T}, B::AbstractMatrix{S})
+    @inbounds begin
     if tA == 'T'
         A11 = transpose(A[1,1]); A12 = transpose(A[2,1]); A13 = transpose(A[3,1]);
         A21 = transpose(A[1,2]); A22 = transpose(A[2,2]); A23 = transpose(A[3,2]);
@@ -582,6 +712,6 @@ function matmul3x3!{T,S,R}(C::AbstractMatrix{R}, tA, tB, A::AbstractMatrix{T}, B
     C[3,1] = A31*B11 + A32*B21 + A33*B31
     C[3,2] = A31*B12 + A32*B22 + A33*B32
     C[3,3] = A31*B13 + A32*B23 + A33*B33
-
+    end # inbounds
     C
 end
