@@ -1,71 +1,103 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 #Methods operating on different special matrix types
 
 #Interconversion between special matrix types
 convert{T}(::Type{Bidiagonal}, A::Diagonal{T})=Bidiagonal(A.diag, zeros(T, size(A.diag,1)-1), true)
 convert{T}(::Type{SymTridiagonal}, A::Diagonal{T})=SymTridiagonal(A.diag, zeros(T, size(A.diag,1)-1))
 convert{T}(::Type{Tridiagonal}, A::Diagonal{T})=Tridiagonal(zeros(T, size(A.diag,1)-1), A.diag, zeros(T, size(A.diag,1)-1))
-convert(::Type{Triangular}, A::Diagonal) = Triangular(full(A), :L)
-convert(::Type{Triangular}, A::Bidiagonal) = Triangular(full(A), A.isupper ? :U : :L)
+convert(::Type{LowerTriangular}, A::Bidiagonal) = !A.isupper ? LowerTriangular(full(A)) : throw(ArgumentError("Bidiagonal matrix must have lower off diagonal to be converted to LowerTriangular"))
+convert(::Type{UpperTriangular}, A::Bidiagonal) = A.isupper ? UpperTriangular(full(A)) : throw(ArgumentError("Bidiagonal matrix must have upper off diagonal to be converted to UpperTriangular"))
 convert(::Type{Matrix}, D::Diagonal) = diagm(D.diag)
 
-function convert(::Type{Diagonal}, A::Union(Bidiagonal, SymTridiagonal))
-    all(A.ev .== 0) || throw(ArgumentError("Matrix cannot be represented as Diagonal"))
+function convert(::Type{UnitUpperTriangular}, A::Diagonal)
+    if !all(A.diag .== one(eltype(A)))
+        throw(ArgumentError("matrix cannot be represented as UnitUpperTriangular"))
+    end
+    UnitUpperTriangular(full(A))
+end
+
+function convert(::Type{UnitLowerTriangular}, A::Diagonal)
+    if !all(A.diag .== one(eltype(A)))
+        throw(ArgumentError("matrix cannot be represented as UnitLowerTriangular"))
+    end
+    UnitLowerTriangular(full(A))
+end
+
+function convert(::Type{Diagonal}, A::Union{Bidiagonal, SymTridiagonal})
+    if !all(A.ev .== 0)
+        throw(ArgumentError("matrix cannot be represented as Diagonal"))
+    end
     Diagonal(A.dv)
 end
 
 function convert(::Type{SymTridiagonal}, A::Bidiagonal)
-    all(A.ev .== 0) || throw(ArgumentError("Matrix cannot be represented as SymTridiagonal"))
+    if !all(A.ev .== 0)
+        throw(ArgumentError("matrix cannot be represented as SymTridiagonal"))
+    end
     SymTridiagonal(A.dv, A.ev)
 end
 
 convert{T}(::Type{Tridiagonal}, A::Bidiagonal{T})=Tridiagonal(A.isupper?zeros(T, size(A.dv,1)-1):A.ev, A.dv, A.isupper?A.ev:zeros(T, size(A.dv,1)-1))
 
 function convert(::Type{Bidiagonal}, A::SymTridiagonal)
-    all(A.ev .== 0) || throw(ArgumentError("Matrix cannot be represented as Bidiagonal"))
+    if !all(A.ev .== 0)
+        throw(ArgumentError("matrix cannot be represented as Bidiagonal"))
+    end
     Bidiagonal(A.dv, A.ev, true)
 end
 
 function convert(::Type{Diagonal}, A::Tridiagonal)
-    all(A.dl .== 0) && all(A.du .== 0) || throw(ArgumentError("Matrix cannot be represented as Diagonal"))
+    if !(all(A.dl .== 0) && all(A.du .== 0))
+        throw(ArgumentError("matrix cannot be represented as Diagonal"))
+    end
     Diagonal(A.d)
 end
 
 function convert(::Type{Bidiagonal}, A::Tridiagonal)
     if all(A.dl .== 0) return Bidiagonal(A.d, A.du, true)
     elseif all(A.du .== 0) return Bidiagonal(A.d, A.dl, false)
-    else throw(ArgumentError("Matrix cannot be represented as Bidiagonal"))
+    else throw(ArgumentError("matrix cannot be represented as Bidiagonal"))
     end
 end
 
 function convert(::Type{SymTridiagonal}, A::Tridiagonal)
-    all(A.dl .== A.du) || throw(ArgumentError("Matrix cannot be represented as SymTridiagonal"))
+    if !all(A.dl .== A.du)
+        throw(ArgumentError("matrix cannot be represented as SymTridiagonal"))
+    end
     SymTridiagonal(A.d, A.dl)
 end
 
-function convert(::Type{Diagonal}, A::Triangular)
-    full(A) == diagm(diag(A)) || throw(ArgumentError("Matrix cannot be represented as Diagonal"))
+function convert(::Type{Tridiagonal}, A::SymTridiagonal)
+    Tridiagonal(copy(A.ev), A.dv, A.ev)
+end
+
+function convert(::Type{Diagonal}, A::AbstractTriangular)
+    if full(A) != diagm(diag(A))
+        throw(ArgumentError("matrix cannot be represented as Diagonal"))
+    end
     Diagonal(diag(A))
 end
 
-function convert(::Type{Bidiagonal}, A::Triangular)
+function convert(::Type{Bidiagonal}, A::AbstractTriangular)
     fA = full(A)
     if fA == diagm(diag(A)) + diagm(diag(fA, 1), 1)
         return Bidiagonal(diag(A), diag(fA,1), true)
     elseif fA == diagm(diag(A)) + diagm(diag(fA, -1), -1)
         return Bidiagonal(diag(A), diag(fA,-1), false)
     else
-        throw(ArgumentError("Matrix cannot be represented as Bidiagonal"))
+        throw(ArgumentError("matrix cannot be represented as Bidiagonal"))
     end
 end
 
-convert(::Type{SymTridiagonal}, A::Triangular) = convert(SymTridiagonal, convert(Tridiagonal, A))
+convert(::Type{SymTridiagonal}, A::AbstractTriangular) = convert(SymTridiagonal, convert(Tridiagonal, A))
 
-function convert(::Type{Tridiagonal}, A::Triangular)
+function convert(::Type{Tridiagonal}, A::AbstractTriangular)
     fA = full(A)
     if fA == diagm(diag(A)) + diagm(diag(fA, 1), 1) + diagm(diag(fA, -1), -1)
         return Tridiagonal(diag(fA, -1), diag(A), diag(fA,1))
     else
-        throw(ArgumentError("Matrix cannot be represented as Tridiagonal"))
+        throw(ArgumentError("matrix cannot be represented as Tridiagonal"))
     end
 end
 
@@ -82,7 +114,7 @@ macro commutative(myexpr)
 end
 
 for op in (:+, :-)
-    SpecialMatrices = [:Diagonal, :Bidiagonal, :Tridiagonal, :Triangular, :Matrix]
+    SpecialMatrices = [:Diagonal, :Bidiagonal, :Tridiagonal, :Matrix]
     for (idx, matrixtype1) in enumerate(SpecialMatrices) #matrixtype1 is the sparser matrix type
         for matrixtype2 in SpecialMatrices[idx+1:end] #matrixtype2 is the denser matrix type
             @eval begin #TODO quite a few of these conversions are NOT defined...
@@ -93,7 +125,7 @@ for op in (:+, :-)
     end
 
     for  matrixtype1 in (:SymTridiagonal,)                      #matrixtype1 is the sparser matrix type
-        for matrixtype2 in (:Tridiagonal, :Triangular, :Matrix) #matrixtype2 is the denser matrix type
+        for matrixtype2 in (:Tridiagonal, :Matrix) #matrixtype2 is the denser matrix type
             @eval begin
                 ($op)(A::($matrixtype1), B::($matrixtype2)) = ($op)(convert(($matrixtype2), A), B)
                 ($op)(A::($matrixtype2), B::($matrixtype1)) = ($op)(A, convert(($matrixtype2), B))
@@ -109,9 +141,26 @@ for op in (:+, :-)
             end
         end
     end
+
+    for matrixtype1 in (:Diagonal,)
+        for (matrixtype2,matrixtype3) in ((:UpperTriangular,:UpperTriangular),
+                                          (:UnitUpperTriangular,:UpperTriangular),
+                                          (:LowerTriangular,:LowerTriangular),
+                                          (:UnitLowerTriangular,:LowerTriangular))
+            @eval begin
+                ($op)(A::($matrixtype1), B::($matrixtype2)) = ($op)(convert(($matrixtype3), A), B)
+                ($op)(A::($matrixtype2), B::($matrixtype1)) = ($op)(A, convert(($matrixtype3), B))
+            end
+        end
+    end
+    for matrixtype in (:SymTridiagonal,:Tridiagonal,:Bidiagonal,:Matrix)
+        @eval begin
+            ($op)(A::AbstractTriangular, B::($matrixtype)) = ($op)(full(A), B)
+            ($op)(A::($matrixtype), B::AbstractTriangular) = ($op)(A, full(B))
+        end
+    end
 end
 
-A_mul_Bc!(A::Triangular, B::QRCompactWYQ) = A_mul_Bc!(full!(A),B)
-A_mul_Bc!(A::Triangular, B::QRPackedQ) = A_mul_Bc!(full!(A),B)
-A_mul_Bc(A::Triangular, B::Union(QRCompactWYQ,QRPackedQ)) = A_mul_Bc(full(A), B)
-
+A_mul_Bc!(A::AbstractTriangular, B::QRCompactWYQ) = A_mul_Bc!(full!(A),B)
+A_mul_Bc!(A::AbstractTriangular, B::QRPackedQ) = A_mul_Bc!(full!(A),B)
+A_mul_Bc(A::AbstractTriangular, B::Union{QRCompactWYQ,QRPackedQ}) = A_mul_Bc(full(A), B)
