@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # Method and method-table pretty-printing
 
 function argtype_decl(n, t) # -> (argname, argtype)
@@ -12,22 +14,28 @@ function argtype_decl(n, t) # -> (argname, argtype)
     if t === Any && !isempty(s)
         return s, ""
     end
-    if t <: Vararg && t !== None && t.parameters[1] === Any
-        return string(s, "..."), ""
+    if isvarargtype(t)
+        if t.parameters[1] === Any
+            return string(s, "..."), ""
+        else
+            return s, string(t.parameters[1], "...")
+        end
+    elseif t == ByteString
+        return s, "ByteString"
     end
     return s, string(t)
 end
 
 function arg_decl_parts(m::Method)
     tv = m.tvars
-    if !isa(tv,Tuple)
-        tv = (tv,)
+    if !isa(tv,SimpleVector)
+        tv = svec(tv)
     end
     li = m.func.code
     e = uncompressed_ast(li)
     argnames = e.args[1]
     s = symbol("?")
-    decls = [argtype_decl(get(argnames,i,s), m.sig[i]) for i=1:length(m.sig)]
+    decls = [argtype_decl(get(argnames,i,s), m.sig.parameters[i]) for i=1:length(m.sig.parameters)]
     return tv, decls, li.file, li.line
 end
 
@@ -39,7 +47,7 @@ function show(io::IO, m::Method)
     end
     print(io, "(")
     print_joined(io, [isempty(d[2]) ? d[1] : d[1]*"::"*d[2] for d in decls],
-                 ",", ",")
+                 ", ", ", ")
     print(io, ")")
     if line > 0
         print(io, " at ", file, ":", line)
@@ -55,8 +63,8 @@ function show_method_table(io::IO, mt::MethodTable, max::Int=-1, header::Bool=tr
     end
     d = mt.defs
     n = rest = 0
-    while !is(d,())
-        if max==-1 || n<max || (rest==0 && n==max && d.next === ())
+    while d !== nothing
+        if max==-1 || n<max || (rest==0 && n==max && d.next === nothing)
             println(io)
             show(io, d)
             n += 1
@@ -73,8 +81,15 @@ end
 
 show(io::IO, mt::MethodTable) = show_method_table(io, mt)
 
-inbase(m::Module) = m == Base ? true : m == Main ? false : inbase(module_parent(m))
-fileurl(file) = let f = find_source_file(file); f == nothing ? "" : "file://"*f; end
+function inbase(m::Module)
+    if m == Base
+        true
+    else
+        parent = module_parent(m)
+        parent === m ? false : inbase(parent)
+    end
+end
+fileurl(file) = let f = find_source_file(file); f === nothing ? "" : "file://"*f; end
 function url(m::Method)
     M = m.func.code.module
     (m.func.code.file == :null || m.func.code.file == :string) && return ""
@@ -82,7 +97,12 @@ function url(m::Method)
     line = m.func.code.line
     line <= 0 || ismatch(r"In\[[0-9]+\]", file) && return ""
     if inbase(M)
-        return "https://github.com/JuliaLang/julia/tree/$(Base.GIT_VERSION_INFO.commit)/base/$file#L$line"
+        if isempty(Base.GIT_VERSION_INFO.commit)
+            # this url will only work if we're on a tagged release
+            return "https://github.com/JuliaLang/julia/tree/v$VERSION/base/$file#L$line"
+        else
+            return "https://github.com/JuliaLang/julia/tree/$(Base.GIT_VERSION_INFO.commit)/base/$file#L$line"
+        end
     else
         try
             d = dirname(file)
@@ -91,7 +111,7 @@ function url(m::Method)
             root = cd(d) do # dir=d confuses --show-toplevel, apparently
                 Git.readchomp(`rev-parse --show-toplevel`)
             end
-            if beginswith(file, root)
+            if startswith(file, root)
                 commit = Git.readchomp(`rev-parse HEAD`, dir=d)
                 return "https://github.com/$u/tree/$commit/"*file[length(root)+2:end]*"#L$line"
             else
@@ -113,7 +133,7 @@ function writemime(io::IO, ::MIME"text/html", m::Method)
     end
     print(io, "(")
     print_joined(io, [isempty(d[2]) ? d[1] : d[1]*"::<b>"*d[2]*"</b>"
-                      for d in decls], ",", ",")
+                      for d in decls], ", ", ", ")
     print(io, ")")
     if line > 0
         u = url(m)
@@ -132,7 +152,7 @@ function writemime(io::IO, mime::MIME"text/html", mt::MethodTable)
     meths = n==1 ? "method" : "methods"
     print(io, "$n $meths for generic function <b>$name</b>:<ul>")
     d = mt.defs
-    while !is(d,())
+    while d !== nothing
         print(io, "<li> ")
         writemime(io, mime, d)
         d = d.next
