@@ -1,35 +1,51 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module MPFR
 
 export
     BigFloat,
     get_bigfloat_precision,
     set_bigfloat_precision,
-    with_bigfloat_precision
+    with_bigfloat_precision,
+    bigfloat_str,
+    big_str
 
 import
     Base: (*), +, -, /, <, <=, ==, >, >=, ^, besselj, besselj0, besselj1, bessely,
-        bessely0, bessely1, ceil, cmp, convert, copysign, deg2rad,
-        exp, exp2, exponent, factorial, floor, hypot, isinteger, iround,
+        bessely0, bessely1, ceil, cmp, convert, copysign, div,
+        exp, exp2, exponent, factorial, floor, fma, hypot, isinteger,
         isfinite, isinf, isnan, ldexp, log, log2, log10, max, min, mod, modf,
-        nextfloat, prevfloat, promote_rule, rad2deg, rem, round, show,
+        nextfloat, prevfloat, promote_rule, rem, round, show,
         showcompact, sum, sqrt, string, print, trunc, precision, exp10, expm1,
-        gamma, lgamma, digamma, erf, erfc, zeta, eta, log1p, airyai, iceil, ifloor,
-        itrunc, eps, signbit, sin, cos, tan, sec, csc, cot, acos, asin, atan,
+        gamma, lgamma, digamma, erf, erfc, zeta, eta, log1p, airyai,
+        eps, signbit, sin, cos, tan, sec, csc, cot, acos, asin, atan,
         cosh, sinh, tanh, sech, csch, coth, acosh, asinh, atanh, atan2,
-        serialize, deserialize, inf, nan, cbrt, typemax, typemin,
-        realmin, realmax, get_rounding, set_rounding, maxintfloat, widen,
-        significand, frexp
+        cbrt, typemax, typemin, unsafe_trunc, realmin, realmax, get_rounding,
+        set_rounding, maxintfloat, widen, significand, frexp, tryparse
+
+import Base.Rounding: get_rounding_raw, set_rounding_raw
 
 import Base.GMP: ClongMax, CulongMax, CdoubleMax
 
 import Base.Math.lgamma_r
 
-const ROUNDING_MODE = [0]
+function __init__()
+    try
+        # set exponent to full range by default
+        set_emin!(get_emin_min())
+        set_emax!(get_emax_max())
+    catch ex
+        Base.showerror_nostdio(ex,
+            "WARNING: Error during initialization of module MPFR")
+    end
+end
+
+const ROUNDING_MODE = Cint[0]
 const DEFAULT_PRECISION = [256]
 
 # Basic type and initialization definitions
 
-type BigFloat <: FloatingPoint
+type BigFloat <: AbstractFloat
     prec::Clong
     sign::Cint
     exp::Clong
@@ -50,11 +66,12 @@ end
 widen(::Type{Float64}) = BigFloat
 widen(::Type{BigFloat}) = BigFloat
 
-BigFloat(x::BigFloat) = x
+convert(::Type{BigFloat}, x::BigFloat) = x
 
+# convert to BigFloat
 for (fJ, fC) in ((:si,:Clong), (:ui,:Culong), (:d,:Float64))
     @eval begin
-        function BigFloat(x::($fC))
+        function convert(::Type{BigFloat}, x::($fC))
             z = BigFloat()
             ccall(($(string(:mpfr_set_,fJ)), :libmpfr), Int32, (Ptr{BigFloat}, ($fC), Int32), &z, x, ROUNDING_MODE[end])
             return z
@@ -62,69 +79,120 @@ for (fJ, fC) in ((:si,:Clong), (:ui,:Culong), (:d,:Float64))
     end
 end
 
-function BigFloat(x::BigInt)
+function convert(::Type{BigFloat}, x::BigInt)
     z = BigFloat()
     ccall((:mpfr_set_z, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigInt}, Int32), &z, &x, ROUNDING_MODE[end])
     return z
 end
 
-function BigFloat(x::String, base::Int)
+convert(::Type{BigFloat}, x::Integer) = BigFloat(BigInt(x))
+
+convert(::Type{BigFloat}, x::Union{Bool,Int8,Int16,Int32}) = BigFloat(convert(Clong,x))
+convert(::Type{BigFloat}, x::Union{UInt8,UInt16,UInt32}) = BigFloat(convert(Culong,x))
+
+convert(::Type{BigFloat}, x::Union{Float16,Float32}) = BigFloat(Float64(x))
+convert(::Type{BigFloat}, x::Rational) = BigFloat(num(x)) / BigFloat(den(x))
+
+function tryparse(::Type{BigFloat}, s::AbstractString, base::Int=0)
     z = BigFloat()
-    err = ccall((:mpfr_set_str, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{Uint8}, Int32, Int32), &z, x, base, ROUNDING_MODE[end])
-    if err != 0; error("incorrectly formatted number"); end
-    return z
+    err = ccall((:mpfr_set_str, :libmpfr), Int32, (Ptr{BigFloat}, Cstring, Int32, Int32), &z, s, base, ROUNDING_MODE[end])
+    err == 0 ? Nullable(z) : Nullable{BigFloat}()
 end
-BigFloat(x::String) = BigFloat(x, 10)
-
-BigFloat(x::Integer) = BigFloat(BigInt(x))
-
-BigFloat(x::Union(Bool,Int8,Int16,Int32)) = BigFloat(convert(Clong,x))
-BigFloat(x::Union(Uint8,Uint16,Uint32)) = BigFloat(convert(Culong,x))
-
-BigFloat(x::Union(Float16,Float32)) = BigFloat(float64(x))
-BigFloat(x::Rational) = BigFloat(num(x)) / BigFloat(den(x))
 
 convert(::Type{Rational}, x::BigFloat) = convert(Rational{BigInt}, x)
-convert{S}(::Type{BigFloat}, x::Rational{S}) = BigFloat(x) # to resolve ambiguity
-convert(::Type{BigFloat}, x::Real) = BigFloat(x)
-convert(::Type{FloatingPoint}, x::BigInt) = BigFloat(x)
+convert(::Type{AbstractFloat}, x::BigInt) = BigFloat(x)
 
-for to in (Int8, Int16, Int32, Int64)
-    @eval begin
-        function convert(::Type{$to}, x::BigFloat)
-            (isinteger(x) && (typemin($to) <= x <= typemax($to))) || throw(InexactError())
-            convert($to, ccall((:mpfr_get_si,:libmpfr),
-                               Clong, (Ptr{BigFloat}, Int32), &x, 0))
-        end
-    end
+## BigFloat -> Integer
+function unsafe_cast(::Type{Int64}, x::BigFloat, ri::Cint)
+    ccall((:__gmpfr_mpfr_get_sj,:libmpfr), Cintmax_t,
+          (Ptr{BigFloat}, Cint), &x, ri)
+end
+function unsafe_cast(::Type{UInt64}, x::BigFloat, ri::Cint)
+    ccall((:__gmpfr_mpfr_get_uj,:libmpfr), Cuintmax_t,
+          (Ptr{BigFloat}, Cint), &x, ri)
 end
 
-for to in (Uint8, Uint16, Uint32, Uint64)
-    @eval begin
-        function convert(::Type{$to}, x::BigFloat)
-            (isinteger(x) && (typemin($to) <= x <= typemax($to))) || throw(InexactError())
-            convert($to, ccall((:mpfr_get_ui,:libmpfr),
-                               Culong, (Ptr{BigFloat}, Int32), &x, 0))
-        end
-    end
+function unsafe_cast{T<:Signed}(::Type{T}, x::BigFloat, ri::Cint)
+    unsafe_cast(Int64, x, ri) % T
+end
+function unsafe_cast{T<:Unsigned}(::Type{T}, x::BigFloat, ri::Cint)
+    unsafe_cast(UInt64, x, ri) % T
 end
 
-function Base.BigInt(x::BigFloat)
-    !isinteger(x) && throw(InexactError())
-    return itrunc(x)
+function unsafe_cast(::Type{BigInt}, x::BigFloat, ri::Cint)
+    # actually safe, just keep naming consistent
+    z = BigInt()
+    ccall((:mpfr_get_z, :libmpfr), Int32, (Ptr{BigInt}, Ptr{BigFloat}, Int32),
+          &z, &x, ri)
+    z
 end
+unsafe_cast(::Type{Int128}, x::BigFloat, ri::Cint) = Int128(unsafe_cast(BigInt,x,ri))
+unsafe_cast(::Type{UInt128}, x::BigFloat, ri::Cint) = UInt128(unsafe_cast(BigInt,x,ri))
+unsafe_cast{T<:Integer}(::Type{T}, x::BigFloat, r::RoundingMode) = unsafe_cast(T,x,to_mpfr(r))
+
+unsafe_trunc{T<:Integer}(::Type{T}, x::BigFloat) = unsafe_cast(T,x,RoundToZero)
+
+function trunc{T<:Union{Signed,Unsigned}}(::Type{T}, x::BigFloat)
+    (typemin(T) <= x <= typemax(T)) || throw(InexactError())
+    unsafe_cast(T,x,RoundToZero)
+end
+function floor{T<:Union{Signed,Unsigned}}(::Type{T}, x::BigFloat)
+    (typemin(T) <= x <= typemax(T)) || throw(InexactError())
+    unsafe_cast(T,x,RoundDown)
+end
+function ceil{T<:Union{Signed,Unsigned}}(::Type{T}, x::BigFloat)
+    (typemin(T) <= x <= typemax(T)) || throw(InexactError())
+    unsafe_cast(T,x,RoundUp)
+end
+
+function round{T<:Union{Signed,Unsigned}}(::Type{T}, x::BigFloat)
+    (typemin(T) <= x <= typemax(T)) || throw(InexactError())
+    unsafe_cast(T,x,ROUNDING_MODE[end])
+end
+
+trunc(::Type{BigInt}, x::BigFloat) = unsafe_cast(BigInt, x, RoundToZero)
+floor(::Type{BigInt}, x::BigFloat) = unsafe_cast(BigInt, x, RoundDown)
+ceil(::Type{BigInt}, x::BigFloat) = unsafe_cast(BigInt, x, RoundUp)
+round(::Type{BigInt}, x::BigFloat) = unsafe_cast(BigInt, x, ROUNDING_MODE[end])
+
+# convert/round/trunc/floor/ceil(Integer, x) should return a BigInt
+trunc(::Type{Integer}, x::BigFloat) = trunc(BigInt, x)
+floor(::Type{Integer}, x::BigFloat) = floor(BigInt, x)
+ceil(::Type{Integer}, x::BigFloat) = ceil(BigInt, x)
+round(::Type{Integer}, x::BigFloat) = round(BigInt, x)
+
+convert(::Type{Bool}, x::BigFloat) = x==0 ? false : x==1 ? true : throw(InexactError())
+function convert(::Type{BigInt},x::BigFloat)
+    isinteger(x) || throw(InexactError())
+    trunc(BigInt,x)
+end
+
+function convert{T<:Integer}(::Type{T},x::BigFloat)
+    isinteger(x) || throw(InexactError())
+    trunc(T,x)
+end
+
+## BigFloat -> AbstractFloat
 convert(::Type{Float64}, x::BigFloat) =
     ccall((:mpfr_get_d,:libmpfr), Float64, (Ptr{BigFloat},Int32), &x, ROUNDING_MODE[end])
 convert(::Type{Float32}, x::BigFloat) =
     ccall((:mpfr_get_flt,:libmpfr), Float32, (Ptr{BigFloat},Int32), &x, ROUNDING_MODE[end])
+# TODO: avoid double rounding
+convert(::Type{Float16}, x::BigFloat) = convert(Float16, convert(Float32, x))
 
-convert(::Type{Integer}, x::BigFloat) = convert(BigInt, x)
+call(::Type{Float64}, x::BigFloat, r::RoundingMode) =
+    ccall((:mpfr_get_d,:libmpfr), Float64, (Ptr{BigFloat},Int32), &x, to_mpfr(r))
+call(::Type{Float32}, x::BigFloat, r::RoundingMode) =
+    ccall((:mpfr_get_flt,:libmpfr), Float32, (Ptr{BigFloat},Int32), &x, to_mpfr(r))
+# TODO: avoid double rounding
+call(::Type{Float16}, x::BigFloat, r::RoundingMode) =
+    convert(Float16, call(Float32, x, r))
 
 promote_rule{T<:Real}(::Type{BigFloat}, ::Type{T}) = BigFloat
-promote_rule{T<:FloatingPoint}(::Type{BigInt},::Type{T}) = BigFloat
-promote_rule{T<:FloatingPoint}(::Type{BigFloat},::Type{T}) = BigFloat
+promote_rule{T<:AbstractFloat}(::Type{BigInt},::Type{T}) = BigFloat
+promote_rule{T<:AbstractFloat}(::Type{BigFloat},::Type{T}) = BigFloat
 
-function convert(::Type{Rational{BigInt}}, x::FloatingPoint)
+function convert(::Type{Rational{BigInt}}, x::AbstractFloat)
     if isnan(x); return zero(BigInt)//zero(BigInt); end
     if isinf(x); return copysign(one(BigInt),x)//zero(BigInt); end
     if x == 0;   return zero(BigInt) // one(BigInt); end
@@ -132,15 +200,6 @@ function convert(::Type{Rational{BigInt}}, x::FloatingPoint)
     BigInt(ldexp(x,s)) // (BigInt(1) << s)
 end
 
-# serialization
-
-function serialize(s, n::BigFloat)
-    Base.serialize_type(s, BigFloat)
-    serialize(s, string(n))
-end
-
-deserialize(s, ::Type{BigFloat}) = BigFloat(deserialize(s))
-
 # Basic arithmetic without promotion
 for (fJ, fC) in ((:+,:add), (:*,:mul))
     @eval begin
@@ -246,6 +305,71 @@ function -(c::BigInt, x::BigFloat)
     return z
 end
 
+function fma(x::BigFloat, y::BigFloat, z::BigFloat)
+    r = BigFloat()
+    ccall(("mpfr_fma",:libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Ptr{BigFloat}, Ptr{BigFloat}, Int32), &r, &x, &y, &z, ROUNDING_MODE[end])
+    return r
+end
+
+# div
+# BigFloat
+function div(x::BigFloat, y::BigFloat)
+    z = BigFloat()
+    ccall((:mpfr_div,:libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Ptr{BigFloat}, Int32), &z, &x, &y, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+
+# Unsigned Int
+function div(x::BigFloat, c::CulongMax)
+    z = BigFloat()
+    ccall((:mpfr_div_ui, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Culong, Int32), &z, &x, c, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+function div(c::CulongMax, x::BigFloat)
+    z = BigFloat()
+    ccall((:mpfr_ui_div, :libmpfr), Int32, (Ptr{BigFloat}, Culong, Ptr{BigFloat}, Int32), &z, c, &x, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+
+# Signed Integer
+function div(x::BigFloat, c::ClongMax)
+    z = BigFloat()
+    ccall((:mpfr_div_si, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Clong, Int32), &z, &x, c, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+function div(c::ClongMax, x::BigFloat)
+    z = BigFloat()
+    ccall((:mpfr_si_div, :libmpfr), Int32, (Ptr{BigFloat}, Clong, Ptr{BigFloat}, Int32), &z, c, &x, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+
+# Float32/Float64
+function div(x::BigFloat, c::CdoubleMax)
+    z = BigFloat()
+    ccall((:mpfr_div_d, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Cdouble, Int32), &z, &x, c, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+function div(c::CdoubleMax, x::BigFloat)
+    z = BigFloat()
+    ccall((:mpfr_d_div, :libmpfr), Int32, (Ptr{BigFloat}, Cdouble, Ptr{BigFloat}, Int32), &z, c, &x, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+
+# BigInt
+function div(x::BigFloat, c::BigInt)
+    z = BigFloat()
+    ccall((:mpfr_div_z, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Ptr{BigInt}, Int32), &z, &x, &c, to_mpfr(RoundToZero))
+    ccall((:mpfr_trunc, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &z)
+    return z
+end
+
 
 
 # More efficient commutative operations
@@ -282,6 +406,7 @@ function -(x::BigFloat)
 end
 
 function sqrt(x::BigFloat)
+    isnan(x) && return x
     z = BigFloat()
     ccall((:mpfr_sqrt, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Int32), &z, &x, ROUNDING_MODE[end])
     if isnan(z)
@@ -292,9 +417,6 @@ end
 
 sqrt(x::BigInt) = sqrt(BigFloat(x))
 
-rad2deg(z::BigFloat) = 180/big(pi)*z
-deg2rad(z::BigFloat) = big(pi)/180*z
-
 function ^(x::BigFloat, y::BigFloat)
     z = BigFloat()
     ccall((:mpfr_pow, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Ptr{BigFloat}, Int32), &z, &x, &y, ROUNDING_MODE[end])
@@ -574,23 +696,33 @@ end
 maxintfloat(x::BigFloat) = BigFloat(2)^precision(x)
 maxintfloat(::Type{BigFloat}) = BigFloat(2)^get_bigfloat_precision()
 
-function to_mpfr(r::RoundingMode)
-    c = r.code
-    if !(0 <= c <= 4)
-        error("invalid BigFloat rounding mode")
-    end
-    c
-end
+to_mpfr(::RoundingMode{:Nearest}) = Cint(0)
+to_mpfr(::RoundingMode{:ToZero}) = Cint(1)
+to_mpfr(::RoundingMode{:Up}) = Cint(2)
+to_mpfr(::RoundingMode{:Down}) = Cint(3)
+to_mpfr(::RoundingMode{:FromZero}) = Cint(4)
 
 function from_mpfr(c::Integer)
-    if !(0 <= c <= 4)
-        error("invalid MPFR rounding mode code")
+    if c == 0
+        return RoundNearest
+    elseif c == 1
+        return RoundToZero
+    elseif c == 2
+        return RoundUp
+    elseif c == 3
+        return RoundDown
+    elseif c == 4
+        return RoundFromZero
+    else
+        throw(ArgumentError("invalid MPFR rounding mode code: $c"))
     end
     RoundingMode(c)
 end
 
-get_rounding(::Type{BigFloat}) = from_mpfr(ROUNDING_MODE[end])
-set_rounding(::Type{BigFloat},r::RoundingMode) = ROUNDING_MODE[end] = to_mpfr(r)
+get_rounding_raw(::Type{BigFloat}) = ROUNDING_MODE[end]
+set_rounding_raw(::Type{BigFloat},i::Integer) = ROUNDING_MODE[end] = i
+get_rounding(::Type{BigFloat}) = from_mpfr(get_rounding_raw(BigFloat))
+set_rounding(::Type{BigFloat},r::RoundingMode) = set_rounding_raw(BigFloat,to_mpfr(r))
 
 function copysign(x::BigFloat, y::BigFloat)
     z = BigFloat()
@@ -626,7 +758,7 @@ function isinteger(x::BigFloat)
     return ccall((:mpfr_integer_p, :libmpfr), Int32, (Ptr{BigFloat},), &x) != 0
 end
 
-for f in (:ceil, :floor, :trunc, :round)
+for f in (:ceil, :floor, :trunc)
     @eval begin
         function ($f)(x::BigFloat)
             z = BigFloat()
@@ -636,13 +768,16 @@ for f in (:ceil, :floor, :trunc, :round)
     end
 end
 
-function itrunc(x::BigFloat)
-    z = BigInt()
-    ccall((:mpfr_get_z, :libmpfr), Int32, (Ptr{BigInt}, Ptr{BigFloat}, Int32), &z, &x, to_mpfr(RoundToZero))
+function round(x::BigFloat)
+    z = BigFloat()
+    ccall((:mpfr_rint, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}, Cint), &z, &x, ROUNDING_MODE[end])
+    return z
+end
+function round(x::BigFloat,::RoundingMode{:NearestTiesAway})
+    z = BigFloat()
+    ccall((:mpfr_round, :libmpfr), Int32, (Ptr{BigFloat}, Ptr{BigFloat}), &z, &x)
     return z
 end
-
-iround(x::BigFloat) = itrunc(round(x))
 
 function isinf(x::BigFloat)
     return ccall((:mpfr_inf_p, :libmpfr), Int32, (Ptr{BigFloat},), &x) != 0
@@ -654,10 +789,7 @@ end
 
 isfinite(x::BigFloat) = !isinf(x) && !isnan(x)
 
-@eval inf(::Type{BigFloat}) = $(BigFloat(Inf))
-@eval nan(::Type{BigFloat}) = $(BigFloat(NaN))
-
-typemax(::Type{BigFloat}) = inf(BigFloat)
+@eval typemax(::Type{BigFloat}) = $(BigFloat( Inf))
 @eval typemin(::Type{BigFloat}) = $(BigFloat(-Inf))
 
 function nextfloat(x::BigFloat)
@@ -679,7 +811,7 @@ end
 eps(::Type{BigFloat}) = nextfloat(BigFloat(1)) - BigFloat(1)
 
 realmin(::Type{BigFloat}) = nextfloat(zero(BigFloat))
-realmax(::Type{BigFloat}) = prevfloat(inf(BigFloat))
+realmax(::Type{BigFloat}) = prevfloat(BigFloat(Inf))
 
 function with_bigfloat_precision(f::Function, precision::Integer)
     old_precision = get_bigfloat_precision()
@@ -692,18 +824,35 @@ function with_bigfloat_precision(f::Function, precision::Integer)
 end
 
 function string(x::BigFloat)
-    lng = 128
-    for i = 1:2
-        z = Array(Uint8, lng + 1)
-        lng = ccall((:mpfr_snprintf,:libmpfr), Int32, (Ptr{Uint8}, Culong, Ptr{Uint8}, Ptr{BigFloat}...), z, lng + 1, "%.Re", &x)
-        if lng < 128 || i == 2
-            return bytestring(z[1:lng])
-        end
+    # In general, the number of decimal places needed to read back the number exactly
+    # is, excluding the most significant, ceil(log(10, 2^precision(x)))
+    k = ceil(Int32, precision(x) * 0.3010299956639812)
+    lng = k + Int32(8) # Add space for the sign, the most significand digit, the dot and the exponent
+    buf = Array(UInt8, lng + 1)
+    lng = ccall((:mpfr_snprintf,:libmpfr), Int32, (Ptr{UInt8}, Culong, Ptr{UInt8}, Ptr{BigFloat}...), buf, lng + 1, "%.Re", &x)
+    if lng < k + 5 # print at least k decimal places
+        lng = ccall((:mpfr_sprintf,:libmpfr), Int32, (Ptr{UInt8}, Ptr{UInt8}, Ptr{BigFloat}...), buf, "%.$(k)Re", &x)
+    elseif lng > k + 8
+        buf = Array(UInt8, lng + 1)
+        lng = ccall((:mpfr_snprintf,:libmpfr), Int32, (Ptr{UInt8}, Culong, Ptr{UInt8}, Ptr{BigFloat}...), buf, lng + 1, "%.Re", &x)
     end
+    return bytestring(pointer(buf), (1 <= x < 10 || -10 < x <= -1 || x == 0) ? lng - 4 : lng)
 end
 
 print(io::IO, b::BigFloat) = print(io, string(b))
-show(io::IO, b::BigFloat) = print(io, string(b), " with $(precision(b)) bits of precision")
+show(io::IO, b::BigFloat) = print(io, string(b))
 showcompact(io::IO, b::BigFloat) = print(io, string(b))
 
+# get/set exponent min/max
+get_emax() = ccall((:mpfr_get_emax, :libmpfr), Clong, ())
+get_emax_min() = ccall((:mpfr_get_emax_min, :libmpfr), Clong, ())
+get_emax_max() = ccall((:mpfr_get_emax_max, :libmpfr), Clong, ())
+
+get_emin() = ccall((:mpfr_get_emin, :libmpfr), Clong, ())
+get_emin_min() = ccall((:mpfr_get_emin_min, :libmpfr), Clong, ())
+get_emin_max() = ccall((:mpfr_get_emin_max, :libmpfr), Clong, ())
+
+set_emax!(x) = ccall((:mpfr_set_emax, :libmpfr), Void, (Clong,), x)
+set_emin!(x) = ccall((:mpfr_set_emin, :libmpfr), Void, (Clong,), x)
+
 end #module
