@@ -1,9 +1,11 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 function is_unix(os::Symbol)
     if (os==:Windows) return false;
     elseif (os==:Linux) return true;
     elseif (os==:FreeBSD) return true;
     elseif (os==:Darwin) return true;
-    else error("unknown operating system")
+    else throw(ArgumentError("unknown operating system, $(repr(os))"))
     end
 end
 
@@ -29,7 +31,7 @@ macro osx(qm,ex)
     _os_test(qm, ex, OS_NAME===:Darwin)
 end
 macro linux(qm,ex)
-    _os_test(qm, ex, is_unix(OS_NAME) && OS_NAME!==:Darwin)
+    _os_test(qm, ex, OS_NAME===:Linux)
 end
 
 macro windows_only(ex)
@@ -48,17 +50,9 @@ end
 # Windows version macros
 
 @windows_only function windows_version()
-    verinfo = ccall(:GetVersion, Uint32, ())
+    verinfo = ccall(:GetVersion, UInt32, ())
     (verinfo & 0xFF, (verinfo >> 8) & 0xFF)
 end
 @unix_only windows_version() = (0,0)
 
-WINDOWS_XP_VER = (5,1)
-
-macro windowsxp_only(ex)
-    _os_test(:?, Expr(:(:), ex, nothing), OS_NAME===:Windows && windows_version() <= WINDOWS_XP_VER)
-end
-
-macro non_windowsxp_only(ex)
-    _os_test(:?, Expr(:(:), nothing, ex), OS_NAME===:Windows && windows_version() <= WINDOWS_XP_VER)
-end
+WINDOWS_VISTA_VER = (6,0)
