@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 @unix_only begin
     const path_separator    = "/"
     const path_separator_re = r"/+"
@@ -6,7 +8,7 @@
     const path_dir_splitter = r"^(.*?)(/+)([^/]*)$"
     const path_ext_splitter = r"^((?:.*/)?(?:\.|[^/\.])[^/]*?)(\.[^/\.]*|)$"
 
-    splitdrive(path::String) = ("",path)
+    splitdrive(path::AbstractString) = ("",path)
     homedir() = ENV["HOME"]
 end
 @windows_only begin
@@ -17,58 +19,58 @@ end
     const path_dir_splitter = r"^(.*?)([/\\]+)([^/\\]*)$"
     const path_ext_splitter = r"^((?:.*[/\\])?(?:\.|[^/\\\.])[^/\\]*?)(\.[^/\\\.]*|)$"
 
-    function splitdrive(path::String)
+    function splitdrive(path::AbstractString)
         m = match(r"^(\w+:|\\\\\w+\\\w+|\\\\\?\\UNC\\\w+\\\w+|\\\\\?\\\w+:|)(.*)$", path)
         bytestring(m.captures[1]), bytestring(m.captures[2])
     end
     homedir() = get(ENV,"HOME",string(ENV["HOMEDRIVE"],ENV["HOMEPATH"]))
 end
 
-isabspath(path::String) = ismatch(path_absolute_re, path)
-isdirpath(path::String) = ismatch(path_directory_re, splitdrive(path)[2])
+isabspath(path::AbstractString) = ismatch(path_absolute_re, path)
+isdirpath(path::AbstractString) = ismatch(path_directory_re, splitdrive(path)[2])
 
 function splitdir(path::ByteString)
     a, b = splitdrive(path)
     m = match(path_dir_splitter,b)
-    m == nothing && return (a,b)
+    m === nothing && return (a,b)
     a = string(a, isempty(m.captures[1]) ? m.captures[2][1] : m.captures[1])
     a, bytestring(m.captures[3])
 end
-splitdir(path::String) = splitdir(bytestring(path))
+splitdir(path::AbstractString) = splitdir(bytestring(path))
 
- dirname(path::String) = splitdir(path)[1]
-basename(path::String) = splitdir(path)[2]
+ dirname(path::AbstractString) = splitdir(path)[1]
+basename(path::AbstractString) = splitdir(path)[2]
 
-function splitext(path::String)
+function splitext(path::AbstractString)
     a, b = splitdrive(path)
     m = match(path_ext_splitter, b)
-    m == nothing && return (path,"")
+    m === nothing && return (path,"")
     a*m.captures[1], bytestring(m.captures[2])
 end
 
-function pathsep(paths::String...)
+function pathsep(paths::AbstractString...)
     for path in paths
         m = match(path_separator_re, path)
-        m != nothing && return m.match[1]
+        m !== nothing && return m.match[1]
     end
     return path_separator
 end
 
-joinpath(a::String) = a
-joinpath(a::String, b::String, c::String...) = joinpath(joinpath(a,b), c...)
+joinpath(a::AbstractString) = a
+joinpath(a::AbstractString, b::AbstractString, c::AbstractString...) = joinpath(joinpath(a,b), c...)
 
-function joinpath(a::String, b::String)
+function joinpath(a::AbstractString, b::AbstractString)
     isabspath(b) && return b
     A, a = splitdrive(a)
     B, b = splitdrive(b)
-    !isempty(B) && A != B && error("drive mismatch: $A$a $B$b")
+    !isempty(B) && A != B && throw(ArgumentError("drive mismatch: $A$a $B$b"))
     C = isempty(B) ? A : B
     isempty(a)                             ? string(C,b) :
     ismatch(path_separator_re, a[end:end]) ? string(C,a,b) :
                                              string(C,a,pathsep(a,b),b)
 end
 
-function normpath(path::String)
+function normpath(path::AbstractString)
     isabs = isabspath(path)
     isdir = isdirpath(path)
     drive, path = splitdrive(path)
@@ -101,19 +103,19 @@ function normpath(path::String)
     end
     string(drive,path)
 end
-normpath(a::String, b::String...) = normpath(joinpath(a,b...))
+normpath(a::AbstractString, b::AbstractString...) = normpath(joinpath(a,b...))
 
-abspath(a::String) = normpath(isabspath(a) ? a : joinpath(pwd(),a))
-abspath(a::String, b::String...) = abspath(joinpath(a,b...))
+abspath(a::AbstractString) = normpath(isabspath(a) ? a : joinpath(pwd(),a))
+abspath(a::AbstractString, b::AbstractString...) = abspath(joinpath(a,b...))
 
-@windows_only realpath(path::String) = realpath(utf16(path))
+@windows_only realpath(path::AbstractString) = realpath(utf16(path))
 @windows_only function realpath(path::UTF16String)
-    p = uint32((sizeof(path)>>2) + 1)
+    p = UInt32((sizeof(path)>>2) + 1)
     while true
         buflength = p
-        buf = zeros(Uint16,buflength)
+        buf = zeros(UInt16,buflength)
         p = ccall((:GetFullPathNameW, "Kernel32"), stdcall,
-            Uint32, (Ptr{Uint16}, Uint32, Ptr{Uint16}, Ptr{Void}),
+            UInt32, (Cwstring, UInt32, Ptr{UInt16}, Ptr{Void}),
             path, buflength, buf, C_NULL)
         systemerror(:realpath, p == 0)
         if (p < buflength)
@@ -123,21 +125,50 @@ abspath(a::String, b::String...) = abspath(joinpath(a,b...))
     end
 end
 
-@unix_only function realpath(path::String)
-    p = ccall(:realpath, Ptr{Uint8}, (Ptr{Uint8}, Ptr{Uint8}), path, C_NULL)
+@unix_only function realpath(path::AbstractString)
+    p = ccall(:realpath, Ptr{UInt8}, (Cstring, Ptr{UInt8}), path, C_NULL)
     systemerror(:realpath, p == C_NULL)
     s = bytestring(p)
-    c_free(p)
+    Libc.free(p)
     return s
 end
 
-@windows_only expanduser(path::String) = path # on windows, ~ means "temporary file"
-@unix_only function expanduser(path::String)
+@windows_only expanduser(path::AbstractString) = path # on windows, ~ means "temporary file"
+@unix_only function expanduser(path::AbstractString)
     i = start(path)
     c, i = next(path,i)
     if c != '~' return path end
     if done(path,i) return homedir() end
     c, j = next(path,i)
     if c == '/' return homedir()*path[i:end] end
-    error("~user tilde expansion not yet implemented")
+    throw(ArgumentError("~user tilde expansion not yet implemented"))
+end
+
+function relpath(path::AbstractString, startpath::AbstractString = ".")
+    isempty(path) && throw(ArgumentError("`path` must be specified"))
+    isempty(startpath) && throw(ArgumentError("`startpath` must be specified"))
+    curdir = "."
+    pardir = ".."
+    path == startpath && return curdir
+    path_arr  = split(abspath(path),      path_separator_re)
+    start_arr = split(abspath(startpath), path_separator_re)
+    i = 0
+    while i < min(length(path_arr), length(start_arr))
+        i += 1
+        if path_arr[i] != start_arr[i]
+            i -= 1
+            break
+        end
+    end
+    pathpart = join(path_arr[i+1:findlast(x -> !isempty(x), path_arr)], path_separator)
+    prefix_num = findlast(x -> !isempty(x), start_arr) - i - 1
+    if prefix_num >= 0
+        prefix = pardir * path_separator
+        relpath_ = isempty(pathpart)     ?
+            (prefix^prefix_num) * pardir :
+            (prefix^prefix_num) * pardir * path_separator * pathpart
+    else
+        relpath_ = pathpart
+    end
+    return isempty(relpath_) ? curdir :  relpath_
 end
