@@ -1,9 +1,11 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module Cache
 
 import ..Git, ..Dir
 using ..Types
 
-path(pkg::String) = abspath(".cache", pkg)
+path(pkg::AbstractString) = abspath(".cache", pkg)
 
 function mkcachedir()
     cache = joinpath(realpath("."), ".cache")
@@ -11,29 +13,26 @@ function mkcachedir()
         return
     end
 
-    @windows_only mkdir(cache)
-    @unix_only begin
-        if Dir.isversioned(pwd())
-            rootcache = joinpath(realpath(".."), ".cache")
-            if !isdir(rootcache)
-                mkdir(rootcache)
-            end
-            symlink(rootcache, cache)
-            return
+    @unix_only if Dir.isversioned(pwd())
+        rootcache = joinpath(realpath(".."), ".cache")
+        if !isdir(rootcache)
+            mkdir(rootcache)
         end
-        mkdir(cache)
+        symlink(rootcache, cache)
+        return
     end
+    mkdir(cache)
 end
 
 
-function prefetch{S<:String}(pkg::String, url::String, sha1s::Vector{S})
+function prefetch(pkg::AbstractString, url::AbstractString, sha1s::Vector)
     isdir(".cache") || mkcachedir()
     cache = path(pkg)
     if !isdir(cache)
         info("Cloning cache of $pkg from $url")
         try Git.run(`clone -q --mirror $url $cache`)
         catch
-            rm(cache, recursive=true)
+            isdir(cache) && rm(cache, recursive=true)
             rethrow()
         end
     end
@@ -47,6 +46,6 @@ function prefetch{S<:String}(pkg::String, url::String, sha1s::Vector{S})
     end
     sha1s[!in_cache]
 end
-prefetch(pkg::String, url::String, sha1::String...) = prefetch(pkg, url, String[sha1...])
+prefetch(pkg::AbstractString, url::AbstractString, sha1::AbstractString...) = prefetch(pkg, url, AbstractString[sha1...])
 
 end # module
