@@ -1,7 +1,9 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module Entry
 
 import Base: thispatch, nextpatch, nextminor, nextmajor, check_new_version
-import ..Git, ..Reqs, ..Read, ..Query, ..Resolve, ..Cache, ..Write, ..GitHub
+import ..Git, ..Reqs, ..Read, ..Query, ..Resolve, ..Cache, ..Write, ..GitHub, ..Dir
 using ..Types
 
 macro recover(ex)
@@ -13,7 +15,7 @@ macro recover(ex)
     end
 end
 
-function edit(f::Function, pkg::String, args...)
+function edit(f::Function, pkg::AbstractString, args...)
     r = Reqs.read("REQUIRE")
     reqs = Reqs.parse(r)
     avail = Read.available()
@@ -29,7 +31,7 @@ end
 
 function edit()
     editor = get(ENV,"VISUAL",get(ENV,"EDITOR",nothing))
-    editor != nothing ||
+    editor !== nothing ||
         error("set the EDITOR environment variable to an edit command")
     editor = Base.shell_split(editor)
     reqs = Reqs.parse("REQUIRE")
@@ -40,20 +42,20 @@ function edit()
     resolve(reqsʹ)
 end
 
-function add(pkg::String, vers::VersionSet)
+function add(pkg::AbstractString, vers::VersionSet)
     outdated = :maybe
     @sync begin
         @async if !edit(Reqs.add,pkg,vers)
             ispath(pkg) || error("unknown package $pkg")
             info("Nothing to be done")
         end
-        branch = Pkg.META_BRANCH
+        branch = Dir.getmetabranch()
         if Git.branch(dir="METADATA") == branch
             if !Git.success(`diff --quiet origin/$branch`, dir="METADATA")
                 outdated = :yes
             else
                 try
-                    run(Git.cmd(`fetch -q --all`, dir="METADATA") |>DevNull .>DevNull)
+                    run(pipeline(Git.cmd(`fetch -q --all`, dir="METADATA"),stdout=DevNull,stderr=DevNull))
                     outdated = Git.success(`diff --quiet origin/$branch`, dir="METADATA") ?
                         (:no) : (:yes)
                 end
@@ -68,21 +70,28 @@ function add(pkg::String, vers::VersionSet)
         info("Use `Pkg.update()` to get the latest versions of your packages")
     end
 end
-add(pkg::String, vers::VersionNumber...) = add(pkg,VersionSet(vers...))
+add(pkg::AbstractString, vers::VersionNumber...) = add(pkg,VersionSet(vers...))
 
-function rm(pkg::String)
+function rm(pkg::AbstractString)
     edit(Reqs.rm,pkg) && return
     ispath(pkg) || return info("Nothing to be done")
     info("Removing $pkg (unregistered)")
     Write.remove(pkg)
 end
 
-available() = sort!(ASCIIString[keys(Read.available())...], by=lowercase)
+function available()
+    all_avail = Read.available()
+    avail = AbstractString[]
+    for (pkg, vers) in all_avail
+        any(x->Types.satisfies("julia", VERSION, x[2].requires), vers) && push!(avail, pkg)
+    end
+    sort!(avail, by=lowercase)
+end
 
-function available(pkg::String)
+function available(pkg::AbstractString)
     avail = Read.available(pkg)
     if !isempty(avail) || Read.isinstalled(pkg)
-        return sort!([keys(avail)...])
+        return sort!(collect(keys(avail)))
     end
     error("$pkg is not a package (not registered or installed)")
 end
@@ -95,45 +104,47 @@ function installed()
     return pkgs
 end
 
-function installed(pkg::String)
+function installed(pkg::AbstractString)
     avail = Read.available(pkg)
     Read.isinstalled(pkg) && return Read.installed_version(pkg,avail)
     isempty(avail) && error("$pkg is not a package (not registered or installed)")
     return nothing # registered but not installed
 end
 
-function status(io::IO)
+function status(io::IO; pkgname::AbstractString = "")
+    showpkg(pkg) = (pkgname == "") ? (true) : (pkg == pkgname)
     reqs = Reqs.parse("REQUIRE")
     instd = Read.installed()
-    required = sort!([keys(reqs)...])
+    required = sort!(collect(keys(reqs)))
     if !isempty(required)
-        println(io, "$(length(required)) required packages:")
+        showpkg("") && println(io, "$(length(required)) required packages:")
         for pkg in required
             ver,fix = pop!(instd,pkg)
-            status(io,pkg,ver,fix)
+            showpkg(pkg) && status(io,pkg,ver,fix)
         end
     end
-    additional = sort!([keys(instd)...])
+    additional = sort!(collect(keys(instd)))
     if !isempty(additional)
-        println(io, "$(length(additional)) additional packages:")
+        showpkg("") && println(io, "$(length(additional)) additional packages:")
         for pkg in additional
             ver,fix = instd[pkg]
-            status(io,pkg,ver,fix)
+            showpkg(pkg) && status(io,pkg,ver,fix)
         end
     end
     if isempty(required) && isempty(additional)
         println(io, "No packages installed")
     end
 end
-# TODO: status(io::IO, pkg::String)
 
-function status(io::IO, pkg::String, ver::VersionNumber, fix::Bool)
+status(io::IO, pkg::AbstractString) = status(io, pkgname = pkg)
+
+function status(io::IO, pkg::AbstractString, ver::VersionNumber, fix::Bool)
     @printf io " - %-29s " pkg
     fix || return println(io,ver)
     @printf io "%-19s" ver
     if ispath(pkg,".git")
         print(io, Git.attached(dir=pkg) ? Git.branch(dir=pkg) : Git.head(dir=pkg)[1:8])
-        attrs = String[]
+        attrs = AbstractString[]
         isfile("METADATA",pkg,"url") || push!(attrs,"unregistered")
         Git.dirty(dir=pkg) && push!(attrs,"dirty")
         isempty(attrs) || print(io, " (",join(attrs,", "),")")
@@ -143,7 +154,7 @@ function status(io::IO, pkg::String, ver::VersionNumber, fix::Bool)
     println(io)
 end
 
-function clone(url::String, pkg::String)
+function clone(url::AbstractString, pkg::AbstractString)
     info("Cloning $pkg from $url")
     ispath(pkg) && error("$pkg already exists")
     try
@@ -160,22 +171,22 @@ function clone(url::String, pkg::String)
     end
 end
 
-function clone(url_or_pkg::String)
+function clone(url_or_pkg::AbstractString)
     urlpath = joinpath("METADATA",url_or_pkg,"url")
-    if isfile(urlpath)
+    if !(':' in url_or_pkg) && isfile(urlpath)
         pkg = url_or_pkg
         url = readchomp(urlpath)
         # TODO: Cache.prefetch(pkg,url)
     else
         url = url_or_pkg
         m = match(r"(?:^|[/\\])(\w+?)(?:\.jl)?(?:\.git)?$", url)
-        m != nothing || error("can't determine package name from URL: $url")
+        m !== nothing || error("can't determine package name from URL: $url")
         pkg = m.captures[1]
     end
     clone(url,pkg)
 end
 
-function _checkout(pkg::String, what::String, merge::Bool=false, pull::Bool=false, branch::Bool=false)
+function _checkout(pkg::AbstractString, what::AbstractString, merge::Bool=false, pull::Bool=false, branch::Bool=false)
     Git.transact(dir=pkg) do
         Git.dirty(dir=pkg) && error("$pkg is dirty, bailing")
         branch ? Git.run(`checkout -q -B $what -t origin/$what`, dir=pkg) : Git.run(`checkout -q $what`, dir=pkg)
@@ -188,20 +199,20 @@ function _checkout(pkg::String, what::String, merge::Bool=false, pull::Bool=fals
     end
 end
 
-function checkout(pkg::String, branch::String, merge::Bool, pull::Bool)
+function checkout(pkg::AbstractString, branch::AbstractString, merge::Bool, pull::Bool)
     ispath(pkg,".git") || error("$pkg is not a git repo")
     info("Checking out $pkg $branch...")
     _checkout(pkg,branch,merge,pull,true)
 end
 
-function free(pkg::String)
+function free(pkg::AbstractString)
     ispath(pkg,".git") || error("$pkg is not a git repo")
     Read.isinstalled(pkg) || error("$pkg cannot be freed – not an installed package")
     avail = Read.available(pkg)
     isempty(avail) && error("$pkg cannot be freed – not a registered package")
-    Git.dirty(dir=pkg) && error("$pkg cannot be freed – repo is dirty")
+    Git.dirty(dir=pkg) && error("$pkg cannot be freed – repo is dirty")
     info("Freeing $pkg")
-    vers = sort!([keys(avail)...], rev=true)
+    vers = sort!(collect(keys(avail)), rev=true)
     while true
         for ver in vers
             sha1 = avail[ver].sha1
@@ -213,7 +224,31 @@ function free(pkg::String)
     end
 end
 
-function pin(pkg::String, head::String)
+function free(pkgs)
+    try
+        for pkg in pkgs
+            ispath(pkg,".git") || error("$pkg is not a git repo")
+            Read.isinstalled(pkg) || error("$pkg cannot be freed – not an installed package")
+            avail = Read.available(pkg)
+            isempty(avail) && error("$pkg cannot be freed – not a registered package")
+            Git.dirty(dir=pkg) && error("$pkg cannot be freed – repo is dirty")
+            info("Freeing $pkg")
+            vers = sort!(collect(keys(avail)), rev=true)
+            for ver in vers
+                sha1 = avail[ver].sha1
+                Git.iscommit(sha1, dir=pkg) || continue
+                Git.run(`checkout -q $sha1`, dir=pkg)
+                break
+            end
+            isempty(Cache.prefetch(pkg, Read.url(pkg), [a.sha1 for (v,a)=avail])) && continue
+            error("can't find any registered versions of $pkg to checkout")
+        end
+    finally
+        resolve()
+    end
+end
+
+function pin(pkg::AbstractString, head::AbstractString)
     ispath(pkg,".git") || error("$pkg is not a git repo")
     branch = "pinned.$(head[1:8]).tmp"
     rslv = (head != Git.head(dir=pkg))
@@ -221,18 +256,18 @@ function pin(pkg::String, head::String)
     Git.run(`checkout -q -B $branch $head`, dir=pkg)
     rslv ? resolve() : nothing
 end
-pin(pkg::String) = pin(pkg,Git.head(dir=pkg))
+pin(pkg::AbstractString) = pin(pkg,Git.head(dir=pkg))
 
-function pin(pkg::String, ver::VersionNumber)
+function pin(pkg::AbstractString, ver::VersionNumber)
     ispath(pkg,".git") || error("$pkg is not a git repo")
-    Read.isinstalled(pkg) || error("$pkg cannot be pinned – not an installed package".tmp)
+    Read.isinstalled(pkg) || error("$pkg cannot be pinned – not an installed package")
     avail = Read.available(pkg)
-    isempty(avail) && error("$pkg cannot be pinned – not a registered package".tmp)
-    haskey(avail,ver) || error("$pkg – $ver is not a registered version")
-    pin(pkg,avail[ver].sha1)
+    isempty(avail) && error("$pkg cannot be pinned – not a registered package")
+    haskey(avail,ver) || error("$pkg – $ver is not a registered version")
+    pin(pkg, avail[ver].sha1)
 end
 
-function update(branch::String)
+function update(branch::AbstractString)
     info("Updating METADATA...")
     cd("METADATA") do
         if Git.branch() != branch
@@ -244,24 +279,24 @@ function update(branch::String)
             Git.run(`checkout -q $branch`)
         end
         # TODO: handle merge conflicts
-        Base.with_env("GIT_MERGE_AUTOEDIT","no") do
-            Git.run(`pull -q`, out=DevNull)
+        Base.withenv("GIT_MERGE_AUTOEDIT"=>"no") do
+            Git.run(`pull --rebase -q`, out=DevNull)
         end
     end
     avail = Read.available()
     # this has to happen before computing free/fixed
-    @sync for pkg in filter!(Read.isinstalled,[keys(avail)...])
-        @async Cache.prefetch(pkg, Read.url(pkg), [a.sha1 for (v,a)=avail[pkg]])
+    for pkg in filter!(Read.isinstalled,collect(keys(avail)))
+        Cache.prefetch(pkg, Read.url(pkg), [a.sha1 for (v,a)=avail[pkg]])
     end
     instd = Read.installed(avail)
     free = Read.free(instd)
-    @sync for (pkg,ver) in free
-        @async Cache.prefetch(pkg, Read.url(pkg), [a.sha1 for (v,a)=avail[pkg]])
+    for (pkg,ver) in free
+        Cache.prefetch(pkg, Read.url(pkg), [a.sha1 for (v,a)=avail[pkg]])
     end
     fixed = Read.fixed(avail,instd)
-    @sync for (pkg,ver) in fixed
+    for (pkg,ver) in fixed
         ispath(pkg,".git") || continue
-        @async begin
+        begin
             if Git.attached(dir=pkg) && !Git.dirty(dir=pkg)
                 info("Updating $pkg...")
                 @recover begin
@@ -277,15 +312,15 @@ function update(branch::String)
     info("Computing changes...")
     resolve(Reqs.parse("REQUIRE"), avail, instd, fixed, free)
     # Don't use instd here since it may have changed
-    updatehook(sort!([keys(installed())...]))
+    updatehook(sort!(collect(keys(installed()))))
 end
 
-function pull_request(dir::String, commit::String="", url::String="")
+function pull_request(dir::AbstractString, commit::AbstractString="", url::AbstractString="")
     commit = isempty(commit) ? Git.head(dir=dir) :
         Git.readchomp(`rev-parse --verify $commit`, dir=dir)
     isempty(url) && (url = Git.readchomp(`config remote.origin.url`, dir=dir))
     m = match(Git.GITHUB_REGEX, url)
-    m == nothing && error("not a GitHub repo URL, can't make a pull request: $url")
+    m === nothing && error("not a GitHub repo URL, can't make a pull request: $url")
     owner, repo = m.captures[2:3]
     user = GitHub.user()
     info("Forking $owner/$repo to $user")
@@ -298,17 +333,17 @@ function pull_request(dir::String, commit::String="", url::String="")
     @osx? run(`open $pr_url`) : info("To create a pull-request, open:\n\n  $pr_url\n")
 end
 
-function submit(pkg::String, commit::String="")
+function submit(pkg::AbstractString, commit::AbstractString="")
     urlpath = joinpath("METADATA",pkg,"url")
     url = ispath(urlpath) ? readchomp(urlpath) : ""
     pull_request(pkg, commit, url)
 end
 
-function publish(branch::String)
+function publish(branch::AbstractString)
     Git.branch(dir="METADATA") == branch ||
         error("METADATA must be on $branch to publish changes")
     Git.run(`fetch -q`, dir="METADATA")
-    ahead_remote, ahead_local = map(int,split(Git.readchomp(`rev-list --count --left-right origin/$branch...$branch`, dir="METADATA"),'\t'))
+    ahead_remote, ahead_local = map(x->parse(Int,x),split(Git.readchomp(`rev-list --count --left-right origin/$branch...$branch`, dir="METADATA"),'\t'))
     ahead_remote > 0 && error("METADATA is behind origin/$branch – run `Pkg.update()` before publishing")
     ahead_local == 0 && error("There are no METADATA changes to publish")
 
@@ -318,7 +353,7 @@ function publish(branch::String)
     for line in eachline(Git.cmd(cmd, dir="METADATA"))
         path = chomp(line)
         m = match(r"^(.+?)/versions/([^/]+)/sha1$", path)
-        m != nothing && ismatch(Base.VERSION_REGEX, m.captures[2]) || continue
+        m !== nothing && ismatch(Base.VERSION_REGEX, m.captures[2]) || continue
         pkg, ver = m.captures; ver = convert(VersionNumber,ver)
         sha1 = readchomp(joinpath("METADATA",path))
         if Git.success(`cat-file -e origin/$branch:$path`, dir="METADATA")
@@ -335,7 +370,7 @@ function publish(branch::String)
     isempty(tags) && info("No new package versions to publish")
     info("Validating METADATA")
     check_metadata(Set(keys(tags)))
-    @sync for pkg in sort!([keys(tags)...])
+    @sync for pkg in sort!(collect(keys(tags)))
         @async begin
             forced = ASCIIString[]
             unforced = ASCIIString[]
@@ -391,7 +426,7 @@ function resolve(
     isempty(changes) && return info("No packages to install, update or remove")
 
     # prefetch phase isolates network activity, nothing to roll back
-    missing = {}
+    missing = []
     for (pkg,(ver1,ver2)) in changes
         vers = ASCIIString[]
         ver1 !== nothing && push!(vers,Git.head(dir=pkg))
@@ -409,13 +444,13 @@ function resolve(
     end
 
     # try applying changes, roll back everything if anything fails
-    changed = {}
+    changed = []
     try
         for (pkg,(ver1,ver2)) in changes
             if ver1 === nothing
                 info("Installing $pkg v$ver2")
                 Write.install(pkg, Read.sha1(pkg,ver2))
-            elseif ver2 == nothing
+            elseif ver2 === nothing
                 info("Removing $pkg v$ver1")
                 Write.remove(pkg)
             else
@@ -427,10 +462,10 @@ function resolve(
         end
     catch
         for (pkg,(ver1,ver2)) in reverse!(changed)
-            if ver1 == nothing
+            if ver1 === nothing
                 info("Rolling back install of $pkg")
                 @recover Write.remove(pkg)
-            elseif ver2 == nothing
+            elseif ver2 === nothing
                 info("Rolling back deleted $pkg to v$ver1")
                 @recover Write.install(pkg, Read.sha1(pkg,ver1))
             else
@@ -441,10 +476,10 @@ function resolve(
         rethrow()
     end
     # re/build all updated/installed packages
-    build(map(x->x[1],filter(x->x[2][2]!=nothing,changes)))
+    build(map(x->x[1], filter(x -> x[2][2] !== nothing, changes)))
 end
 
-function write_tag_metadata(pkg::String, ver::VersionNumber, commit::String, force::Bool=false)
+function write_tag_metadata(pkg::AbstractString, ver::VersionNumber, commit::AbstractString, force::Bool=false)
     cmd = Git.cmd(`cat-file blob $commit:REQUIRE`, dir=pkg)
     reqs = success(cmd) ? Reqs.read(cmd) : Reqs.Line[]
     cd("METADATA") do
@@ -469,7 +504,7 @@ function write_tag_metadata(pkg::String, ver::VersionNumber, commit::String, for
     return nothing
 end
 
-function register(pkg::String, url::String)
+function register(pkg::AbstractString, url::AbstractString)
     ispath(pkg,".git") || error("$pkg is not a git repo")
     isfile("METADATA",pkg,"url") && error("$pkg already registered")
     tags = split(Git.readall(`tag -l v*`, dir=pkg))
@@ -487,7 +522,7 @@ function register(pkg::String, url::String)
             open(io->println(io,url), path, "w")
             Git.run(`add $path`)
         end
-        vers = sort!([keys(versions)...])
+        vers = sort!(collect(keys(versions)))
         for ver in vers
             info("Tagging $pkg v$ver")
             write_tag_metadata(pkg,ver,versions[ver])
@@ -505,7 +540,7 @@ function register(pkg::String, url::String)
     end
 end
 
-function register(pkg::String)
+function register(pkg::AbstractString)
     Git.success(`config remote.origin.url`, dir=pkg) ||
         error("$pkg: no URL configured")
     url = Git.readchomp(`config remote.origin.url`, dir=pkg)
@@ -520,12 +555,12 @@ end
 
 nextbump(v::VersionNumber) = isrewritable(v) ? v : nextpatch(v)
 
-function tag(pkg::String, ver::Union(Symbol,VersionNumber), force::Bool=false, commit::String="HEAD")
+function tag(pkg::AbstractString, ver::Union{Symbol,VersionNumber}, force::Bool=false, commit::AbstractString="HEAD")
     ispath(pkg,".git") || error("$pkg is not a git repo")
     Git.dirty(dir=pkg) &&
-        error("$pkg is dirty – commit or stash changes to tag")
+        error("$pkg is dirty – commit or stash changes to tag")
     Git.dirty(pkg, dir="METADATA") &&
-        error("METADATA/$pkg is dirty – commit or stash changes to tag")
+        error("METADATA/$pkg is dirty – commit or stash changes to tag")
     commit = Git.readchomp(`rev-parse $commit`, dir=pkg)
     registered = isfile("METADATA",pkg,"url")
     if !force
@@ -611,20 +646,72 @@ function warnbanner(msg...; label="[ WARNING ]", prefix="")
     warn(prefix="", "="^cols)
 end
 
-function build!(pkgs::Vector, errs::Dict, seen::Set=Set())
+function build!(pkgs::Vector, buildstream::IO, seen::Set)
     for pkg in pkgs
+        pkg == "julia" && continue
         pkg in seen && continue
-        build!(Read.requires_list(pkg),errs,push!(seen,pkg))
+        build!(Read.requires_list(pkg),buildstream,push!(seen,pkg))
+        Read.isinstalled(pkg) || error("$pkg is not an installed package")
         path = abspath(pkg,"deps","build.jl")
         isfile(path) || continue
-        info("Building $pkg")
-        cd(dirname(path)) do
-            try evalfile(path)
-            catch err
-                warnbanner(err, label="[ ERROR: $pkg ]")
+        println(buildstream, path) # send to build process for evalfile
+        flush(buildstream)
+    end
+end
+
+function build!(pkgs::Vector, errs::Dict, seen::Set=Set())
+    # To isolate the build from the running Julia process, we
+    # execute the build.jl files in a separate process that
+    # is sitting there waiting for paths to evaluate.   Errors
+    # are serialized to errfile for later retrieval into errs[pkg]
+    errfile = tempname()
+    close(open(errfile, "w")) # create empty file
+    code = """
+        empty!(Base.LOAD_PATH)
+        append!(Base.LOAD_PATH, $(repr(Base.LOAD_PATH)))
+        empty!(Base.LOAD_CACHE_PATH)
+        append!(Base.LOAD_CACHE_PATH, $(repr(Base.LOAD_CACHE_PATH)))
+        empty!(Base.DL_LOAD_PATH)
+        append!(Base.DL_LOAD_PATH, $(repr(Base.DL_LOAD_PATH)))
+        open("$(escape_string(errfile))", "a") do f
+            for path_ in eachline(STDIN)
+                path = chomp(path_)
+                pkg = basename(dirname(dirname(path)))
+                try
+                    info("Building \$pkg")
+                    cd(dirname(path)) do
+                        evalfile(path)
+                    end
+                catch err
+                    Base.Pkg.Entry.warnbanner(err, label="[ ERROR: \$pkg ]")
+                    serialize(f, pkg)
+                    serialize(f, err)
+                end
+            end
+        end
+    """
+    io, pobj = open(detach(`$(Base.julia_cmd())
+                           --history-file=no
+                           --color=$(Base.have_color ? "yes" : "no")
+                           --eval $code`), "w", STDOUT)
+    try
+        build!(pkgs, io, seen)
+        close(io)
+        wait(pobj)
+        success(pobj) || error("Build process failed.")
+        open(errfile, "r") do f
+            while !eof(f)
+                pkg = deserialize(f)
+                err = deserialize(f)
                 errs[pkg] = err
             end
         end
+    catch
+        kill(pobj)
+        close(io)
+        rethrow()
+    finally
+        isfile(errfile) && Base.rm(errfile)
     end
 end
 
@@ -641,7 +728,7 @@ function build(pkgs::Vector)
      - build a single package by running its `deps/build.jl` script
     """)
 end
-build() = build(sort!([keys(installed())...]))
+build() = build(sort!(collect(keys(installed()))))
 
 function updatehook!(pkgs::Vector, errs::Dict, seen::Set=Set())
     for pkg in pkgs
@@ -673,7 +760,7 @@ function updatehook(pkgs::Vector)
     """)
 end
 
-function test!(pkg::String, errs::Vector{String}, notests::Vector{String}; coverage::Bool=false)
+function test!(pkg::AbstractString, errs::Vector{AbstractString}, notests::Vector{AbstractString}; coverage::Bool=false)
     reqs_path = abspath(pkg,"test","REQUIRE")
     if isfile(reqs_path)
         tests_require = Reqs.parse(reqs_path)
@@ -687,12 +774,10 @@ function test!(pkg::String, errs::Vector{String}, notests::Vector{String}; cover
         info("Testing $pkg")
         cd(dirname(test_path)) do
             try
-                if coverage
-                    cmd = `$JULIA_HOME/julia --code-coverage $test_path`
-                else
-                    cmd = `$JULIA_HOME/julia $test_path`
-                end
-                run(cmd)
+                color = Base.have_color? "--color=yes" : "--color=no"
+                codecov = coverage? ["--code-coverage=user", "--inline=no"] : ["--code-coverage=none"]
+                julia_exe = joinpath(JULIA_HOME, Base.julia_exename())
+                run(`$julia_exe --check-bounds=yes $codecov $color $test_path`)
                 info("$pkg tests passed")
             catch err
                 warnbanner(err, label="[ ERROR: $pkg ]")
@@ -702,23 +787,23 @@ function test!(pkg::String, errs::Vector{String}, notests::Vector{String}; cover
     else
         push!(notests,pkg)
     end
-    resolve()
+    isfile(reqs_path) && resolve()
 end
 
-function test(pkgs::Vector{String}; coverage::Bool=false)
-    errs = String[]
-    notests = String[]
+function test(pkgs::Vector{AbstractString}; coverage::Bool=false)
+    errs = AbstractString[]
+    notests = AbstractString[]
     for pkg in pkgs
         test!(pkg,errs,notests; coverage=coverage)
     end
     if !isempty(errs) || !isempty(notests)
-        messages = String[]
+        messages = AbstractString[]
         isempty(errs) || push!(messages, "$(join(errs,", "," and ")) had test errors")
         isempty(notests) || push!(messages, "$(join(notests,", "," and ")) did not provide a test/runtests.jl file")
         error(join(messages, "and"))
     end
 end
 
-test(;coverage::Bool=false) = test(sort!(String[keys(installed())...]); coverage=coverage)
+test(;coverage::Bool=false) = test(sort!(AbstractString[keys(installed())...]); coverage=coverage)
 
 end # module
