@@ -1,22 +1,24 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module Generate
 
 import ..Git, ..Read
 
-copyright_year() = readchomp(`date +%Y`)
-copyright_name(dir::String) = readchomp(Git.cmd(`config --get user.name`, dir=dir))
+copyright_year() = string(Dates.year(Dates.today()))
+copyright_name(dir::AbstractString) = readchomp(Git.cmd(`config --get user.name`, dir=dir))
 github_user() = readchomp(ignorestatus(`git config --global --get github.user`))
 
-function git_contributors(dir::String, n::Int=typemax(Int))
+function git_contributors(dir::AbstractString, n::Int=typemax(Int))
     contrib = Dict()
     tty = @windows? "CON:" : "/dev/tty"
-    for line in eachline(tty |> Git.cmd(`shortlog -nes`, dir=dir))
+    for line in eachline(pipeline(tty, Git.cmd(`shortlog -nes`, dir=dir)))
         m = match(r"\s*(\d+)\s+(.+?)\s+\<(.+?)\>\s*$", line)
-        m == nothing && continue
+        m === nothing && continue
         commits, name, email = m.captures
         if haskey(contrib,email)
-            contrib[email][1] += int(commits)
+            contrib[email][1] += parse(Int,commits)
         else
-            contrib[email] = [int(commits),name]
+            contrib[email] = [parse(Int,commits), name]
         end
     end
     names = Dict()
@@ -24,25 +26,25 @@ function git_contributors(dir::String, n::Int=typemax(Int))
         names[name] = get(names,name,0) + commits
     end
     names = sort!(collect(keys(names)),by=name->names[name],rev=true)
-    length(names) <= n ? names : [names[1:n], "et al."]
+    length(names) <= n ? names : [names[1:n]; "et al."]
 end
 
 function package(
-    pkg::String,
-    license::String;
+    pkg::AbstractString,
+    license::AbstractString;
     force::Bool = false,
-    authors::Union(String,Array) = "",
-    years::Union(Int,String) = copyright_year(),
-    user::String = github_user(),
-    config::Dict = {},
+    authors::Union{AbstractString,Array} = "",
+    years::Union{Int,AbstractString} = copyright_year(),
+    user::AbstractString = github_user(),
+    config::Dict = Dict(),
 )
     isnew = !ispath(pkg)
     try
         if isnew
-            url = isempty(user) ? "" : "git://github.com/$user/$pkg.jl.git"
+            url = isempty(user) ? "" : "https://github.com/$user/$pkg.jl.git"
             Generate.init(pkg,url,config=config)
         else
-            Git.dirty(dir=pkg) && error("$pkg is dirty – commit or stash your changes")
+            Git.dirty(dir=pkg) && error("$pkg is dirty – commit or stash your changes")
         end
 
         Git.transact(dir=pkg) do
@@ -53,14 +55,16 @@ function package(
             Generate.readme(pkg,user,force=force)
             Generate.entrypoint(pkg,force=force)
             Generate.tests(pkg,force=force)
+            Generate.require(pkg,force=force)
             Generate.travis(pkg,force=force)
+            Generate.appveyor(pkg,force=force)
             Generate.gitignore(pkg,force=force)
 
             msg = """
             $pkg.jl $(isnew ? "generated" : "regenerated") files.
 
                 license:  $license
-                authors:  $(join([authors],", "))
+                authors:  $(join(vcat(authors),", "))
                 years:    $years
                 user:     $user
 
@@ -84,7 +88,7 @@ function package(
     end
 end
 
-function init(pkg::String, url::String=""; config::Dict=Dict())
+function init(pkg::AbstractString, url::AbstractString=""; config::Dict=Dict())
     if !ispath(pkg)
         info("Initializing $pkg repo: $(abspath(pkg))")
         Git.run(`init -q $pkg`)
@@ -98,22 +102,24 @@ function init(pkg::String, url::String=""; config::Dict=Dict())
     info("Origin: $url")
     Git.run(`remote add origin $url`,dir=pkg)
     Git.set_remote_url(url,dir=pkg)
+    Git.run(`config branch.master.remote origin`, dir=pkg)
+    Git.run(`config branch.master.merge refs/heads/master`, dir=pkg)
 end
 
-function license(pkg::String, license::String,
-                 years::Union(Int,String),
-                 authors::Union(String,Array);
+function license(pkg::AbstractString, license::AbstractString,
+                 years::Union{Int,AbstractString},
+                 authors::Union{AbstractString,Array};
                  force::Bool=false)
     genfile(pkg,"LICENSE.md",force) do io
         if !haskey(LICENSES,license)
-            licenses = join(sort!([keys(LICENSES)...], by=lowercase), ", ")
+            licenses = join(sort!(collect(keys(LICENSES)), by=lowercase), ", ")
             error("$license is not a known license choice, choose one of: $licenses.")
         end
         print(io, LICENSES[license](pkg, string(years), authors))
     end || info("License file exists, leaving unmodified; use `force=true` to overwrite")
 end
 
-function readme(pkg::String, user::String=""; force::Bool=false)
+function readme(pkg::AbstractString, user::AbstractString=""; force::Bool=false)
     genfile(pkg,"README.md",force) do io
         println(io, "# $pkg")
         isempty(user) && return
@@ -122,7 +128,7 @@ function readme(pkg::String, user::String=""; force::Bool=false)
     end
 end
 
-function tests(pkg::String; force::Bool=false)
+function tests(pkg::AbstractString; force::Bool=false)
     genfile(pkg,"test/runtests.jl",force) do io
         print(io, """
         using $pkg
@@ -134,9 +140,31 @@ function tests(pkg::String; force::Bool=false)
     end
 end
 
-function travis(pkg::String; force::Bool=false)
+function versionfloor(ver::VersionNumber)
+    # return "major.minor" for the most recent release version relative to ver
+    # for prereleases with ver.minor == ver.patch == 0, return "major-" since we
+    # don't know what the most recent minor version is for the previous major
+    if isempty(ver.prerelease) || ver.patch > 0
+        return string(ver.major, '.', ver.minor)
+    elseif ver.minor > 0
+        return string(ver.major, '.', ver.minor - 1)
+    else
+        return string(ver.major, '-')
+    end
+end
+
+function require(pkg::AbstractString; force::Bool=false)
+    genfile(pkg,"REQUIRE",force) do io
+        print(io, """
+        julia $(versionfloor(VERSION))
+        """)
+    end
+end
+
+function travis(pkg::AbstractString; force::Bool=false)
     genfile(pkg,".travis.yml",force) do io
         print(io, """
+        # Documentation: http://docs.travis-ci.com/user/languages/julia/
         language: julia
         os:
           - linux
@@ -149,21 +177,72 @@ function travis(pkg::String; force::Bool=false)
         # uncomment the following lines to override the default test script
         #script:
         #  - if [[ -a .git/shallow ]]; then git fetch --unshallow; fi
-        #  - julia --check-bounds=yes -e 'Pkg.clone(pwd()); Pkg.build("$pkg"); Pkg.test("$pkg"; coverage=true)'
+        #  - julia -e 'Pkg.clone(pwd()); Pkg.build("$pkg"); Pkg.test("$pkg"; coverage=true)'
         """)
     end
 end
 
-function gitignore(pkg::String; force::Bool=false)
+function appveyor(pkg::AbstractString; force::Bool=false)
+    vf = versionfloor(VERSION)
+    if vf[end] == '-' # don't know what previous release was
+        vf = string(VERSION.major, '.', VERSION.minor)
+        rel32 = "#  - JULIAVERSION: \"julialang/bin/winnt/x86/$vf/julia-$vf-latest-win32.exe\""
+        rel64 = "#  - JULIAVERSION: \"julialang/bin/winnt/x64/$vf/julia-$vf-latest-win64.exe\""
+    else
+        rel32 = "  - JULIAVERSION: \"julialang/bin/winnt/x86/$vf/julia-$vf-latest-win32.exe\""
+        rel64 = "  - JULIAVERSION: \"julialang/bin/winnt/x64/$vf/julia-$vf-latest-win64.exe\""
+    end
+    genfile(pkg,"appveyor.yml",force) do io
+        print(io, """
+        environment:
+          matrix:
+        $rel32
+        $rel64
+          - JULIAVERSION: "julianightlies/bin/winnt/x86/julia-latest-win32.exe"
+          - JULIAVERSION: "julianightlies/bin/winnt/x64/julia-latest-win64.exe"
+
+        branches:
+          only:
+            - master
+            - /release-.*/
+
+        notifications:
+          - provider: Email
+            on_build_success: false
+            on_build_failure: false
+            on_build_status_changed: false
+
+        install:
+        # Download most recent Julia Windows binary
+          - ps: (new-object net.webclient).DownloadFile(
+                \$("http://s3.amazonaws.com/"+\$env:JULIAVERSION),
+                "C:\\projects\\julia-binary.exe")
+        # Run installer silently, output to C:\\projects\\julia
+          - C:\\projects\\julia-binary.exe /S /D=C:\\projects\\julia
+
+        build_script:
+        # Need to convert from shallow to complete for Pkg.clone to work
+          - IF EXIST .git\\shallow (git fetch --unshallow)
+          - C:\\projects\\julia\\bin\\julia -e "versioninfo();
+              Pkg.clone(pwd(), \\"$pkg\\"); Pkg.build(\\"$pkg\\")"
+
+        test_script:
+          - C:\\projects\\julia\\bin\\julia --check-bounds=yes -e "Pkg.test(\\"$pkg\\")"
+        """)
+    end
+end
+
+function gitignore(pkg::AbstractString; force::Bool=false)
     genfile(pkg,".gitignore",force) do io
         print(io, """
         *.jl.cov
+        *.jl.*.cov
         *.jl.mem
         """)
     end
 end
 
-function entrypoint(pkg::String; force::Bool=false)
+function entrypoint(pkg::AbstractString; force::Bool=false)
     genfile(pkg,"src/$pkg.jl",force) do io
         print(io, """
         module $pkg
@@ -175,7 +254,7 @@ function entrypoint(pkg::String; force::Bool=false)
     end
 end
 
-function genfile(f::Function, pkg::String, file::String, force::Bool=false)
+function genfile(f::Function, pkg::AbstractString, file::AbstractString, force::Bool=false)
     path = joinpath(pkg,file)
     if force || !ispath(path)
         info("Generating $file")
@@ -187,9 +266,9 @@ function genfile(f::Function, pkg::String, file::String, force::Bool=false)
     return false
 end
 
-copyright(years::String, authors::String) = "> Copyright (c) $years: $authors."
+copyright(years::AbstractString, authors::AbstractString) = "> Copyright (c) $years: $authors."
 
-function copyright(years::String, authors::Array)
+function copyright(years::AbstractString, authors::Array)
     text = "> Copyright (c) $years:"
     for author in authors
         text *= "\n>  * $author"
@@ -197,7 +276,7 @@ function copyright(years::String, authors::Array)
     return text
 end
 
-mit(pkg::String, years::String, authors::Union(String,Array)) =
+mit(pkg::AbstractString, years::AbstractString, authors::Union{AbstractString,Array}) =
 """
 The $pkg.jl package is licensed under the MIT "Expat" License:
 
@@ -223,7 +302,7 @@ $(copyright(years,authors))
 > SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 """
 
-bsd(pkg::String, years::String, authors::Union(String,Array)) =
+bsd(pkg::AbstractString, years::AbstractString, authors::Union{AbstractString,Array}) =
 """
 The $pkg.jl package is licensed under the Simplified "2-clause" BSD License:
 
@@ -252,7 +331,7 @@ $(copyright(years,authors))
 > OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 
-asl(pkg::String, years::String, authors::Union(String,Array)) =
+asl(pkg::AbstractString, years::AbstractString, authors::Union{AbstractString,Array}) =
 """
 The $pkg.jl package is licensed under version 2.0 of the Apache License:
 
@@ -434,6 +513,6 @@ $(copyright(years,authors))
 >      of your accepting any such warranty or additional liability.
 """
 
-const LICENSES = [ "MIT" => mit, "BSD" => bsd, "ASL" => asl ]
+const LICENSES = Dict("MIT" => mit, "BSD" => bsd, "ASL" => asl)
 
 end # module
