@@ -1,7 +1,9 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module Types
 
 export VersionInterval, VersionSet, Requires, Available, Fixed, merge_requires!, satisfies
-import Base: show, isempty, in, intersect, hash, deepcopy_internal
+import Base: show, isempty, in, intersect, ==, hash, deepcopy_internal
 
 immutable VersionInterval
     lower::VersionNumber
@@ -15,6 +17,7 @@ isempty(i::VersionInterval) = i.upper <= i.lower
 in(v::VersionNumber, i::VersionInterval) = i.lower <= v < i.upper
 intersect(a::VersionInterval, b::VersionInterval) = VersionInterval(max(a.lower,b.lower), min(a.upper,b.upper))
 ==(a::VersionInterval, b::VersionInterval) = a.lower == b.lower && a.upper == b.upper
+hash(i::VersionInterval, h::UInt) = hash((i.lower, i.upper), h + (0x0f870a92db508386 % UInt))
 
 immutable VersionSet
     intervals::Vector{VersionInterval}
@@ -44,7 +47,7 @@ function intersect(A::VersionSet, B::VersionSet)
     VersionSet(ivals)
 end
 ==(A::VersionSet, B::VersionSet) = A.intervals == B.intervals
-hash(s::VersionSet, h::Uint) = hash(s.intervals, h + uint(0x2fd2ca6efa023f44))
+hash(s::VersionSet, h::UInt) = hash(s.intervals, h + (0x2fd2ca6efa023f44 % UInt))
 deepcopy_internal(vs::VersionSet, ::ObjectIdDict) = VersionSet(copy(vs.intervals))
 
 typealias Requires Dict{ByteString,VersionSet}
@@ -56,7 +59,7 @@ function merge_requires!(A::Requires, B::Requires)
     return A
 end
 
-satisfies(pkg::String, ver::VersionNumber, reqs::Requires) =
+satisfies(pkg::AbstractString, ver::VersionNumber, reqs::Requires) =
     !haskey(reqs, pkg) || in(ver, reqs[pkg])
 
 immutable Available
@@ -65,6 +68,7 @@ immutable Available
 end
 
 ==(a::Available, b::Available) = a.sha1 == b.sha1 && a.requires == b.requires
+hash(a::Available, h::UInt) = hash((a.sha1, a.requires), h + (0xbc8ae0de9d11d972 % UInt))
 
 show(io::IO, a::Available) = isempty(a.requires) ?
     print(io, "Available(", repr(a.sha1), ")") :
@@ -77,12 +81,13 @@ end
 Fixed(v::VersionNumber) = Fixed(v,Requires())
 
 ==(a::Fixed, b::Fixed) = a.version == b.version && a.requires == b.requires
+hash(f::Fixed, h::UInt) = hash((f.version, f.requires), h + (0x68628b809fd417ca % UInt))
 
 show(io::IO, f::Fixed) = isempty(f.requires) ?
     print(io, "Fixed(", repr(f.version), ")") :
     print(io, "Fixed(", repr(f.version), ",", f.requires, ")")
 
-# TODO: Available & Fixed are almost the same – merge them?
+# TODO: Available & Fixed are almost the same – merge them?
 # Free could include the same information too, it just isn't
 # required by anything that processes these things.
 
