@@ -1,50 +1,33 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 ## reductions ##
 
-###### Functors ######
-
-# Note that functors are merely used as internal machinery to enhance code reuse.
-# They are not exported.
-# When function arguments can be inlined, the use of functors can be removed.
-
-abstract Func{N}
-
-type IdFun <: Func{1} end
-type AbsFun <: Func{1} end
-type Abs2Fun <: Func{1} end
-type ExpFun <: Func{1} end
-type LogFun <: Func{1} end
-
-type AddFun <: Func{2} end
-type MulFun <: Func{2} end
-type AndFun <: Func{2} end
-type OrFun <: Func{2} end
-type MaxFun <: Func{2} end
-type MinFun <: Func{2} end
-
-evaluate(::IdFun, x) = x
-evaluate(::AbsFun, x) = abs(x)
-evaluate(::Abs2Fun, x) = abs2(x)
-evaluate(::ExpFun, x) = exp(x)
-evaluate(::LogFun, x) = log(x)
-evaluate(f::Callable, x) = f(x)
-
-evaluate(::AddFun, x, y) = x + y
-evaluate(::MulFun, x, y) = x * y
-evaluate(::AndFun, x, y) = x & y
-evaluate(::OrFun, x, y) = x | y
-evaluate(::MaxFun, x, y) = scalarmax(x, y)
-evaluate(::MinFun, x, y) = scalarmin(x, y)
-evaluate(f::Callable, x, y) = f(x, y)
-
-
 ###### Generic (map)reduce functions ######
 
+if Int === Int32
+typealias SmallSigned Union{Int8,Int16}
+typealias SmallUnsigned Union{UInt8,UInt16}
+else
+typealias SmallSigned Union{Int8,Int16,Int32}
+typealias SmallUnsigned Union{UInt8,UInt16,UInt32}
+end
+
+typealias CommonReduceResult Union{UInt64,UInt128,Int64,Int128,Float32,Float64}
+typealias WidenReduceResult Union{SmallSigned, SmallUnsigned, Float16}
+
 # r_promote: promote x to the type of reduce(op, [x])
+r_promote(op, x::WidenReduceResult) = widen(x)
 r_promote(op, x) = x
-r_promote(::AddFun, x::Number) = x + zero(x)
-r_promote(::MulFun, x::Number) = x * one(x)
+r_promote(::AddFun, x::WidenReduceResult) = widen(x)
+r_promote(::MulFun, x::WidenReduceResult) = widen(x)
+r_promote(::AddFun, x::Number) = oftype(x + zero(x), x)
+r_promote(::MulFun, x::Number) = oftype(x * one(x), x)
 r_promote(::AddFun, x) = x
 r_promote(::MulFun, x) = x
+r_promote(::MaxFun, x::WidenReduceResult) = x
+r_promote(::MinFun, x::WidenReduceResult) = x
+r_promote(::MaxFun, x) = x
+r_promote(::MinFun, x) = x
 
 
 ## foldl && mapfoldl
@@ -53,13 +36,13 @@ function mapfoldl_impl(f, op, v0, itr, i)
     # Unroll the while loop once; if v0 is known, the call to op may
     # be evaluated at compile time
     if done(itr, i)
-        return v0
+        return r_promote(op, v0)
     else
         (x, i) = next(itr, i)
-        v = evaluate(op, v0, evaluate(f, x))
+        v = op(r_promote(op, v0), f(x))
         while !done(itr, i)
             (x, i) = next(itr, i)
-            v = evaluate(op, v, evaluate(f, x))
+            v = op(v, f(x))
         end
         return v
     end
@@ -67,13 +50,7 @@ end
 
 mapfoldl(f, op, v0, itr) = mapfoldl_impl(f, op, v0, itr, start(itr))
 
-function mapfoldl(f, op::Function, v0, itr)
-    is(op, +) ? mapfoldl(f, AddFun(), v0, itr) :
-    is(op, *) ? mapfoldl(f, MulFun(), v0, itr) :
-    is(op, &) ? mapfoldl(f, AndFun(), v0, itr) :
-    is(op, |) ? mapfoldl(f, OrFun(), v0, itr) :
-    mapfoldl_impl(f, op, v0, itr, start(itr))
-end
+mapfoldl(f, op::Function, v0, itr) = mapfoldl_impl(f, specialized_binary(op), v0, itr, start(itr))
 
 function mapfoldl(f, op, itr)
     i = start(itr)
@@ -81,7 +58,7 @@ function mapfoldl(f, op, itr)
         return Base.mr_empty(f, op, eltype(itr))
     end
     (x, i) = next(itr, i)
-    v0 = evaluate(f, x)
+    v0 = f(x)
     mapfoldl_impl(f, op, v0, itr, i)
 end
 
@@ -94,20 +71,20 @@ function mapfoldr_impl(f, op, v0, itr, i::Integer)
     # Unroll the while loop once; if v0 is known, the call to op may
     # be evaluated at compile time
     if i == 0
-        return v0
+        return r_promote(op, v0)
     else
         x = itr[i]
-        v = evaluate(op, evaluate(f, x), v0)
+        v  = op(f(x), r_promote(op, v0))
         while i > 1
             x = itr[i -= 1]
-            v = evaluate(op, evaluate(f, x), v)
+            v = op(f(x), v)
         end
         return v
     end
 end
 
 mapfoldr(f, op, v0, itr) = mapfoldr_impl(f, op, v0, itr, endof(itr))
-mapfoldr(f, op, itr) = (i = endof(itr); mapfoldr_impl(f, op, evaluate(f, itr[i]), itr, i-1))
+mapfoldr(f, op, itr) = (i = endof(itr); mapfoldr_impl(f, op, f(itr[i]), itr, i-1))
 
 foldr(op, v0, itr) = mapfoldr(IdFun(), op, v0, itr)
 foldr(op, itr) = mapfoldr(IdFun(), op, itr)
@@ -116,12 +93,12 @@ foldr(op, itr) = mapfoldr(IdFun(), op, itr)
 
 # mapreduce_***_impl require ifirst < ilast
 function mapreduce_seq_impl(f, op, A::AbstractArray, ifirst::Int, ilast::Int)
-    @inbounds fx1 = evaluate(f, A[ifirst])
-    @inbounds fx2 = evaluate(f, A[ifirst+=1])
-    @inbounds v = evaluate(op, fx1, fx2)
+    @inbounds fx1 = r_promote(op, f(A[ifirst]))
+    @inbounds fx2 = f(A[ifirst+=1])
+    @inbounds v = op(fx1, fx2)
     while ifirst < ilast
-        @inbounds fx = evaluate(f, A[ifirst+=1])
-        v = evaluate(op, v, fx)
+        @inbounds fx = f(A[ifirst+=1])
+        v = op(v, fx)
     end
     return v
 end
@@ -133,17 +110,17 @@ function mapreduce_pairwise_impl(f, op, A::AbstractArray, ifirst::Int, ilast::In
         imid = (ifirst + ilast) >>> 1
         v1 = mapreduce_pairwise_impl(f, op, A, ifirst, imid, blksize)
         v2 = mapreduce_pairwise_impl(f, op, A, imid+1, ilast, blksize)
-        return evaluate(op, v1, v2)
+        return op(v1, v2)
     end
 end
 
 mapreduce(f, op, itr) = mapfoldl(f, op, itr)
 mapreduce(f, op, v0, itr) = mapfoldl(f, op, v0, itr)
 mapreduce_impl(f, op, A::AbstractArray, ifirst::Int, ilast::Int) =
-    mapreduce_seq_impl(f, op, A, ifirst, ilast)
+    mapreduce_pairwise_impl(f, op, A, ifirst, ilast, 1024)
 
 # handling empty arrays
-mr_empty(f, op, T) = error("Reducing over an empty array is not allowed.")
+mr_empty(f, op, T) = throw(ArgumentError("reducing over an empty collection is not allowed"))
 # use zero(T)::T to improve type information when zero(T) is not defined
 mr_empty(::IdFun, op::AddFun, T) = r_promote(op, zero(T)::T)
 mr_empty(::AbsFun, op::AddFun, T) = r_promote(op, abs(zero(T)::T))
@@ -154,20 +131,22 @@ mr_empty(::Abs2Fun, op::MaxFun, T) = abs2(zero(T)::T)
 mr_empty(f, op::AndFun, T) = true
 mr_empty(f, op::OrFun, T) = false
 
-function _mapreduce{T}(f, op, A::AbstractArray{T})
-    n = int(length(A))
+_mapreduce(f, op, A::AbstractArray) = _mapreduce(f, op, linearindexing(A), A)
+
+function _mapreduce{T}(f, op, ::LinearFast, A::AbstractArray{T})
+    n = Int(length(A))
     if n == 0
         return mr_empty(f, op, T)
     elseif n == 1
-        return r_promote(op, evaluate(f, A[1]))
+        return r_promote(op, f(A[1]))
     elseif n < 16
-        @inbounds fx1 = evaluate(f, A[1])
-        @inbounds fx2 = evaluate(f, A[2])
-        s = evaluate(op, fx1, fx2)
+        @inbounds fx1 = r_promote(op, f(A[1]))
+        @inbounds fx2 = r_promote(op, f(A[2]))
+        s = op(fx1, fx2)
         i = 2
         while i < n
-            @inbounds fx = evaluate(f, A[i+=1])
-            s = evaluate(op, s, fx)
+            @inbounds fx = f(A[i+=1])
+            s = op(s, fx)
         end
         return s
     else
@@ -175,59 +154,87 @@ function _mapreduce{T}(f, op, A::AbstractArray{T})
     end
 end
 
-mapreduce(f, op, A::AbstractArray) = _mapreduce(f, op, A)
-mapreduce(f, op, a::Number) = evaluate(f, a)
+_mapreduce{T}(f, op, ::LinearSlow, A::AbstractArray{T}) = mapfoldl(f, op, A)
 
-function mapreduce(f, op::Function, A::AbstractArray)
-    is(op, +) ? _mapreduce(f, AddFun(), A) :
-    is(op, *) ? _mapreduce(f, MulFun(), A) :
-    is(op, &) ? _mapreduce(f, AndFun(), A) :
-    is(op, |) ? _mapreduce(f, OrFun(), A) :
-    _mapreduce(f, op, A)
-end
+mapreduce(f, op, A::AbstractArray) = _mapreduce(f, op, linearindexing(A), A)
+mapreduce(f, op, a::Number) = f(a)
+
+mapreduce(f, op::Function, A::AbstractArray) = mapreduce(f, specialized_binary(op), A)
 
 reduce(op, v0, itr) = mapreduce(IdFun(), op, v0, itr)
 reduce(op, itr) = mapreduce(IdFun(), op, itr)
 reduce(op, a::Number) = a
 
+### short-circuiting specializations of mapreduce
+
+## conditions and results of short-circuiting
+
+const ShortCircuiting = Union{AndFun, OrFun}
+const ReturnsBool     = Union{EqX, Predicate}
+
+shortcircuits(::AndFun, x::Bool) = !x
+shortcircuits(::OrFun,  x::Bool) =  x
+
+shorted(::AndFun) = false
+shorted(::OrFun)  = true
+
+sc_finish(::AndFun) = true
+sc_finish(::OrFun)  = false
+
+## short-circuiting (sc) mapreduce definitions
+
+function mapreduce_sc_impl(f, op, itr::AbstractArray)
+    @inbounds for x in itr
+        shortcircuits(op, f(x)) && return shorted(op)
+    end
+    return sc_finish(op)
+end
+
+function mapreduce_sc_impl(f, op, itr)
+    for x in itr
+        shortcircuits(op, f(x)) && return shorted(op)
+    end
+    return sc_finish(op)
+end
+
+# mapreduce_sc tests if short-circuiting is safe;
+# if so, mapreduce_sc_impl is called. If it's not
+# safe, call mapreduce_no_sc, which redirects to
+# non-short-circuiting definitions.
+
+mapreduce_no_sc(f, op, itr::Any)           =  mapfoldl(f, op, itr)
+mapreduce_no_sc(f, op, itr::AbstractArray) = _mapreduce(f, op, itr)
+
+mapreduce_sc(f::Function,    op, itr) = mapreduce_sc(specialized_unary(f), op, itr)
+mapreduce_sc(f::ReturnsBool, op, itr) = mapreduce_sc_impl(f, op, itr)
+mapreduce_sc(f::Func{1},     op, itr) = mapreduce_no_sc(f, op, itr)
+
+mapreduce_sc(f::IdFun, op, itr) =
+    eltype(itr) <: Bool ?
+        mapreduce_sc_impl(f, op, itr) :
+        mapreduce_no_sc(f, op, itr)
+
+mapreduce(f, op::ShortCircuiting, n::Number) = n
+mapreduce(f, op::ShortCircuiting, itr::AbstractArray) = mapreduce_sc(f,op,itr)
+mapreduce(f, op::ShortCircuiting, itr::Any)           = mapreduce_sc(f,op,itr)
+
 
 ###### Specific reduction functions ######
 
 ## sum
 
 function mapreduce_seq_impl(f, op::AddFun, a::AbstractArray, ifirst::Int, ilast::Int)
-    @inbounds if ifirst + 6 >= ilast  # length(a) < 8
-        i = ifirst
-        s = evaluate(f, a[i]) + evaluate(f, a[i+1])
-        i = i+1
-        while i < ilast
-            s += evaluate(f, a[i+=1])
+    @inbounds begin
+        s = r_promote(op, f(a[ifirst])) + f(a[ifirst+1])
+        @simd for i = ifirst+2:ilast
+            s += f(a[i])
         end
-        return s
-
-    else # length(a) >= 8, manual unrolling
-        s1 = evaluate(f, a[ifirst]) + evaluate(f, a[ifirst + 4])
-        s2 = evaluate(f, a[ifirst + 1]) + evaluate(f, a[ifirst + 5])
-        s3 = evaluate(f, a[ifirst + 2]) + evaluate(f, a[ifirst + 6])
-        s4 = evaluate(f, a[ifirst + 3]) + evaluate(f, a[ifirst + 7])
-        i = ifirst + 8
-        il = ilast - 3
-        while i <= il
-            s1 += evaluate(f, a[i])
-            s2 += evaluate(f, a[i+1])
-            s3 += evaluate(f, a[i+2])
-            s4 += evaluate(f, a[i+3])
-            i += 4
-        end
-        while i <= ilast
-            s1 += evaluate(f, a[i])
-            i += 1
-        end
-        return s1 + s2 + s3 + s4
     end
+    s
 end
 
-# Note: sum_seq uses four accumulators, so each accumulator gets at most 256 numbers
+# Note: sum_seq usually uses four or more accumulators after partial
+# unrolling, so each accumulator gets at most 256 numbers
 sum_pairwise_blocksize(f) = 1024
 
 # This appears to show a benefit from a larger block size
@@ -236,7 +243,7 @@ sum_pairwise_blocksize(::Abs2Fun) = 4096
 mapreduce_impl(f, op::AddFun, A::AbstractArray, ifirst::Int, ilast::Int) =
     mapreduce_pairwise_impl(f, op, A, ifirst, ilast, sum_pairwise_blocksize(f))
 
-sum(f::Union(Callable,Func{1}), a) = mapreduce(f, AddFun(), a)
+sum(f::Union{Callable,Func{1}}, a) = mapreduce(f, AddFun(), a)
 sum(a) = mapreduce(IdFun(), AddFun(), a)
 sum(a::AbstractArray{Bool}) = countnz(a)
 sumabs(a) = mapreduce(AbsFun(), AddFun(), a)
@@ -244,7 +251,7 @@ sumabs2(a) = mapreduce(Abs2Fun(), AddFun(), a)
 
 # Kahan (compensated) summation: O(1) error growth, at the expense
 # of a considerable increase in computational expense.
-function sum_kbn{T<:FloatingPoint}(A::AbstractArray{T})
+function sum_kbn{T<:AbstractFloat}(A::AbstractArray{T})
     n = length(A)
     c = r_promote(AddFun(), zero(T)::T)
     if n == 0
@@ -267,7 +274,7 @@ end
 
 ## prod
 
-prod(f::Union(Callable,Func{1}), a) = mapreduce(f, MulFun(), a)
+prod(f::Union{Callable,Func{1}}, a) = mapreduce(f, MulFun(), a)
 prod(a) = mapreduce(IdFun(), MulFun(), a)
 
 prod(A::AbstractArray{Bool}) =
@@ -277,14 +284,14 @@ prod(A::AbstractArray{Bool}) =
 
 function mapreduce_impl(f, op::MaxFun, A::AbstractArray, first::Int, last::Int)
     # locate the first non NaN number
-    v = evaluate(f, A[first])
+    v = f(A[first])
     i = first + 1
     while v != v && i <= last
-        @inbounds v = evaluate(f, A[i])
+        @inbounds v = f(A[i])
         i += 1
     end
     while i <= last
-        @inbounds x = evaluate(f, A[i])
+        @inbounds x = f(A[i])
         if x > v
             v = x
         end
@@ -295,14 +302,14 @@ end
 
 function mapreduce_impl(f, op::MinFun, A::AbstractArray, first::Int, last::Int)
     # locate the first non NaN number
-    v = evaluate(f, A[first])
+    v = f(A[first])
     i = first + 1
     while v != v && i <= last
-        @inbounds v = evaluate(f, A[i])
+        @inbounds v = f(A[i])
         i += 1
     end
     while i <= last
-        @inbounds x = evaluate(f, A[i])
+        @inbounds x = f(A[i])
         if x < v
             v = x
         end
@@ -311,8 +318,8 @@ function mapreduce_impl(f, op::MinFun, A::AbstractArray, first::Int, last::Int)
     v
 end
 
-maximum(f::Union(Callable,Func{1}), a) = mapreduce(f, MaxFun(), a)
-minimum(f::Union(Callable,Func{1}), a) = mapreduce(f, MinFun(), a)
+maximum(f::Union{Callable,Func{1}}, a) = mapreduce(f, MaxFun(), a)
+minimum(f::Union{Callable,Func{1}}, a) = mapreduce(f, MinFun(), a)
 
 maximum(a) = mapreduce(IdFun(), MaxFun(), a)
 minimum(a) = mapreduce(IdFun(), MinFun(), a)
@@ -327,7 +334,7 @@ extrema(x::Real) = (x, x)
 
 function extrema(itr)
     s = start(itr)
-    done(itr, s) && error("argument is empty")
+    done(itr, s) && throw(ArgumentError("collection must be non-empty"))
     (v, s) = next(itr, s)
     while v != v && !done(itr, s)
         (x, s) = next(itr, s)
@@ -348,61 +355,25 @@ end
 
 ## all & any
 
-function mapfoldl(f, ::AndFun, itr)
-    for x in itr
-        if !evaluate(f, x)
-            return false
-        end
-    end
-    return true
-end
+any(itr) = any(IdFun(), itr)
+all(itr) = all(IdFun(), itr)
 
-function mapfoldl(f, ::OrFun, itr)
-    for x in itr
-        if evaluate(f, x)
-            return true
-        end
-    end
-    return false
-end
-
-function mapreduce_impl(f, op::AndFun, A::AbstractArray{Bool}, ifirst::Int, ilast::Int)
-    while ifirst <= ilast
-        @inbounds x = A[ifirst]
-        if !evaluate(f, x)
-            return false
-        end
-        ifirst += 1
-    end
-    return true
-end
-
-function mapreduce_impl(f, op::OrFun, A::AbstractArray{Bool}, ifirst::Int, ilast::Int)
-    while ifirst <= ilast
-        @inbounds x = A[ifirst]
-        if evaluate(f, x)
-            return true
-        end
-        ifirst += 1
-    end
-    return false
-end
-
-all(a) = mapreduce(IdFun(), AndFun(), a)
-any(a) = mapreduce(IdFun(), OrFun(), a)
-
-all(pred::Union(Callable,Func{1}), a) = mapreduce(pred, AndFun(), a)
-any(pred::Union(Callable,Func{1}), a) = mapreduce(pred, OrFun(), a)
+any(f::Any,       itr) = any(Predicate(f), itr)
+any(f::Predicate, itr) = mapreduce_sc_impl(f, OrFun(), itr)
+any(f::IdFun,     itr) =
+    eltype(itr) <: Bool ?
+        mapreduce_sc_impl(f, OrFun(), itr) :
+        nonboolean_any(itr)
 
+all(f::Any,       itr) = all(Predicate(f), itr)
+all(f::Predicate, itr) = mapreduce_sc_impl(f, AndFun(), itr)
+all(f::IdFun,     itr) =
+    eltype(itr) <: Bool ?
+        mapreduce_sc_impl(f, AndFun(), itr) :
+        nonboolean_all(itr)
 
 ## in & contains
 
-immutable EqX{T} <: Func{1}
-    x::T
-end
-EqX{T}(x::T) = EqX{T}(x)
-evaluate(f::EqX, y) = (y == f.x)
-
 in(x, itr) = any(EqX(x), itr)
 
 const ∈ = in
@@ -412,9 +383,7 @@ const ∈ = in
 
 function contains(eq::Function, itr, x)
     for y in itr
-        if eq(y, x)
-            return true
-        end
+        eq(y, x) && return true
     end
     return false
 end
@@ -422,28 +391,29 @@ end
 
 ## countnz & count
 
-function count(pred::Union(Function,Func{1}), itr)
+function count(pred, itr)
     n = 0
     for x in itr
-        if evaluate(pred, x)
-            n += 1
-        end
+        pred(x) && (n += 1)
     end
     return n
 end
 
-function count(pred::Union(Function,Func{1}), a::AbstractArray)
+function count(pred, A::AbstractArray)
     n = 0
-    for i = 1:length(a)
-        @inbounds if evaluate(pred, a[i])
-            n += 1
-        end
+    @inbounds for a in A
+        pred(a) && (n += 1)
     end
     return n
 end
 
-type NotEqZero <: Func{1} end
-evaluate(NotEqZero, x) = (x != 0)
+immutable NotEqZero <: Func{1} end
+call(::NotEqZero, x) = x != 0
 
+"""
+    countnz(A)
+
+Counts the number of nonzero values in array `A` (dense or sparse). Note that this is not a constant-time operation.
+For sparse matrices, one should usually use `nnz`, which returns the number of stored values.
+"""
 countnz(a) = count(NotEqZero(), a)
-
