@@ -1,8 +1,15 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 abstract AbstractSparseArray{Tv,Ti,N} <: AbstractArray{Tv,N}
 
 typealias AbstractSparseVector{Tv,Ti} AbstractSparseArray{Tv,Ti,1}
 typealias AbstractSparseMatrix{Tv,Ti} AbstractSparseArray{Tv,Ti,2}
 
+"""
+    issparse(S)
+
+Returns `true` if `S` is sparse, and `false` otherwise.
+"""
 issparse(A::AbstractArray) = false
 issparse(S::AbstractSparseArray) = true
 
