@@ -1,19 +1,18 @@
-@unix_only if WORD_SIZE == 64
-# TODO: complex return only on 64-bit for now
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 for f in (:erf, :erfc, :erfcx, :erfi, :Dawson)
     fname = (f === :Dawson) ? :dawson : f
     @eval begin
-        ($fname)(z::Complex128) = complex128(ccall(($(string("Faddeeva_",f)),openspecfun), Complex{Float64}, (Complex{Float64}, Float64), z, zero(Float64)))
-        ($fname)(z::Complex64) = complex64(ccall(($(string("Faddeeva_",f)),openspecfun), Complex{Float64}, (Complex{Float64}, Float64), complex128(z), float64(eps(Float32))))
-        ($fname)(z::Complex) = ($fname)(complex128(z))
+        ($fname)(z::Complex128) = Complex128(ccall(($(string("Faddeeva_",f)),openspecfun), Complex{Float64}, (Complex{Float64}, Float64), z, zero(Float64)))
+        ($fname)(z::Complex64) = Complex64(ccall(($(string("Faddeeva_",f)),openspecfun), Complex{Float64}, (Complex{Float64}, Float64), Complex128(z), Float64(eps(Float32))))
+        ($fname)(z::Complex) = ($fname)(Complex128(z))
     end
 end
-end
 for f in (:erfcx, :erfi, :Dawson)
     fname = (f === :Dawson) ? :dawson : f
     @eval begin
         ($fname)(x::Float64) = ccall(($(string("Faddeeva_",f,"_re")),openspecfun), Float64, (Float64,), x)
-        ($fname)(x::Float32) = float32(ccall(($(string("Faddeeva_",f,"_re")),openspecfun), Float64, (Float64,), float64(x)))
+        ($fname)(x::Float32) = Float32(ccall(($(string("Faddeeva_",f,"_re")),openspecfun), Float64, (Float64,), Float64(x)))
         ($fname)(x::Integer) = ($fname)(float(x))
         @vectorize_1arg Number $fname
     end
@@ -30,9 +29,9 @@ function erfinv(x::Float64)
     a = abs(x)
     if a >= 1.0
         if x == 1.0
-            return inf(Float64)
+            return Inf
         elseif x == -1.0
-            return -inf(Float64)
+            return -Inf
         end
         throw(DomainError())
     elseif a <= 0.75 # Table 17 in Blair et al.
@@ -98,9 +97,9 @@ function erfinv(x::Float32)
     a = abs(x)
     if a >= 1.0f0
         if x == 1.0f0
-            return inf(Float32)
+            return Inf32
         elseif x == -1.0f0
-            return -inf(Float32)
+            return -Inf32
         end
         throw(DomainError())
     elseif a <= 0.75f0 # Table 10 in Blair et al.
@@ -147,7 +146,7 @@ function erfcinv(y::Float64)
         return erfinv(1.0 - y)
     elseif y <= 0.0
         if y == 0.0
-            return inf(Float64)
+            return Inf
         end
         throw(DomainError())
     elseif y >= 1e-100 # Table 57 in Blair et al.
@@ -199,7 +198,7 @@ function erfcinv(y::Float32)
         return erfinv(1.0f0 - y)
     elseif y <= 0.0f0
         if y == 0.0f0
-            return inf(Float32)
+            return Inf32
         end
         throw(DomainError())
     else # Table 50 in Blair et al.
