@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 gamma(x::Float64) = nan_dom_err(ccall((:tgamma,libm),  Float64, (Float64,), x), x)
 gamma(x::Float32) = nan_dom_err(ccall((:tgammaf,libm),  Float32, (Float32,), x), x)
 gamma(x::Real) = gamma(float(x))
@@ -65,7 +67,7 @@ gamma(z::Complex) = exp(lgamma(z))
 #   const A002445 = [1,6,30,42,30,66,2730,6,510,798,330,138,2730,6,870,14322,510,6,1919190,6,13530]
 #   const bernoulli = A000367 .// A002445 # even-index Bernoulli numbers
 
-function digamma(z::Union(Float64,Complex{Float64}))
+function digamma(z::Union{Float64,Complex{Float64}})
     # Based on eq. (12), without looking at the accompanying source
     # code, of: K. S. Kölbig, "Programs for computing the logarithm of
     # the gamma function, and the digamma function, for complex
@@ -80,7 +82,7 @@ function digamma(z::Union(Float64,Complex{Float64}))
     end
     if x < 7
         # shift using recurrence formula
-        n = 7 - ifloor(x)
+        n = 7 - floor(Int,x)
         for ν = 1:n-1
             ψ -= inv(z + ν)
         end
@@ -90,11 +92,11 @@ function digamma(z::Union(Float64,Complex{Float64}))
     t = inv(z)
     ψ += log(z) - 0.5*t
     t *= t # 1/z^2
-    # the coefficients here are float64(bernoulli[2:9] .// (2*(1:8)))
+    # the coefficients here are Float64(bernoulli[2:9] .// (2*(1:8)))
     ψ -= t * @evalpoly(t,0.08333333333333333,-0.008333333333333333,0.003968253968253968,-0.004166666666666667,0.007575757575757576,-0.021092796092796094,0.08333333333333333,-0.4432598039215686)
 end
 
-function trigamma(z::Union(Float64,Complex{Float64}))
+function trigamma(z::Union{Float64,Complex{Float64}})
     # via the derivative of the Kölbig digamma formulation
     x = real(z)
     if x <= 0 # reflection formula
@@ -103,7 +105,7 @@ function trigamma(z::Union(Float64,Complex{Float64}))
     ψ = zero(z)
     if x < 8
         # shift using recurrence formula
-        n = 8 - ifloor(x)
+        n = 8 - floor(Int,x)
         ψ += inv(z)^2
         for ν = 1:n-1
             ψ += inv(z + ν)^2
@@ -113,7 +115,7 @@ function trigamma(z::Union(Float64,Complex{Float64}))
     t = inv(z)
     w = t * t # 1/z^2
     ψ += t + 0.5*w
-    # the coefficients here are float64(bernoulli[2:9])
+    # the coefficients here are Float64(bernoulli[2:9])
     ψ += t*w * @evalpoly(w,0.16666666666666666,-0.03333333333333333,0.023809523809523808,-0.03333333333333333,0.07575757575757576,-0.2531135531135531,1.1666666666666667,-7.092156862745098)
 end
 
@@ -235,8 +237,8 @@ inv_oftype(x::Real, y::Real) = oftype(x, inv(y))
 # the zeta function for free and might as well export it, especially
 # since this is a common generalization of the Riemann zeta function
 # (which Julia already exports).
-function zeta(s::Union(Int,Float64,Complex{Float64}),
-              z::Union(Float64,Complex{Float64}))
+function zeta(s::Union{Int,Float64,Complex{Float64}},
+              z::Union{Float64,Complex{Float64}})
     ζ = zero(promote_type(typeof(s), typeof(z)))
 
     # like sqrt, require complex inputs to get complex outputs
@@ -284,8 +286,8 @@ function zeta(s::Union(Int,Float64,Complex{Float64}),
             end
             throw(DomainError()) # or return NaN?
         end
-        nx = int(xf)
-        n = iceil(cutoff - nx)
+        nx = Int(xf)
+        n = ceil(Int,cutoff - nx)
         ζ += inv_oftype(ζ, z)^s
         for ν = -nx:-1:1
             ζₒ= ζ
@@ -311,7 +313,7 @@ function zeta(s::Union(Int,Float64,Complex{Float64}),
     return ζ
 end
 
-function polygamma(m::Integer, z::Union(Float64,Complex{Float64}))
+function polygamma(m::Integer, z::Union{Float64,Complex{Float64}})
 
     m == 0 && return digamma(z)
     m == 1 && return trigamma(z)
@@ -338,34 +340,36 @@ function polygamma(m::Integer, z::Union(Float64,Complex{Float64}))
     end
 end
 
+# TODO: better way to do this
+f64(x::Real) = Float64(x)
+f64(z::Complex) = Complex128(z)
+f32(x::Real) = Float32(x)
+f32(z::Complex) = Complex64(z)
+f16(x::Real) = Float16(x)
+f16(z::Complex) = Complex32(z)
+
 # If we really cared about single precision, we could make a faster
 # Float32 version by truncating the Stirling series at a smaller cutoff.
-for (f,T) in ((float32,Float32), (float16,Float16))
+for (f,T) in ((:f32,Float32),(:f16,Float16))
     @eval begin
-        zeta(s::Integer, z::Union($T,Complex{$T})) =
-            $f(zeta(int(s), float64(z)))
-        zeta(s::Union(Float64,Complex{Float64}), z::Union($T,Complex{$T})) =
-            zeta(s, float64(z))
-        zeta(s::Number, z::Union($T,Complex{$T})) =
-            $f(zeta(float64(s), float64(z)))
-        polygamma(m::Integer, z::Union($T,Complex{$T})) =
-            $f(polygamma(int(m), float64(z)))
-        digamma(z::Union($T,Complex{$T})) =
-            $f(digamma(float64(z)))
-        trigamma(z::Union($T,Complex{$T})) =
-            $f(trigamma(float64(z)))
+        zeta(s::Integer, z::Union{$T,Complex{$T}}) = $f(zeta(Int(s), f64(z)))
+        zeta(s::Union{Float64,Complex128}, z::Union{$T,Complex{$T}}) = zeta(s, f64(z))
+        zeta(s::Number, z::Union{$T,Complex{$T}}) = $f(zeta(f64(s), f64(z)))
+        polygamma(m::Integer, z::Union{$T,Complex{$T}}) = $f(polygamma(Int(m), f64(z)))
+        digamma(z::Union{$T,Complex{$T}}) = $f(digamma(f64(z)))
+        trigamma(z::Union{$T,Complex{$T}}) = $f(trigamma(f64(z)))
     end
 end
 
-zeta(s::Integer, z::Number) = zeta(int(s), float64(z))
-zeta(s::Number, z::Number) = zeta(float64(s), float64(z))
+zeta(s::Integer, z::Number) = zeta(Int(s), f64(z))
+zeta(s::Number, z::Number) = zeta(f64(s), f64(z))
 for f in (:digamma, :trigamma)
     @eval begin
-        $f(z::Number) = $f(float64(z))
+        $f(z::Number) = $f(f64(z))
         @vectorize_1arg Number $f
     end
 end
-polygamma(m::Integer, z::Number) = polygamma(m, float64(z))
+polygamma(m::Integer, z::Number) = polygamma(m, f64(z))
 @vectorize_2arg Number polygamma
 @vectorize_2arg Number zeta
 
@@ -394,8 +398,8 @@ function invdigamma(y::Float64)
 
     return x_new
 end
-invdigamma(x::Float32) = float32(invdigamma(float64(x)))
-invdigamma(x::Real) = invdigamma(float64(x))
+invdigamma(x::Float32) = Float32(invdigamma(Float64(x)))
+invdigamma(x::Real) = invdigamma(Float64(x))
 @vectorize_1arg Real invdigamma
 
 function beta(x::Number, w::Number)
@@ -410,7 +414,7 @@ lbeta(x::Number, w::Number) = lgamma(x)+lgamma(w)-lgamma(x+w)
 
 # Riemann zeta function; algorithm is based on specializing the Hurwitz
 # zeta function above for z==1.
-function zeta(s::Union(Float64,Complex{Float64}))
+function zeta(s::Union{Float64,Complex{Float64}})
     # blows up to ±Inf, but get correct sign of imaginary zero
     s == 1 && return NaN + zero(s) * imag(s)
 
@@ -437,7 +441,7 @@ function zeta(s::Union(Float64,Complex{Float64}))
     # shift using recurrence formula:
     #   n is a semi-empirical cutoff for the Stirling series, based
     #   on the error term ~ (|m|/n)^18 / n^real(m)
-    n = iceil(6 + 0.7*abs(imag(s-1))^inv(1 + real(m)*0.05))
+    n = ceil(Int,6 + 0.7*abs(imag(s-1))^inv(1 + real(m)*0.05))
     ζ = one(s)
     for ν = 2:n
         ζₒ= ζ
@@ -455,12 +459,12 @@ function zeta(s::Union(Float64,Complex{Float64}))
     return ζ
 end
 
-zeta(x::Integer) = zeta(float64(x))
-zeta(x::Real)    = oftype(float(x),zeta(float64(x)))
-zeta(z::Complex) = oftype(float(z),zeta(complex128(z)))
+zeta(x::Integer) = zeta(Float64(x))
+zeta(x::Real)    = oftype(float(x),zeta(Float64(x)))
+zeta(z::Complex) = oftype(float(z),zeta(Complex128(z)))
 @vectorize_1arg Number zeta
 
-function eta(z::Union(Float64,Complex{Float64}))
+function eta(z::Union{Float64,Complex{Float64}})
     δz = 1 - z
     if abs(real(δz)) + abs(imag(δz)) < 7e-3 # Taylor expand around z==1
         return 0.6931471805599453094172321214581765 *
@@ -474,7 +478,7 @@ function eta(z::Union(Float64,Complex{Float64}))
         return -zeta(z) * expm1(0.6931471805599453094172321214581765*δz)
     end
 end
-eta(x::Integer) = eta(float64(x))
-eta(x::Real)    = oftype(float(x),eta(float64(x)))
-eta(z::Complex) = oftype(float(z),eta(complex128(z)))
+eta(x::Integer) = eta(Float64(x))
+eta(x::Real)    = oftype(float(x),eta(Float64(x)))
+eta(z::Complex) = oftype(float(z),eta(Complex128(z)))
 @vectorize_1arg Number eta
