@@ -5,11 +5,15 @@ extern size_t jl_cholmod_common_size(void) {
     return sizeof(cholmod_common);
 }
 
+extern size_t jl_cholmod_sizeof_long(void) {
+    return sizeof(SuiteSparse_long);
+}
+
 extern int jl_cholmod_version(int *ver) {
     if (ver != (int*) NULL) {
-	ver[0] = CHOLMOD_MAIN_VERSION;
-	ver[1] = CHOLMOD_SUB_VERSION;
-	ver[2] = CHOLMOD_SUBSUB_VERSION;
+        ver[0] = CHOLMOD_MAIN_VERSION;
+        ver[1] = CHOLMOD_SUB_VERSION;
+        ver[2] = CHOLMOD_SUBSUB_VERSION;
     }
     return CHOLMOD_VERSION;
 }
