@@ -644,8 +644,8 @@ static void uv__drain(uv_stream_t* stream) {
     uv__req_unregister(stream->loop, req);
 
     err = 0;
-    if (shutdown(uv__stream_fd(stream), SHUT_WR))
-      err = -errno;
+    //if (shutdown(uv__stream_fd(stream), SHUT_WR))
+    //  err = -errno;
 
     if (err == 0)
       stream->flags |= UV_STREAM_SHUT;
