@@ -129,9 +129,7 @@ static void init_once(void) {
 }
 
 
-#if defined(__GNUC__)
-__attribute__((destructor))
-static void cleanup(void) {
+UV_DESTRUCTOR(static void cleanup(void)) {
   unsigned int i;
 
   if (initialized == 0)
@@ -153,7 +151,6 @@ static void cleanup(void) {
   nthreads = 0;
   initialized = 0;
 }
-#endif
 
 
 void uv__work_submit(uv_loop_t* loop,
@@ -194,7 +191,7 @@ static int uv__work_cancel(uv_loop_t* loop, uv_req_t* req, struct uv__work* w) {
 }
 
 
-void uv__work_done(uv_async_t* handle, int status) {
+void uv__work_done(uv_async_t* handle) {
   struct uv__work* w;
   uv_loop_t* loop;
   QUEUE* q;
@@ -271,6 +268,10 @@ int uv_cancel(uv_req_t* req) {
     loop =  ((uv_getaddrinfo_t*) req)->loop;
     wreq = &((uv_getaddrinfo_t*) req)->work_req;
     break;
+  case UV_GETNAMEINFO:
+    loop = ((uv_getnameinfo_t*) req)->loop;
+    wreq = &((uv_getnameinfo_t*) req)->work_req;
+    break;
   case UV_WORK:
     loop =  ((uv_work_t*) req)->loop;
     wreq = &((uv_work_t*) req)->work_req;
