@@ -1089,26 +1089,30 @@ static DWORD WINAPI uv_pipe_zero_readfile_thread_proc(void* parameter) {
     uv_mutex_unlock(m);
   }
 restart_readfile:
-  result = ReadFile(handle->handle,
-                    &uv_zero_,
-                    0,
-                    &bytes,
-                    NULL);
-  if (!result) {
-      err = GetLastError();
-      if (err == ERROR_OPERATION_ABORTED && m) {
-          if (handle->flags & UV_HANDLE_READING) {
-            /* just a brief break to do something else */
-            handle->readfile_thread = NULL;
-            /* resume after it is finished */
-            uv_mutex_lock(m);
-            handle->readfile_thread = hThread;
-            uv_mutex_unlock(m);
-            goto restart_readfile;
-         } else {
-            result = 1; /* successfully stopped reading */
-         }
+  if (handle->flags & UV_HANDLE_READING) {
+      result = ReadFile(handle->handle,
+                        &uv_zero_,
+                        0,
+                        &bytes,
+                        NULL);
+      if (!result) {
+          err = GetLastError();
+          if (err == ERROR_OPERATION_ABORTED && m) {
+              if (handle->flags & UV_HANDLE_READING) {
+                /* just a brief break to do something else */
+                handle->readfile_thread = NULL;
+                /* resume after it is finished */
+                uv_mutex_lock(m);
+                handle->readfile_thread = hThread;
+                uv_mutex_unlock(m);
+                goto restart_readfile;
+             } else {
+                result = 1; /* successfully stopped reading */
+             }
+          }
       }
+  } else {
+      result = 1; /* successfully aborted read before it even started */
   }
   if (hThread) {
     assert(hThread == handle->readfile_thread);
@@ -2015,6 +2019,7 @@ int uv_pipe_open(uv_pipe_t* pipe, uv_file file) {
   NTSTATUS nt_status;
   IO_STATUS_BLOCK io_status;
   FILE_ACCESS_INFORMATION access;
+  DWORD duplex_flags = 0;
 
   /* Determine what kind of permissions we have on this handle */
   nt_status = pNtQueryInformationFile(os_handle,
@@ -2033,7 +2038,6 @@ int uv_pipe_open(uv_pipe_t* pipe, uv_file file) {
     }
   }
 
-  DWORD duplex_flags = 0;
   if (access.AccessFlags&FILE_WRITE_DATA)
     duplex_flags |= UV_HANDLE_WRITABLE;
   if (access.AccessFlags&FILE_READ_DATA)
@@ -2057,14 +2061,13 @@ int uv_pipe_open(uv_pipe_t* pipe, uv_file file) {
     }
   }
 
-  if (os_handle == INVALID_HANDLE_VALUE ||
+  if (os_handle2 == INVALID_HANDLE_VALUE ||
       uv_set_pipe_handle(pipe->loop, pipe, os_handle2, 0) == -1) {
     return UV_EINVAL;
   }
 
   uv_pipe_connection_init(pipe);
 
-  pipe->handle = os_handle;
   if (access.AccessFlags&FILE_WRITE_DATA)
     pipe->flags |= UV_HANDLE_WRITABLE;
   if (access.AccessFlags&FILE_READ_DATA)
