@@ -20,6 +20,7 @@
  */
 
 #include <assert.h>
+#include <stdlib.h>
 
 #include "uv.h"
 #include "internal.h"
@@ -61,15 +62,6 @@ static int uv_udp_set_socket(uv_loop_t* loop, uv_udp_t* handle, SOCKET socket,
 
   assert(handle->socket == INVALID_SOCKET);
 
-  /* Set SO_REUSEADDR on the socket. */
-  if (setsockopt(socket,
-                 SOL_SOCKET,
-                 SO_REUSEADDR,
-                 (char*) &yes,
-                 sizeof yes) == SOCKET_ERROR) {
-    return WSAGetLastError();
-  }
-
   /* Set the socket to nonblocking mode */
   if (ioctlsocket(socket, FIONBIO, &yes) == SOCKET_ERROR) {
     return WSAGetLastError();
@@ -167,37 +159,53 @@ void uv_udp_endgame(uv_loop_t* loop, uv_udp_t* handle) {
 }
 
 
-static int uv__bind(uv_udp_t* handle,
-                    int family,
-                    struct sockaddr* addr,
-                    int addrsize,
-                    unsigned int flags) {
+static int uv_udp_maybe_bind(uv_udp_t* handle,
+                             const struct sockaddr* addr,
+                             unsigned int addrlen,
+                             unsigned int flags) {
   int r;
   int err;
   DWORD no = 0;
 
-  if ((flags & UV_UDP_IPV6ONLY) && family != AF_INET6) {
+  if (handle->flags & UV_HANDLE_BOUND)
+    return 0;
+
+  if ((flags & UV_UDP_IPV6ONLY) && addr->sa_family != AF_INET6) {
     /* UV_UDP_IPV6ONLY is supported only for IPV6 sockets */
     return ERROR_INVALID_PARAMETER;
   }
 
   if (handle->socket == INVALID_SOCKET) {
-    SOCKET sock = socket(family, SOCK_DGRAM, 0);
+    SOCKET sock = socket(addr->sa_family, SOCK_DGRAM, 0);
     if (sock == INVALID_SOCKET) {
       return WSAGetLastError();
     }
 
-    err = uv_udp_set_socket(handle->loop, handle, sock, family);
+    err = uv_udp_set_socket(handle->loop, handle, sock, addr->sa_family);
     if (err) {
       closesocket(sock);
       return err;
     }
 
-    if (family == AF_INET6)
+    if (flags & UV_UDP_REUSEADDR) {
+      DWORD yes = 1;
+      /* Set SO_REUSEADDR on the socket. */
+      if (setsockopt(sock,
+                     SOL_SOCKET,
+                     SO_REUSEADDR,
+                     (char*) &yes,
+                     sizeof yes) == SOCKET_ERROR) {
+        err = WSAGetLastError();
+        closesocket(sock);
+        return err;
+      }
+    }
+
+    if (addr->sa_family == AF_INET6)
       handle->flags |= UV_HANDLE_IPV6;
   }
 
-  if (family == AF_INET6 && !(flags & UV_UDP_IPV6ONLY)) {
+  if (addr->sa_family == AF_INET6 && !(flags & UV_UDP_IPV6ONLY)) {
     /* On windows IPV6ONLY is on by default. */
     /* If the user doesn't specify it libuv turns it off. */
 
@@ -211,7 +219,7 @@ static int uv__bind(uv_udp_t* handle,
                sizeof no);
   }
 
-  r = bind(handle->socket, addr, addrsize);
+  r = bind(handle->socket, addr, addrlen);
   if (r == SOCKET_ERROR) {
     return WSAGetLastError();
   }
@@ -222,36 +230,6 @@ static int uv__bind(uv_udp_t* handle,
 }
 
 
-int uv__udp_bind(uv_udp_t* handle,
-                 struct sockaddr_in addr,
-                 unsigned int flags) {
-  int err;
-
-  err = uv__bind(handle,
-                 AF_INET,
-                 (struct sockaddr*) &addr,
-                 sizeof(addr),
-                 flags);
-
-  return uv_translate_sys_error(err);
-}
-
-
-int uv__udp_bind6(uv_udp_t* handle,
-                  struct sockaddr_in6 addr,
-                  unsigned int flags) {
-  int err;
-
-  err = uv__bind(handle,
-                 AF_INET6,
-                 (struct sockaddr*) &addr,
-                 sizeof(addr),
-                 flags);
-
-  return uv_translate_sys_error(err);
-}
-
-
 static void uv_udp_queue_recv(uv_loop_t* loop, uv_udp_t* handle) {
   uv_req_t* req;
   uv_buf_t buf;
@@ -271,9 +249,9 @@ static void uv_udp_queue_recv(uv_loop_t* loop, uv_udp_t* handle) {
   if (loop->active_udp_streams < uv_active_udp_streams_threshold) {
     handle->flags &= ~UV_HANDLE_ZERO_READ;
 
-    handle->recv_buffer = handle->alloc_cb((uv_handle_t*) handle, 65536);
+    handle->alloc_cb((uv_handle_t*) handle, 65536, &handle->recv_buffer);
     if (handle->recv_buffer.len == 0) {
-      handle->recv_cb(handle, UV_ENOBUFS, handle->recv_buffer, NULL, 0);
+      handle->recv_cb(handle, UV_ENOBUFS, &handle->recv_buffer, NULL, 0);
       return;
     }
     assert(handle->recv_buffer.base != NULL);
@@ -354,11 +332,12 @@ int uv__udp_recv_start(uv_udp_t* handle, uv_alloc_cb alloc_cb,
     return WSAEALREADY;
   }
 
-  if (!(handle->flags & UV_HANDLE_BOUND)) {
-    err = uv_udp_bind(handle, uv_addr_ip4_any_, 0);
-    if (err)
-      return err;
-  }
+  err = uv_udp_maybe_bind(handle,
+                          (const struct sockaddr*) &uv_addr_ip4_any_,
+                          sizeof(uv_addr_ip4_any_),
+                          0);
+  if (err)
+    return err;
 
   handle->flags |= UV_HANDLE_READING;
   INCREASE_ACTIVE_COUNT(loop, handle);
@@ -387,8 +366,13 @@ int uv__udp_recv_stop(uv_udp_t* handle) {
 }
 
 
-static int uv__send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
-    int bufcnt, struct sockaddr* addr, int addr_len, uv_udp_send_cb cb) {
+static int uv__send(uv_udp_send_t* req,
+                    uv_udp_t* handle,
+                    const uv_buf_t bufs[],
+                    unsigned int nbufs,
+                    const struct sockaddr* addr,
+                    unsigned int addrlen,
+                    uv_udp_send_cb cb) {
   uv_loop_t* loop = handle->loop;
   DWORD result, bytes;
 
@@ -400,11 +384,11 @@ static int uv__send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
 
   result = WSASendTo(handle->socket,
                      (WSABUF*)bufs,
-                     bufcnt,
+                     nbufs,
                      &bytes,
                      0,
                      addr,
-                     addr_len,
+                     addrlen,
                      &req->overlapped,
                      NULL);
 
@@ -416,7 +400,7 @@ static int uv__send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
     uv_insert_pending_req(loop, (uv_req_t*)req);
   } else if (UV_SUCCEEDED_WITH_IOCP(result == 0)) {
     /* Request queued by the kernel. */
-    req->queued_bytes = uv_count_bufs(bufs, bufcnt);
+    req->queued_bytes = uv_count_bufs(bufs, nbufs);
     handle->reqs_pending++;
     REGISTER_HANDLE_REQ(loop, handle, req);
   } else {
@@ -428,46 +412,6 @@ static int uv__send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
 }
 
 
-int uv__udp_send(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
-    int bufcnt, struct sockaddr_in addr, uv_udp_send_cb cb) {
-  int err;
-
-  if (!(handle->flags & UV_HANDLE_BOUND)) {
-    err = uv_udp_bind(handle, uv_addr_ip4_any_, 0);
-    if (err)
-      return err;
-  }
-
-  return uv__send(req,
-                  handle,
-                  bufs,
-                  bufcnt,
-                  (struct sockaddr*) &addr,
-                  sizeof addr,
-                  cb);
-}
-
-
-int uv__udp_send6(uv_udp_send_t* req, uv_udp_t* handle, uv_buf_t bufs[],
-    int bufcnt, struct sockaddr_in6 addr, uv_udp_send_cb cb) {
-  int err;
-
-  if (!(handle->flags & UV_HANDLE_BOUND)) {
-    err = uv_udp_bind6(handle, uv_addr_ip6_any_, 0);
-    if (err)
-      return err;
-  }
-
-  return uv__send(req,
-                  handle,
-                  bufs,
-                  bufcnt,
-                  (struct sockaddr*) &addr,
-                  sizeof addr,
-                  cb);
-}
-
-
 void uv_process_udp_recv_req(uv_loop_t* loop, uv_udp_t* handle,
     uv_req_t* req) {
   uv_buf_t buf;
@@ -497,7 +441,7 @@ void uv_process_udp_recv_req(uv_loop_t* loop, uv_udp_t* handle,
         uv_udp_recv_stop(handle);
         buf = (handle->flags & UV_HANDLE_ZERO_READ) ?
               uv_buf_init(NULL, 0) : handle->recv_buffer;
-        handle->recv_cb(handle, uv_translate_sys_error(err), buf, NULL, 0);
+        handle->recv_cb(handle, uv_translate_sys_error(err), &buf, NULL, 0);
       }
       goto done;
     }
@@ -508,8 +452,8 @@ void uv_process_udp_recv_req(uv_loop_t* loop, uv_udp_t* handle,
     partial = !REQ_SUCCESS(req);
     handle->recv_cb(handle,
                     req->overlapped.InternalHigh,
-                    handle->recv_buffer,
-                    (struct sockaddr*) &handle->recv_from,
+                    &handle->recv_buffer,
+                    (const struct sockaddr*) &handle->recv_from,
                     partial ? UV_UDP_PARTIAL : 0);
   } else if (handle->flags & UV_HANDLE_READING) {
     DWORD bytes, err, flags;
@@ -518,9 +462,9 @@ void uv_process_udp_recv_req(uv_loop_t* loop, uv_udp_t* handle,
 
     /* Do a nonblocking receive */
     /* TODO: try to read multiple datagrams at once. FIONREAD maybe? */
-    buf = handle->alloc_cb((uv_handle_t*) handle, 65536);
+    handle->alloc_cb((uv_handle_t*) handle, 65536, &buf);
     if (buf.len == 0) {
-      handle->recv_cb(handle, UV_ENOBUFS, buf, NULL, 0);
+      handle->recv_cb(handle, UV_ENOBUFS, &buf, NULL, 0);
       goto done;
     }
     assert(buf.base != NULL);
@@ -541,24 +485,24 @@ void uv_process_udp_recv_req(uv_loop_t* loop, uv_udp_t* handle,
                     NULL) != SOCKET_ERROR) {
 
       /* Message received */
-      handle->recv_cb(handle, bytes, buf, (struct sockaddr*) &from, 0);
+      handle->recv_cb(handle, bytes, &buf, (const struct sockaddr*) &from, 0);
     } else {
       err = WSAGetLastError();
       if (err == WSAEMSGSIZE) {
         /* Message truncated */
         handle->recv_cb(handle,
                         bytes,
-                        buf,
-                        (struct sockaddr*) &from,
+                        &buf,
+                        (const struct sockaddr*) &from,
                         UV_UDP_PARTIAL);
-      } if (err == WSAEWOULDBLOCK) {
+      } else if (err == WSAEWOULDBLOCK) {
         /* Kernel buffer empty */
-        handle->recv_cb(handle, 0, buf, NULL, 0);
+        handle->recv_cb(handle, 0, &buf, NULL, 0);
       } else if (err != WSAECONNRESET && err != WSAENETRESET) {
         /* Serious error. WSAECONNRESET/WSANETRESET is ignored because this */
         /* just indicates that a previous sendto operation failed. */
         uv_udp_recv_stop(handle);
-        handle->recv_cb(handle, uv_translate_sys_error(err), buf, NULL, 0);
+        handle->recv_cb(handle, uv_translate_sys_error(err), &buf, NULL, 0);
       }
     }
   }
@@ -594,22 +538,24 @@ void uv_process_udp_send_req(uv_loop_t* loop, uv_udp_t* handle,
 }
 
 
-int uv_udp_set_membership(uv_udp_t* handle, const char* multicast_addr,
-    const char* interface_addr, uv_membership membership) {
+static int uv__udp_set_membership4(uv_udp_t* handle,
+                                   const struct sockaddr_in* multicast_addr,
+                                   const char* interface_addr,
+                                   uv_membership membership) {
   int err;
   int optname;
   struct ip_mreq mreq;
 
-  /* If the socket is unbound, bind to inaddr_any. */
-  if (!(handle->flags & UV_HANDLE_BOUND)) {
-    err = uv__udp_bind(handle, uv_addr_ip4_any_, 0);
-    if (err)
-      return uv_translate_sys_error(err);
-  }
+  if (handle->flags & UV_HANDLE_IPV6)
+    return UV_EINVAL;
 
-  if (handle->flags & UV_HANDLE_IPV6) {
-    return UV_ENOSYS;
-  }
+  /* If the socket is unbound, bind to inaddr_any. */
+  err = uv_udp_maybe_bind(handle,
+                          (const struct sockaddr*) &uv_addr_ip4_any_,
+                          sizeof(uv_addr_ip4_any_),
+                          UV_UDP_REUSEADDR);
+  if (err)
+    return uv_translate_sys_error(err);
 
   memset(&mreq, 0, sizeof mreq);
 
@@ -619,7 +565,7 @@ int uv_udp_set_membership(uv_udp_t* handle, const char* multicast_addr,
     mreq.imr_interface.s_addr = htonl(INADDR_ANY);
   }
 
-  mreq.imr_multiaddr.s_addr = inet_addr(multicast_addr);
+  mreq.imr_multiaddr.s_addr = multicast_addr->sin_addr.s_addr;
 
   switch (membership) {
     case UV_JOIN_GROUP:
@@ -644,16 +590,135 @@ int uv_udp_set_membership(uv_udp_t* handle, const char* multicast_addr,
 }
 
 
+int uv__udp_set_membership6(uv_udp_t* handle,
+                            const struct sockaddr_in6* multicast_addr,
+                            const char* interface_addr,
+                            uv_membership membership) {
+  int optname;
+  int err;
+  struct ipv6_mreq mreq;
+  struct sockaddr_in6 addr6;
+
+  if ((handle->flags & UV_HANDLE_BOUND) && !(handle->flags & UV_HANDLE_IPV6))
+    return UV_EINVAL;
+
+  err = uv_udp_maybe_bind(handle,
+                          (const struct sockaddr*) &uv_addr_ip6_any_,
+                          sizeof(uv_addr_ip6_any_),
+                          UV_UDP_REUSEADDR);
+
+  if (err)
+    return uv_translate_sys_error(err);
+
+  memset(&mreq, 0, sizeof(mreq));
+
+  if (interface_addr) {
+    if (uv_ip6_addr(interface_addr, 0, &addr6))
+      return UV_EINVAL;
+    mreq.ipv6mr_interface = addr6.sin6_scope_id;
+  } else {
+    mreq.ipv6mr_interface = 0;
+  }
+
+  mreq.ipv6mr_multiaddr = multicast_addr->sin6_addr;
+
+  switch (membership) {
+  case UV_JOIN_GROUP:
+    optname = IPV6_ADD_MEMBERSHIP;
+    break;
+  case UV_LEAVE_GROUP:
+    optname = IPV6_DROP_MEMBERSHIP;
+    break;
+  default:
+    return UV_EINVAL;
+  }
+
+  if (setsockopt(handle->socket,
+                 IPPROTO_IPV6,
+                 optname,
+                 (char*) &mreq,
+                 sizeof mreq) == SOCKET_ERROR) {
+    return uv_translate_sys_error(WSAGetLastError());
+  }
+
+  return 0;
+}
+
+
+int uv_udp_set_membership(uv_udp_t* handle,
+                          const char* multicast_addr,
+                          const char* interface_addr,
+                          uv_membership membership) {
+  struct sockaddr_in addr4;
+  struct sockaddr_in6 addr6;
+
+  if (uv_ip4_addr(multicast_addr, 0, &addr4) == 0)
+    return uv__udp_set_membership4(handle, &addr4, interface_addr, membership);
+  else if (uv_ip6_addr(multicast_addr, 0, &addr6) == 0)
+    return uv__udp_set_membership6(handle, &addr6, interface_addr, membership);
+  else
+    return UV_EINVAL;
+}
+
+
+int uv_udp_set_multicast_interface(uv_udp_t* handle, const char* interface_addr) {
+  struct sockaddr_storage addr_st;
+  struct sockaddr_in* addr4;
+  struct sockaddr_in6* addr6;
+
+  addr4 = (struct sockaddr_in*) &addr_st;
+  addr6 = (struct sockaddr_in6*) &addr_st;
+
+  if (!interface_addr) {
+    memset(&addr_st, 0, sizeof addr_st);
+    if (handle->flags & UV_HANDLE_IPV6) {
+      addr_st.ss_family = AF_INET6;
+      addr6->sin6_scope_id = 0;
+    } else {
+      addr_st.ss_family = AF_INET;
+      addr4->sin_addr.s_addr = htonl(INADDR_ANY);
+    }
+  } else if (uv_ip4_addr(interface_addr, 0, addr4) == 0) {
+    /* nothing, address was parsed */
+  } else if (uv_ip6_addr(interface_addr, 0, addr6) == 0) {
+    /* nothing, address was parsed */
+  } else {
+    return UV_EINVAL;
+  }
+
+  if (!(handle->flags & UV_HANDLE_BOUND))
+    return UV_EBADF;
+
+  if (addr_st.ss_family == AF_INET) {
+    if (setsockopt(handle->socket,
+                   IPPROTO_IP,
+                   IP_MULTICAST_IF,
+                   (char*) &addr4->sin_addr,
+                   sizeof(addr4->sin_addr)) == SOCKET_ERROR) {
+      return uv_translate_sys_error(WSAGetLastError());
+    }
+  } else if (addr_st.ss_family == AF_INET6) {
+    if (setsockopt(handle->socket,
+                   IPPROTO_IPV6,
+                   IPV6_MULTICAST_IF,
+                   (char*) &addr6->sin6_scope_id,
+                   sizeof(addr6->sin6_scope_id)) == SOCKET_ERROR) {
+      return uv_translate_sys_error(WSAGetLastError());
+    }
+  } else {
+    assert(0 && "unexpected address family");
+    abort();
+  }
+
+  return 0;
+}
+
+
 int uv_udp_set_broadcast(uv_udp_t* handle, int value) {
   BOOL optval = (BOOL) value;
-  int err;
 
-  /* If the socket is unbound, bind to inaddr_any. */
-  if (!(handle->flags & UV_HANDLE_BOUND)) {
-    err = uv__udp_bind(handle, uv_addr_ip4_any_, 0);
-    if (err)
-      return uv_translate_sys_error(err);
-  }
+  if (!(handle->flags & UV_HANDLE_BOUND))
+    return UV_EBADF;
 
   if (setsockopt(handle->socket,
                  SOL_SOCKET,
@@ -693,18 +758,13 @@ int uv_udp_open(uv_udp_t* handle, uv_os_sock_t sock) {
 #define SOCKOPT_SETTER(name, option4, option6, validate)                      \
   int uv_udp_set_##name(uv_udp_t* handle, int value) {                        \
     DWORD optval = (DWORD) value;                                             \
-    int err;                                                                  \
                                                                               \
     if (!(validate(value))) {                                                 \
       return UV_EINVAL;                                                       \
     }                                                                         \
                                                                               \
-    /* If the socket is unbound, bind to inaddr_any. */                       \
-    if (!(handle->flags & UV_HANDLE_BOUND)) {                                 \
-      err = uv__udp_bind(handle, uv_addr_ip4_any_, 0);                        \
-      if (err)                                                                \
-        return uv_translate_sys_error(err);                                   \
-    }                                                                         \
+    if (!(handle->flags & UV_HANDLE_BOUND))                                   \
+      return UV_EBADF;                                                        \
                                                                               \
     if (!(handle->flags & UV_HANDLE_IPV6)) {                                  \
       /* Set IPv4 socket option */                                            \
@@ -749,3 +809,54 @@ SOCKOPT_SETTER(multicast_loop,
 #undef VALIDATE_TTL
 #undef VALIDATE_MULTICAST_TTL
 #undef VALIDATE_MULTICAST_LOOP
+
+
+/* This function is an egress point, i.e. it returns libuv errors rather than
+ * system errors.
+ */
+int uv__udp_bind(uv_udp_t* handle,
+                 const struct sockaddr* addr,
+                 unsigned int addrlen,
+                 unsigned int flags) {
+  int err;
+
+  err = uv_udp_maybe_bind(handle, addr, addrlen, flags);
+  if (err)
+    return uv_translate_sys_error(err);
+
+  return 0;
+}
+
+
+/* This function is an egress point, i.e. it returns libuv errors rather than
+ * system errors.
+ */
+int uv__udp_send(uv_udp_send_t* req,
+                 uv_udp_t* handle,
+                 const uv_buf_t bufs[],
+                 unsigned int nbufs,
+                 const struct sockaddr* addr,
+                 unsigned int addrlen,
+                 uv_udp_send_cb send_cb) {
+  const struct sockaddr* bind_addr;
+  int err;
+
+  if (!(handle->flags & UV_HANDLE_BOUND)) {
+    if (addrlen == sizeof(uv_addr_ip4_any_)) {
+      bind_addr = (const struct sockaddr*) &uv_addr_ip4_any_;
+    } else if (addrlen == sizeof(uv_addr_ip6_any_)) {
+      bind_addr = (const struct sockaddr*) &uv_addr_ip6_any_;
+    } else {
+      abort();
+    }
+    err = uv_udp_maybe_bind(handle, bind_addr, addrlen, 0);
+    if (err)
+      return uv_translate_sys_error(err);
+  }
+
+  err = uv__send(req, handle, bufs, nbufs, addr, addrlen, send_cb);
+  if (err)
+    return uv_translate_sys_error(err);
+
+  return 0;
+}
