@@ -42,11 +42,9 @@ static uv_udp_t udpServer;
 static uv_udp_send_t send_req;
 
 
-static uv_buf_t alloc(uv_handle_t* handle, size_t suggested_size) {
-  uv_buf_t buf;
-  buf.base = (char*) malloc(suggested_size);
-  buf.len = suggested_size;
-  return buf;
+static void alloc(uv_handle_t* handle, size_t suggested_size, uv_buf_t* buf) {
+  buf->base = malloc(suggested_size);
+  buf->len = suggested_size;
 }
 
 
@@ -62,12 +60,14 @@ static void after_shutdown(uv_shutdown_t* req, int status) {
 }
 
 
-static void after_read(uv_stream_t* handle, ssize_t nread, uv_buf_t buf) {
+static void after_read(uv_stream_t* handle,
+                       ssize_t nread,
+                       const uv_buf_t* buf) {
   uv_shutdown_t* req;
   int r;
 
-  if (buf.base) {
-    free(buf.base);
+  if (buf->base) {
+    free(buf->base);
   }
 
   req = (uv_shutdown_t*) malloc(sizeof *req);
@@ -79,10 +79,12 @@ static void after_read(uv_stream_t* handle, ssize_t nread, uv_buf_t buf) {
 static void check_sockname(struct sockaddr* addr, const char* compare_ip,
   int compare_port, const char* context) {
   struct sockaddr_in check_addr = *(struct sockaddr_in*) addr;
-  struct sockaddr_in compare_addr = uv_ip4_addr(compare_ip, compare_port);
+  struct sockaddr_in compare_addr;
   char check_ip[17];
   int r;
 
+  ASSERT(0 == uv_ip4_addr(compare_ip, compare_port, &compare_addr));
+
   /* Both addresses should be ipv4 */
   ASSERT(check_addr.sin_family == AF_INET);
   ASSERT(compare_addr.sin_family == AF_INET);
@@ -165,18 +167,20 @@ static void on_connect(uv_connect_t* req, int status) {
 
 
 static int tcp_listener(void) {
-  struct sockaddr_in addr = uv_ip4_addr("0.0.0.0", server_port);
+  struct sockaddr_in addr;
   struct sockaddr sockname, peername;
   int namelen;
   int r;
 
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", server_port, &addr));
+
   r = uv_tcp_init(loop, &tcpServer);
   if (r) {
     fprintf(stderr, "Socket creation error\n");
     return 1;
   }
 
-  r = uv_tcp_bind(&tcpServer, addr);
+  r = uv_tcp_bind(&tcpServer, (const struct sockaddr*) &addr, 0);
   if (r) {
     fprintf(stderr, "Bind error\n");
     return 1;
@@ -205,15 +209,20 @@ static int tcp_listener(void) {
 
 
 static void tcp_connector(void) {
-  struct sockaddr_in server_addr = uv_ip4_addr("127.0.0.1", server_port);
+  struct sockaddr_in server_addr;
   struct sockaddr sockname;
   int r, namelen;
 
+  ASSERT(0 == uv_ip4_addr("127.0.0.1", server_port, &server_addr));
+
   r = uv_tcp_init(loop, &tcp);
   tcp.data = &connect_req;
   ASSERT(!r);
 
-  r = uv_tcp_connect(&connect_req, &tcp, server_addr, on_connect);
+  r = uv_tcp_connect(&connect_req,
+                     &tcp,
+                     (const struct sockaddr*) &server_addr,
+                     on_connect);
   ASSERT(!r);
 
   /* Fetch the actual port used by the connecting socket. */
@@ -228,15 +237,15 @@ static void tcp_connector(void) {
 
 static void udp_recv(uv_udp_t* handle,
                      ssize_t nread,
-                     uv_buf_t buf,
-                     struct sockaddr* addr,
+                     const uv_buf_t* buf,
+                     const struct sockaddr* addr,
                      unsigned flags) {
   struct sockaddr sockname;
   int namelen;
   int r;
 
   ASSERT(nread >= 0);
-  free(buf.base);
+  free(buf->base);
 
   if (nread == 0) {
     return;
@@ -260,18 +269,20 @@ static void udp_send(uv_udp_send_t* req, int status) {
 
 
 static int udp_listener(void) {
-  struct sockaddr_in addr = uv_ip4_addr("0.0.0.0", server_port);
+  struct sockaddr_in addr;
   struct sockaddr sockname;
   int namelen;
   int r;
 
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", server_port, &addr));
+
   r = uv_udp_init(loop, &udpServer);
   if (r) {
     fprintf(stderr, "Socket creation error\n");
     return 1;
   }
 
-  r = uv_udp_bind(&udpServer, addr, 0);
+  r = uv_udp_bind(&udpServer, (const struct sockaddr*) &addr, 0);
   if (r) {
     fprintf(stderr, "Bind error\n");
     return 1;
@@ -300,9 +311,14 @@ static void udp_sender(void) {
   ASSERT(!r);
 
   buf = uv_buf_init("PING", 4);
-  server_addr = uv_ip4_addr("127.0.0.1", server_port);
+  ASSERT(0 == uv_ip4_addr("127.0.0.1", server_port, &server_addr));
 
-  r = uv_udp_send(&send_req, &udp, &buf, 1, server_addr, udp_send);
+  r = uv_udp_send(&send_req,
+                  &udp,
+                  &buf,
+                  1,
+                  (const struct sockaddr*) &server_addr,
+                  udp_send);
   ASSERT(!r);
 }
 
