@@ -50,22 +50,30 @@ static struct echo_ctx ctx;
 static int num_recv_handles;
 
 
-static uv_buf_t alloc_cb(uv_handle_t* handle, size_t suggested_size) {
+static void alloc_cb(uv_handle_t* handle,
+                     size_t suggested_size,
+                     uv_buf_t* buf) {
   /* we're not actually reading anything so a small buffer is okay */
-  static char buf[8];
-  return uv_buf_init(buf, sizeof(buf));
+  static char slab[8];
+  buf->base = slab;
+  buf->len = sizeof(slab);
 }
 
 
-static void recv_cb(uv_pipe_t* handle,
+static void recv_cb(uv_stream_t* handle,
                     ssize_t nread,
-                    uv_buf_t buf,
-                    uv_handle_type pending) {
+                    const uv_buf_t* buf) {
+  uv_handle_type pending;
+  uv_pipe_t* pipe;
   int r;
 
-  ASSERT(pending == ctx.expected_type);
-  ASSERT(handle == &ctx.channel);
+  pipe = (uv_pipe_t*) handle;
+  ASSERT(pipe == &ctx.channel);
   ASSERT(nread >= 0);
+  ASSERT(1 == uv_pipe_pending_count(pipe));
+
+  pending = uv_pipe_pending_type(pipe);
+  ASSERT(pending == ctx.expected_type);
 
   if (pending == UV_NAMED_PIPE)
     r = uv_pipe_init(ctx.channel.loop, &ctx.recv.pipe, UV_PIPE_READABLE|UV_PIPE_WRITABLE);
@@ -75,7 +83,7 @@ static void recv_cb(uv_pipe_t* handle,
     abort();
   ASSERT(r == 0);
 
-  r = uv_accept((uv_stream_t*)&ctx.channel, &ctx.recv.stream);
+  r = uv_accept(handle, &ctx.recv.stream);
   ASSERT(r == 0);
 
   uv_close((uv_handle_t*)&ctx.channel, NULL);
@@ -100,7 +108,7 @@ static int run_test(void) {
                 NULL);
   ASSERT(r == 0);
 
-  r = uv_read2_start((uv_stream_t*)&ctx.channel, alloc_cb, recv_cb);
+  r = uv_read_start((uv_stream_t*)&ctx.channel, alloc_cb, recv_cb);
   ASSERT(r == 0);
 
   r = uv_run(uv_default_loop(), UV_RUN_DEFAULT);
@@ -132,14 +140,17 @@ TEST_IMPL(ipc_send_recv_pipe) {
 
 
 TEST_IMPL(ipc_send_recv_tcp) {
+  struct sockaddr_in addr;
   int r;
 
+  ASSERT(0 == uv_ip4_addr("127.0.0.1", TEST_PORT, &addr));
+
   ctx.expected_type = UV_TCP;
 
   r = uv_tcp_init(uv_default_loop(), &ctx.send.tcp);
   ASSERT(r == 0);
 
-  r = uv_tcp_bind(&ctx.send.tcp, uv_ip4_addr("127.0.0.1", TEST_PORT));
+  r = uv_tcp_bind(&ctx.send.tcp, (const struct sockaddr*) &addr, 0);
   ASSERT(r == 0);
 
   r = run_test();
@@ -159,17 +170,23 @@ static void write2_cb(uv_write_t* req, int status) {
 }
 
 
-static void read2_cb(uv_pipe_t* handle,
-                     ssize_t nread,
-                     uv_buf_t buf,
-                     uv_handle_type pending) {
+static void read_cb(uv_stream_t* handle,
+                    ssize_t nread,
+                    const uv_buf_t* rdbuf) {
+  uv_buf_t wrbuf;
+  uv_pipe_t* pipe;
+  uv_handle_type pending;
   int r;
 
-  ASSERT(pending == UV_NAMED_PIPE || pending == UV_TCP);
-  ASSERT(handle == &ctx.channel);
+  pipe = (uv_pipe_t*) handle;
+  ASSERT(pipe == &ctx.channel);
   ASSERT(nread >= 0);
+  ASSERT(1 == uv_pipe_pending_count(pipe));
 
-  buf = uv_buf_init(".", 1);
+  pending = uv_pipe_pending_type(pipe);
+  ASSERT(pending == UV_NAMED_PIPE || pending == UV_TCP);
+
+  wrbuf = uv_buf_init(".", 1);
 
   if (pending == UV_NAMED_PIPE)
     r = uv_pipe_init(ctx.channel.loop, &ctx.recv.pipe, UV_PIPE_READABLE|UV_PIPE_WRITABLE);
@@ -179,12 +196,13 @@ static void read2_cb(uv_pipe_t* handle,
     abort();
   ASSERT(r == 0);
 
-  r = uv_accept((uv_stream_t*)handle, &ctx.recv.stream);
+  r = uv_accept(handle, &ctx.recv.stream);
   ASSERT(r == 0);
 
   r = uv_write2(&ctx.write_req,
                 (uv_stream_t*)&ctx.channel,
-                &buf, 1,
+                &wrbuf,
+                1,
                 &ctx.recv.stream,
                 write2_cb);
   ASSERT(r == 0);
@@ -203,11 +221,11 @@ int ipc_send_recv_helper(void) {
   ASSERT(r == 0);
 
   uv_pipe_open(&ctx.channel, 0);
-  ASSERT(uv_is_readable((uv_stream_t*)&ctx.channel));
-  ASSERT(uv_is_writable((uv_stream_t*)&ctx.channel));
-  ASSERT(!uv_is_closing((uv_handle_t*)&ctx.channel));
+  ASSERT(1 == uv_is_readable((uv_stream_t*)&ctx.channel));
+  ASSERT(1 == uv_is_writable((uv_stream_t*)&ctx.channel));
+  ASSERT(0 == uv_is_closing((uv_handle_t*)&ctx.channel));
 
-  r = uv_read2_start((uv_stream_t*)&ctx.channel, alloc_cb, read2_cb);
+  r = uv_read_start((uv_stream_t*)&ctx.channel, alloc_cb, read_cb);
   ASSERT(r == 0);
 
   r = uv_run(uv_default_loop(), UV_RUN_DEFAULT);
