@@ -30,7 +30,7 @@ static uv_tcp_t tcp_server;
 static uv_tcp_t tcp_connection;
 
 static int exit_cb_called;
-static int read2_cb_called;
+static int read_cb_called;
 static int tcp_write_cb_called;
 static int tcp_read_cb_called;
 static int on_pipe_read_called;
@@ -84,7 +84,9 @@ static void on_connection(uv_stream_t* server, int status) {
 }
 
 
-static void exit_cb(uv_process_t* process, int exit_status, int term_signal) {
+static void exit_cb(uv_process_t* process,
+                    int64_t exit_status,
+                    int term_signal) {
   printf("exit_cb\n");
   exit_cb_called++;
   ASSERT(exit_status == 0);
@@ -92,8 +94,11 @@ static void exit_cb(uv_process_t* process, int exit_status, int term_signal) {
 }
 
 
-static uv_buf_t on_alloc(uv_handle_t* handle, size_t suggested_size) {
-  return uv_buf_init(malloc(suggested_size), suggested_size);
+static void on_alloc(uv_handle_t* handle,
+                     size_t suggested_size,
+                     uv_buf_t* buf) {
+  buf->base = malloc(suggested_size);
+  buf->len = suggested_size;
 }
 
 
@@ -120,9 +125,12 @@ static void make_many_connections(void) {
     r = uv_tcp_init(uv_default_loop(), &conn->conn);
     ASSERT(r == 0);
 
-    addr = uv_ip4_addr("127.0.0.1", TEST_PORT);
+    ASSERT(0 == uv_ip4_addr("127.0.0.1", TEST_PORT, &addr));
 
-    r = uv_tcp_connect(&conn->conn_req, (uv_tcp_t*)&conn->conn, addr, connect_cb);
+    r = uv_tcp_connect(&conn->conn_req,
+                       (uv_tcp_t*) &conn->conn,
+                       (const struct sockaddr*) &addr,
+                       connect_cb);
     ASSERT(r == 0);
 
     conn->conn.data = conn;
@@ -130,34 +138,39 @@ static void make_many_connections(void) {
 }
 
 
-static void on_read(uv_pipe_t* pipe, ssize_t nread, uv_buf_t buf,
-    uv_handle_type pending) {
+static void on_read(uv_stream_t* handle,
+                    ssize_t nread,
+                    const uv_buf_t* buf) {
   int r;
+  uv_pipe_t* pipe;
+  uv_handle_type pending;
   uv_buf_t outbuf;
-  uv_err_t err;
+
+  pipe = (uv_pipe_t*) handle;
 
   if (nread == 0) {
     /* Everything OK, but nothing read. */
-    free(buf.base);
+    free(buf->base);
     return;
   }
 
   if (nread < 0) {
-    err = uv_last_error(pipe->loop);
-    if (err.code == UV_EOF) {
-      free(buf.base);
+    if (nread == UV_EOF) {
+      free(buf->base);
       return;
     }
 
-    printf("error recving on channel: %s\n", uv_strerror(err));
+    printf("error recving on channel: %s\n", uv_strerror(nread));
     abort();
   }
 
   fprintf(stderr, "got %d bytes\n", (int)nread);
 
+  pending = uv_pipe_pending_type(pipe);
   if (!tcp_server_listening) {
-    ASSERT(nread > 0 && buf.base && pending != UV_UNKNOWN_HANDLE);
-    read2_cb_called++;
+    ASSERT(1 == uv_pipe_pending_count(pipe));
+    ASSERT(nread > 0 && buf->base && pending != UV_UNKNOWN_HANDLE);
+    read_cb_called++;
 
     /* Accept the pending TCP server, and start listening on it. */
     ASSERT(pending == UV_TCP);
@@ -173,7 +186,7 @@ static void on_read(uv_pipe_t* pipe, ssize_t nread, uv_buf_t buf,
     tcp_server_listening = 1;
 
     /* Make sure that the expected data is correctly multiplexed. */
-    ASSERT(memcmp("hello\n", buf.base, nread) == 0);
+    ASSERT(memcmp("hello\n", buf->base, nread) == 0);
 
     outbuf = uv_buf_init("world\n", 6);
     r = uv_write(&write_req, (uv_stream_t*)pipe, &outbuf, 1, NULL);
@@ -181,14 +194,15 @@ static void on_read(uv_pipe_t* pipe, ssize_t nread, uv_buf_t buf,
 
     /* Create a bunch of connections to get both servers to accept. */
     make_many_connections();
-  } else if (memcmp("accepted_connection\n", buf.base, nread) == 0) {
+  } else if (memcmp("accepted_connection\n", buf->base, nread) == 0) {
     /* Remote server has accepted a connection.  Close the channel. */
+    ASSERT(0 == uv_pipe_pending_count(pipe));
     ASSERT(pending == UV_UNKNOWN_HANDLE);
     remote_conn_accepted = 1;
     uv_close((uv_handle_t*)&channel, NULL);
   }
 
-  free(buf.base);
+  free(buf->base);
 }
 
 
@@ -204,10 +218,10 @@ void spawn_helper(uv_pipe_t* channel,
   uv_pipe_t child_channel;
 
   r = uv_pipe_init(uv_default_loop(), channel,
-	  UV_PIPE_READABLE|UV_PIPE_WRITEABLE|UV_PIPE_IPC);
+	  UV_PIPE_READABLE|UV_PIPE_WRITABLE|UV_PIPE_IPC);
   ASSERT(r == 0);
   r = uv_pipe_init(uv_default_loop(), &child_channel,
-	  UV_PIPE_READABLE|UV_PIPE_WRITEABLE);
+	  UV_PIPE_READABLE|UV_PIPE_WRITABLE);
   ASSERT(r == 0);
   r = uv_pipe_link(&child_channel,channel);
   ASSERT(r == 0);
@@ -231,7 +245,7 @@ void spawn_helper(uv_pipe_t* channel,
   options.stdio[0].data.stream = (uv_stream_t*)&child_channel;
   options.stdio_count = 1;
 
-  r = uv_spawn(uv_default_loop(), process, options);
+  r = uv_spawn(uv_default_loop(), process, &options);
   ASSERT(r == 0);
 
   uv_pipe_close_sync(&child_channel);
@@ -245,19 +259,19 @@ static void on_tcp_write(uv_write_t* req, int status) {
 }
 
 
-static uv_buf_t on_read_alloc(uv_handle_t* handle, size_t suggested_size) {
-  uv_buf_t buf;
-  buf.base = (char*)malloc(suggested_size);
-  buf.len = suggested_size;
-  return buf;
+static void on_read_alloc(uv_handle_t* handle,
+                          size_t suggested_size,
+                          uv_buf_t* buf) {
+  buf->base = malloc(suggested_size);
+  buf->len = suggested_size;
 }
 
 
-static void on_tcp_read(uv_stream_t* tcp, ssize_t nread, uv_buf_t buf) {
+static void on_tcp_read(uv_stream_t* tcp, ssize_t nread, const uv_buf_t* buf) {
   ASSERT(nread > 0);
-  ASSERT(memcmp("hello again\n", buf.base, nread) == 0);
+  ASSERT(memcmp("hello again\n", buf->base, nread) == 0);
   ASSERT(tcp == (uv_stream_t*)&tcp_connection);
-  free(buf.base);
+  free(buf->base);
 
   tcp_read_cb_called++;
 
@@ -266,44 +280,49 @@ static void on_tcp_read(uv_stream_t* tcp, ssize_t nread, uv_buf_t buf) {
 }
 
 
-static void on_read_connection(uv_pipe_t* pipe, ssize_t nread, uv_buf_t buf,
-    uv_handle_type pending) {
+static void on_read_connection(uv_stream_t* handle,
+                               ssize_t nread,
+                               const uv_buf_t* buf) {
   int r;
   uv_buf_t outbuf;
-  uv_err_t err;
+  uv_pipe_t* pipe;
+  uv_handle_type pending;
 
+  pipe = (uv_pipe_t*) handle;
   if (nread == 0) {
     /* Everything OK, but nothing read. */
-    free(buf.base);
+    free(buf->base);
     return;
   }
 
   if (nread < 0) {
-    err = uv_last_error(pipe->loop);
-    if (err.code == UV_EOF) {
-      free(buf.base);
+    if (nread == UV_EOF) {
+      free(buf->base);
       return;
     }
 
-    printf("error recving on channel: %s\n", uv_strerror(err));
+    printf("error recving on channel: %s\n", uv_strerror(nread));
     abort();
   }
 
   fprintf(stderr, "got %d bytes\n", (int)nread);
 
-  ASSERT(nread > 0 && buf.base && pending != UV_UNKNOWN_HANDLE);
-  read2_cb_called++;
+  ASSERT(1 == uv_pipe_pending_count(pipe));
+  pending = uv_pipe_pending_type(pipe);
+
+  ASSERT(nread > 0 && buf->base && pending != UV_UNKNOWN_HANDLE);
+  read_cb_called++;
 
   /* Accept the pending TCP connection */
   ASSERT(pending == UV_TCP);
   r = uv_tcp_init(uv_default_loop(), &tcp_connection);
   ASSERT(r == 0);
 
-  r = uv_accept((uv_stream_t*)pipe, (uv_stream_t*)&tcp_connection);
+  r = uv_accept(handle, (uv_stream_t*)&tcp_connection);
   ASSERT(r == 0);
 
   /* Make sure that the expected data is correctly multiplexed. */
-  ASSERT(memcmp("hello\n", buf.base, nread) == 0);
+  ASSERT(memcmp("hello\n", buf->base, nread) == 0);
 
   /* Write/read to/from the connection */
   outbuf = uv_buf_init("world\n", 6);
@@ -314,16 +333,16 @@ static void on_read_connection(uv_pipe_t* pipe, ssize_t nread, uv_buf_t buf,
   r = uv_read_start((uv_stream_t*)&tcp_connection, on_read_alloc, on_tcp_read);
   ASSERT(r == 0);
 
-  free(buf.base);
+  free(buf->base);
 }
 
 
-static int run_ipc_test(const char* helper, uv_read2_cb read_cb) {
+static int run_ipc_test(const char* helper, uv_read_cb read_cb) {
   uv_process_t process;
   int r;
 
   spawn_helper(&channel, &process, helper);
-  uv_read2_start((uv_stream_t*)&channel, on_alloc, read_cb);
+  uv_read_start((uv_stream_t*)&channel, on_alloc, read_cb);
 
   r = uv_run(uv_default_loop(), UV_RUN_DEFAULT);
   ASSERT(r == 0);
@@ -337,7 +356,7 @@ TEST_IMPL(ipc_listen_before_write) {
   int r = run_ipc_test("ipc_helper_listen_before_write", on_read);
   ASSERT(local_conn_accepted == 1);
   ASSERT(remote_conn_accepted == 1);
-  ASSERT(read2_cb_called == 1);
+  ASSERT(read_cb_called == 1);
   ASSERT(exit_cb_called == 1);
   return r;
 }
@@ -347,7 +366,7 @@ TEST_IMPL(ipc_listen_after_write) {
   int r = run_ipc_test("ipc_helper_listen_after_write", on_read);
   ASSERT(local_conn_accepted == 1);
   ASSERT(remote_conn_accepted == 1);
-  ASSERT(read2_cb_called == 1);
+  ASSERT(read_cb_called == 1);
   ASSERT(exit_cb_called == 1);
   return r;
 }
@@ -355,7 +374,7 @@ TEST_IMPL(ipc_listen_after_write) {
 
 TEST_IMPL(ipc_tcp_connection) {
   int r = run_ipc_test("ipc_helper_tcp_connection", on_read_connection);
-  ASSERT(read2_cb_called == 1);
+  ASSERT(read_cb_called == 1);
   ASSERT(tcp_write_cb_called == 1);
   ASSERT(tcp_read_cb_called == 1);
   ASSERT(exit_cb_called == 1);
@@ -367,12 +386,14 @@ TEST_IMPL(ipc_tcp_connection) {
 TEST_IMPL(listen_with_simultaneous_accepts) {
   uv_tcp_t server;
   int r;
-  struct sockaddr_in addr = uv_ip4_addr("0.0.0.0", TEST_PORT);
+  struct sockaddr_in addr;
+
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", TEST_PORT, &addr));
 
   r = uv_tcp_init(uv_default_loop(), &server);
   ASSERT(r == 0);
 
-  r = uv_tcp_bind(&server, addr);
+  r = uv_tcp_bind(&server, (const struct sockaddr*) &addr, 0);
   ASSERT(r == 0);
 
   r = uv_tcp_simultaneous_accepts(&server, 1);
@@ -390,12 +411,14 @@ TEST_IMPL(listen_with_simultaneous_accepts) {
 TEST_IMPL(listen_no_simultaneous_accepts) {
   uv_tcp_t server;
   int r;
-  struct sockaddr_in addr = uv_ip4_addr("0.0.0.0", TEST_PORT);
+  struct sockaddr_in addr;
+
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", TEST_PORT, &addr));
 
   r = uv_tcp_init(uv_default_loop(), &server);
   ASSERT(r == 0);
 
-  r = uv_tcp_bind(&server, addr);
+  r = uv_tcp_bind(&server, (const struct sockaddr*) &addr, 0);
   ASSERT(r == 0);
 
   r = uv_tcp_simultaneous_accepts(&server, 0);
@@ -436,25 +459,26 @@ static void tcp_connection_write_cb(uv_write_t* req, int status) {
 }
 
 
-static void on_tcp_child_process_read(uv_stream_t* tcp, ssize_t nread, uv_buf_t buf) {
+static void on_tcp_child_process_read(uv_stream_t* tcp,
+                                      ssize_t nread,
+                                      const uv_buf_t* buf) {
   uv_buf_t outbuf;
   int r;
 
   if (nread < 0) {
-    if (uv_last_error(tcp->loop).code == UV_EOF) {
-      free(buf.base);
+    if (nread == UV_EOF) {
+      free(buf->base);
       return;
     }
 
-    printf("error recving on tcp connection: %s\n", 
-      uv_strerror(uv_last_error(tcp->loop)));
+    printf("error recving on tcp connection: %s\n", uv_strerror(nread));
     abort();
   }
 
   ASSERT(nread > 0);
-  ASSERT(memcmp("world\n", buf.base, nread) == 0);
+  ASSERT(memcmp("world\n", buf->base, nread) == 0);
   on_pipe_read_called++;
-  free(buf.base);
+  free(buf->base);
 
   /* Write to the socket */
   outbuf = uv_buf_init("hello again\n", 12);
@@ -527,7 +551,9 @@ static void ipc_on_connection_tcp_conn(uv_stream_t* server, int status) {
     (uv_stream_t*)conn, NULL);
   ASSERT(r == 0);
 
-  r = uv_read_start((uv_stream_t*)conn, on_read_alloc, on_tcp_child_process_read);
+  r = uv_read_start((uv_stream_t*) conn,
+                    on_read_alloc,
+                    on_tcp_child_process_read);
   ASSERT(r == 0);
 
   uv_close((uv_handle_t*)conn, close_cb);
@@ -539,24 +565,26 @@ int ipc_helper(int listen_after_write) {
    * This is launched from test-ipc.c. stdin is a duplex channel that we
    * over which a handle will be transmitted.
    */
-
+  struct sockaddr_in addr;
   uv_write_t write_req;
   int r;
   uv_buf_t buf;
 
-  r = uv_pipe_init(uv_default_loop(), &channel, 1);
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", TEST_PORT, &addr));
+
+  r = uv_pipe_init(uv_default_loop(), &channel, UV_PIPE_READABLE|UV_PIPE_WRITABLE|UV_PIPE_IPC);
   ASSERT(r == 0);
 
   uv_pipe_open(&channel, 0);
 
-  ASSERT(uv_is_readable((uv_stream_t*) &channel));
-  ASSERT(uv_is_writable((uv_stream_t*) &channel));
-  ASSERT(!uv_is_closing((uv_handle_t*) &channel));
+  ASSERT(1 == uv_is_readable((uv_stream_t*) &channel));
+  ASSERT(1 == uv_is_writable((uv_stream_t*) &channel));
+  ASSERT(0 == uv_is_closing((uv_handle_t*) &channel));
 
   r = uv_tcp_init(uv_default_loop(), &tcp_server);
   ASSERT(r == 0);
 
-  r = uv_tcp_bind(&tcp_server, uv_ip4_addr("0.0.0.0", TEST_PORT));
+  r = uv_tcp_bind(&tcp_server, (const struct sockaddr*) &addr, 0);
   ASSERT(r == 0);
 
   if (!listen_after_write) {
@@ -594,19 +622,21 @@ int ipc_helper_tcp_connection(void) {
   int r;
   struct sockaddr_in addr;
 
-  r = uv_pipe_init(uv_default_loop(), &channel, 1);
+  r = uv_pipe_init(uv_default_loop(), &channel, UV_PIPE_READABLE|UV_PIPE_WRITABLE|UV_PIPE_IPC);
   ASSERT(r == 0);
 
   uv_pipe_open(&channel, 0);
 
-  ASSERT(uv_is_readable((uv_stream_t*)&channel));
-  ASSERT(uv_is_writable((uv_stream_t*)&channel));
-  ASSERT(!uv_is_closing((uv_handle_t*)&channel));
+  ASSERT(1 == uv_is_readable((uv_stream_t*) &channel));
+  ASSERT(1 == uv_is_writable((uv_stream_t*) &channel));
+  ASSERT(0 == uv_is_closing((uv_handle_t*) &channel));
 
   r = uv_tcp_init(uv_default_loop(), &tcp_server);
   ASSERT(r == 0);
 
-  r = uv_tcp_bind(&tcp_server, uv_ip4_addr("0.0.0.0", TEST_PORT));
+  ASSERT(0 == uv_ip4_addr("0.0.0.0", TEST_PORT, &addr));
+
+  r = uv_tcp_bind(&tcp_server, (const struct sockaddr*) &addr, 0);
   ASSERT(r == 0);
 
   r = uv_listen((uv_stream_t*)&tcp_server, 12, ipc_on_connection_tcp_conn);
@@ -616,8 +646,12 @@ int ipc_helper_tcp_connection(void) {
   r = uv_tcp_init(uv_default_loop(), &conn.conn);
   ASSERT(r == 0);
 
-  addr = uv_ip4_addr("127.0.0.1", TEST_PORT);
-  r = uv_tcp_connect(&conn.conn_req, (uv_tcp_t*)&conn.conn, addr, connect_child_process_cb);
+  ASSERT(0 == uv_ip4_addr("127.0.0.1", TEST_PORT, &addr));
+
+  r = uv_tcp_connect(&conn.conn_req,
+                     (uv_tcp_t*) &conn.conn,
+                     (const struct sockaddr*) &addr,
+                     connect_child_process_cb);
   ASSERT(r == 0);
 
   r = uv_run(uv_default_loop(), UV_RUN_DEFAULT);
