@@ -1,6 +1,10 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 module ModInts
 export ModInt
 
+import Base: +, -, *
+
 immutable ModInt{n} <: Integer
     k::Int
     ModInt(k) = new(mod(k,n))
@@ -17,4 +21,6 @@ Base.promote_rule{n}(::Type{ModInt{n}}, ::Type{Int}) = ModInt{n}
 Base.show{n}(io::IO, k::ModInt{n}) = print(io, "$(k.k) mod $n")
 Base.showcompact(io::IO, k::ModInt) = print(io, k.k)
 
+Base.inv{n}(a::ModInt{n}) = ModInt{n}(invmod(a.k, n))
+
 end # module
