@@ -4,6 +4,10 @@
 #include "flisp.h"
 #include "opcodes.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 static value_t argv_list(int argc, char *argv[])
 {
     int i;
@@ -28,7 +32,7 @@ int main(int argc, char *argv[])
 
     fname_buf[0] = '\0';
     value_t str = symbol_value(symbol("*install-dir*"));
-    char *exedir = (str == UNBOUND ? NULL : cvalue_data(str));
+    char *exedir = (char*)(str == UNBOUND ? NULL : cvalue_data(str));
     if (exedir != NULL) {
         strcat(fname_buf, exedir);
         strcat(fname_buf, PATHSEPSTRING);
@@ -58,3 +62,7 @@ int main(int argc, char *argv[])
     }
     return 0;
 }
+
+#ifdef __cplusplus
+}
+#endif
