@@ -1,3 +1,5 @@
+// This file is a part of Julia. License is MIT: http://julialang.org/license
+
 #ifndef IOS_H
 #define IOS_H
 
@@ -78,6 +80,7 @@ DLLEXPORT off_t ios_skip(ios_t *s, off_t offs);  // relative seek
 DLLEXPORT off_t ios_pos(ios_t *s);  // get current position
 DLLEXPORT int ios_trunc(ios_t *s, size_t size);
 DLLEXPORT int ios_eof(ios_t *s);
+DLLEXPORT int ios_eof_blocking(ios_t *s);
 DLLEXPORT int ios_flush(ios_t *s);
 DLLEXPORT void ios_close(ios_t *s);
 DLLEXPORT int ios_isopen(ios_t *s);
@@ -93,13 +96,11 @@ DLLEXPORT size_t ios_copyall(ios_t *to, ios_t *from);
 DLLEXPORT size_t ios_copyuntil(ios_t *to, ios_t *from, char delim);
 // ensure at least n bytes are buffered if possible. returns # available.
 DLLEXPORT size_t ios_readprep(ios_t *from, size_t n);
-//void ios_lock(ios_t *s);
-//int ios_trylock(ios_t *s);
-//int ios_unlock(ios_t *s);
 
 /* stream creation */
 DLLEXPORT
-ios_t *ios_file(ios_t *s, char *fname, int rd, int wr, int create, int trunc);
+ios_t *ios_file(ios_t *s, const char *fname, int rd, int wr, int create, int trunc);
+DLLEXPORT ios_t *ios_mkstemp(ios_t *f, char *fname);
 DLLEXPORT ios_t *ios_mem(ios_t *s, size_t initsize);
 ios_t *ios_str(ios_t *s, char *str);
 ios_t *ios_static_buffer(ios_t *s, char *buf, size_t sz);
@@ -111,30 +112,18 @@ extern DLLEXPORT ios_t *ios_stderr;
 void ios_init_stdstreams(void);
 
 /* high-level functions - output */
-int ios_putnum(ios_t *s, char *data, uint32_t type);
-int ios_putint(ios_t *s, int n);
 DLLEXPORT int ios_pututf8(ios_t *s, uint32_t wc);
-int ios_putstringz(ios_t *s, char *str, bool_t do_write_nulterm);
 DLLEXPORT int ios_printf(ios_t *s, const char *format, ...);
 DLLEXPORT int ios_vprintf(ios_t *s, const char *format, va_list args);
 
 /* high-level stream functions - input */
-int ios_getnum(ios_t *s, char *data, uint32_t type);
 DLLEXPORT int ios_getutf8(ios_t *s, uint32_t *pwc);
 DLLEXPORT int ios_peekutf8(ios_t *s, uint32_t *pwc);
-int ios_ungetutf8(ios_t *s, uint32_t wc);
-//int ios_getstringz(ios_t *dest, ios_t *src);
-//int ios_getstringn(ios_t *dest, ios_t *src, size_t nchars);
-//int ios_getline(ios_t *s, char **pbuf, size_t *psz);
 DLLEXPORT char *ios_readline(ios_t *s);
 
 // discard data buffered for reading
 DLLEXPORT void ios_purge(ios_t *s);
 
-// seek by utf8 sequence increments
-int ios_nextutf8(ios_t *s);
-int ios_prevutf8(ios_t *s);
-
 /* stdio-style functions */
 #define IOS_EOF (-1)
 DLLEXPORT int ios_putc(int c, ios_t *s);
