@@ -1,14 +1,18 @@
+// This file is a part of Julia. License is MIT: http://julialang.org/license
+
 /*
   sys.c
   I/O and operating system utility functions
 */
 #include "julia.h"
-#include "uv.h"
+#include "julia_internal.h"
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#ifndef _OS_WINDOWS_
+#ifdef _OS_WINDOWS_
+#include <psapi.h>
+#else
 #include <sys/sysctl.h>
 #include <sys/wait.h>
 #include <sys/ptrace.h>
@@ -20,6 +24,13 @@
 #include <signal.h>
 #include <fcntl.h>
 
+#ifndef _OS_WINDOWS_
+// for getrusage
+#include <sys/types.h>
+#include <sys/time.h>
+#include <sys/resource.h>
+#endif
+
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
 #include <mach-o/nlist.h>
@@ -41,12 +52,17 @@
 #include <intrin.h>
 #endif
 
+#ifdef __has_feature
+#if __has_feature(memory_sanitizer)
+#include <sanitizer/msan_interface.h>
+#endif
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #if defined(_OS_WINDOWS_) && !defined(_COMPILER_MINGW_)
-DLLEXPORT char *basename(char *);
 DLLEXPORT char *dirname(char *);
 #else
 #include <libgen.h>
@@ -59,8 +75,6 @@ DLLEXPORT uint32_t jl_getutf8(ios_t *s)
     return wc;
 }
 
-DLLEXPORT size_t jl_ios_size(ios_t *s) { return s->size; }
-
 DLLEXPORT int jl_sizeof_off_t(void) { return sizeof(off_t); }
 #ifndef _OS_WINDOWS_
 DLLEXPORT off_t jl_lseek(int fd, off_t offset, int whence) { return lseek(fd, offset, whence); }
@@ -85,15 +99,6 @@ DLLEXPORT int32_t jl_nb_available(ios_t *s)
     return (int32_t)(s->size - s->bpos);
 }
 
-DLLEXPORT int jl_ios_eof(ios_t *s)
-{
-    if (ios_eof(s))
-        return 1;
-    if (ios_readprep(s, 1) < 1)
-        return 1;
-    return 0;
-}
-
 // --- dir/file stuff ---
 
 DLLEXPORT int jl_sizeof_uv_fs_t(void) { return sizeof(uv_fs_t); }
@@ -295,9 +300,43 @@ jl_value_t *jl_readuntil(ios_t *s, uint8_t delim)
     return (jl_value_t*)a;
 }
 
-void jl_free2(void *p, void *hint)
+static void NORETURN throw_eof_error(void)
 {
-    free(p);
+    jl_datatype_t *eof_error = (jl_datatype_t*)jl_get_global(jl_base_module, jl_symbol("EOFError"));
+    assert(eof_error != NULL);
+    jl_exceptionf(eof_error, "");
+}
+
+DLLEXPORT uint64_t jl_ios_get_nbyte_int(ios_t *s, const size_t n)
+{
+    assert(n <= 8);
+    size_t space, ret;
+    do {
+        space = s->size - s->bpos;
+        ret = ios_readprep(s, n);
+        if (space == ret && ret < n)
+            throw_eof_error();
+    } while(ret < n);
+    uint64_t x = 0;
+    uint8_t *buf = (uint8_t*)&s->buf[s->bpos];
+    if (n == 8) {
+        // expecting loop unrolling optimization
+        for (size_t i = 0; i < 8; i++)
+            x |= (uint64_t)buf[i] << (i << 3);
+    }
+    else if (n >= 4) {
+        // expecting loop unrolling optimization
+        for (size_t i = 0; i < 4; i++)
+            x |= (uint64_t)buf[i] << (i << 3);
+        for (size_t i = 4; i < n; i++)
+            x |= (uint64_t)buf[i] << (i << 3);
+    }
+    else {
+        for (size_t i = 0; i < n; i++)
+            x |= (uint64_t)buf[i] << (i << 3);
+    }
+    s->bpos += n;
+    return x;
 }
 
 // -- syscall utilities --
@@ -401,16 +440,17 @@ int jl_process_stop_signal(int status) { return WSTOPSIG(status); }
 
 // -- access to std filehandles --
 
-JL_STREAM *JL_STDIN=0;
-JL_STREAM *JL_STDOUT=0;
-JL_STREAM *JL_STDERR=0;
+JL_STREAM *JL_STDIN  = (JL_STREAM*)STDIN_FILENO;
+JL_STREAM *JL_STDOUT = (JL_STREAM*)STDOUT_FILENO;
+JL_STREAM *JL_STDERR = (JL_STREAM*)STDERR_FILENO;
 
-JL_STREAM *jl_stdin_stream(void)  { return (JL_STREAM*)JL_STDIN; }
-JL_STREAM *jl_stdout_stream(void) { return (JL_STREAM*)JL_STDOUT; }
-JL_STREAM *jl_stderr_stream(void) { return (JL_STREAM*)JL_STDERR; }
+JL_STREAM *jl_stdin_stream(void)  { return JL_STDIN; }
+JL_STREAM *jl_stdout_stream(void) { return JL_STDOUT; }
+JL_STREAM *jl_stderr_stream(void) { return JL_STDERR; }
 
 // CPUID
 
+#ifdef HAVE_CPUID
 DLLEXPORT void jl_cpuid(int32_t CPUInfo[4], int32_t InfoType)
 {
 #if defined _MSC_VER
@@ -433,47 +473,79 @@ DLLEXPORT void jl_cpuid(int32_t CPUInfo[4], int32_t InfoType)
     );
 #endif
 }
+#endif
 
 // -- set/clear the FZ/DAZ flags on x86 & x86-64 --
 #ifdef __SSE__
 
-DLLEXPORT uint8_t jl_zero_subnormals(uint8_t isZero)
+// Cache of information recovered from jl_cpuid.
+// In a multithreaded environment, there will be races on subnormal_flags,
+// but they are harmless idempotent races.  If we ever embrace C11, then
+// subnormal_flags should be declared atomic.
+static volatile int32_t subnormal_flags = 1;
+
+static int32_t get_subnormal_flags()
 {
-    uint32_t flags = 0x00000000;
-    int32_t info[4];
-
-    jl_cpuid(info, 0);
-    if (info[0] >= 1) {
-        jl_cpuid(info, 0x00000001);
-        if ((info[3] & ((int)1 << 26)) != 0) {
-            // SSE2 supports both FZ and DAZ
-            flags = 0x00008040;
-        }
-        else if ((info[3] & ((int)1 << 25)) != 0) {
-            // SSE supports only the FZ flag
-            flags = 0x00008000;
+    uint32_t f = subnormal_flags;
+    if (f & 1) {
+        // CPU capabilities not yet inspected.
+        f = 0;
+        int32_t info[4];
+        jl_cpuid(info, 0);
+        if (info[0] >= 1) {
+            jl_cpuid(info, 0x00000001);
+            if (info[3] & (1 << 26)) {
+                // SSE2 supports both FZ and DAZ
+                f = 0x00008040;
+            }
+            else if (info[3] & (1 << 25)) {
+                // SSE supports only the FZ flag
+                f = 0x00008000;
+            }
         }
+        subnormal_flags = f;
     }
+    return f;
+}
 
+// Returns non-zero if subnormals go to 0; zero otherwise.
+DLLEXPORT int32_t jl_get_zero_subnormals(int8_t isZero)
+{
+    uint32_t flags = get_subnormal_flags();
+    return _mm_getcsr() & flags;
+}
+
+// Return zero on success, non-zero on failure.
+DLLEXPORT int32_t jl_set_zero_subnormals(int8_t isZero)
+{
+    uint32_t flags = get_subnormal_flags();
     if (flags) {
-        if (isZero) {
-            _mm_setcsr(_mm_getcsr() | flags);
-        }
-        else {
-            _mm_setcsr(_mm_getcsr() & ~flags);
-        }
-        return 1;
+        uint32_t state = _mm_getcsr();
+        if (isZero)
+            state |= flags;
+        else
+            state &= ~flags;
+        _mm_setcsr(state);
+        return 0;
+    }
+    else {
+        // Report a failure only if user is trying to enable FTZ/DAZ.
+        return isZero;
     }
-    return 0;
 }
 
 #else
 
-DLLEXPORT uint8_t jl_zero_subnormals(uint8_t isZero)
+DLLEXPORT int32_t jl_get_zero_subnormals(int8_t isZero)
 {
     return 0;
 }
 
+DLLEXPORT int32_t jl_set_zero_subnormals(int8_t isZero)
+{
+    return isZero;
+}
+
 #endif
 
 // -- processor native alignment information --
@@ -499,7 +571,7 @@ DLLEXPORT jl_value_t *jl_is_char_signed()
 DLLEXPORT void jl_field_offsets(jl_datatype_t *dt, ssize_t *offsets)
 {
     size_t i;
-    for(i=0; i < jl_tuple_len(dt->names); i++) {
+    for(i=0; i < jl_datatype_nfields(dt); i++) {
         offsets[i] = jl_field_offset(dt, i);
     }
 }
@@ -507,15 +579,15 @@ DLLEXPORT void jl_field_offsets(jl_datatype_t *dt, ssize_t *offsets)
 // -- misc sysconf info --
 
 #ifdef _OS_WINDOWS_
-static long chachedPagesize = 0;
+static long cachedPagesize = 0;
 long jl_getpagesize(void)
 {
-    if (!chachedPagesize) {
+    if (!cachedPagesize) {
         SYSTEM_INFO systemInfo;
         GetSystemInfo (&systemInfo);
-        chachedPagesize = systemInfo.dwPageSize;
+        cachedPagesize = systemInfo.dwPageSize;
     }
-    return chachedPagesize;
+    return cachedPagesize;
 }
 #else
 long jl_getpagesize(void)
@@ -551,64 +623,17 @@ DLLEXPORT long jl_SC_CLK_TCK(void)
 #endif
 }
 
-// Dynamic Library interrogation
-
-#ifdef __APPLE__
-// This code gratefully lifted from: http://stackoverflow.com/questions/20481058
-#ifdef __LP64__
-typedef struct mach_header_64 mach_header_t;
-typedef struct segment_command_64 segment_command_t;
-typedef struct nlist_64 nlist_t;
-#else
-typedef struct mach_header mach_header_t;
-typedef struct segment_command segment_command_t;
-typedef struct nlist nlist_t;
-#endif
-
-static const char *first_external_symbol_for_image(const mach_header_t *header)
+DLLEXPORT size_t jl_get_field_offset(jl_datatype_t *ty, int field)
 {
-    Dl_info info;
-    if (dladdr(header, &info) == 0)
-        return NULL;
-
-    segment_command_t *seg_linkedit = NULL;
-    segment_command_t *seg_text = NULL;
-    struct symtab_command *symtab = NULL;
-
-    struct load_command *cmd = (struct load_command *)((intptr_t)header + sizeof(mach_header_t));
-    for (uint32_t i = 0; i < header->ncmds; i++, cmd = (struct load_command *)((intptr_t)cmd + cmd->cmdsize)) {
-        switch(cmd->cmd) {
-        case LC_SEGMENT:
-        case LC_SEGMENT_64:
-            if (!strcmp(((segment_command_t *)cmd)->segname, SEG_TEXT))
-                seg_text = (segment_command_t *)cmd;
-            else if (!strcmp(((segment_command_t *)cmd)->segname, SEG_LINKEDIT))
-                seg_linkedit = (segment_command_t *)cmd;
-            break;
-
-        case LC_SYMTAB:
-            symtab = (struct symtab_command *)cmd;
-            break;
-        }
-    }
-
-    if ((seg_text == NULL) || (seg_linkedit == NULL) || (symtab == NULL))
-        return NULL;
-
-    intptr_t file_slide = ((intptr_t)seg_linkedit->vmaddr - (intptr_t)seg_text->vmaddr) - seg_linkedit->fileoff;
-    intptr_t strings = (intptr_t)header + (symtab->stroff + file_slide);
-    nlist_t *sym = (nlist_t *)((intptr_t)header + (symtab->symoff + file_slide));
-
-    for (uint32_t i = 0; i < symtab->nsyms; i++, sym++) {
-        if ((sym->n_type & N_EXT) != N_EXT || !sym->n_value)
-            continue;
-
-        return (const char*)strings + sym->n_un.n_strx;
-    }
-
-    return NULL;
+    if (field > jl_datatype_nfields(ty))
+        jl_error("This type does not have that many fields");
+    return jl_field_offset(ty, field);
+}
+
+DLLEXPORT size_t jl_get_alignment(jl_datatype_t *ty)
+{
+    return ty->alignment;
 }
-#endif
 
 // Takes a handle (as returned from dlopen()) and returns the absolute path to the image loaded
 DLLEXPORT const char *jl_pathname_for_handle(uv_lib_t *uv_lib)
@@ -618,22 +643,23 @@ DLLEXPORT const char *jl_pathname_for_handle(uv_lib_t *uv_lib)
 
     void *handle = uv_lib->handle;
 #ifdef __APPLE__
+    // Iterate through all images currently in memory
     for (int32_t i = _dyld_image_count(); i >= 0 ; i--) {
-        const char *first_symbol = first_external_symbol_for_image((const mach_header_t *)_dyld_get_image_header(i));
-        if (first_symbol && strlen(first_symbol) > 1) {
-            handle = (void*)((intptr_t)handle | 1); // in order to trigger findExportedSymbol instead of findExportedSymbolInImageOrDependentImages. See `dlsym` implementation at http://opensource.apple.com/source/dyld/dyld-239.3/src/dyldAPIs.cpp
-            first_symbol++; // in order to remove the leading underscore
-            void *address = dlsym(handle, first_symbol);
-            Dl_info info;
-            if (dladdr(address, &info))
-                return info.dli_fname;
-        }
+        // dlopen() each image, check handle
+        const char *image_name = _dyld_get_image_name(i);
+        uv_lib_t *probe_lib = jl_load_dynamic_library(image_name, JL_RTLD_DEFAULT);
+        void *probe_handle = probe_lib->handle;
+        uv_dlclose(probe_lib);
+
+        // If the handle is the same as what was passed in (modulo mode bits), return this image name
+        if (((intptr_t)handle & (-4)) == ((intptr_t)probe_handle & (-4)))
+            return image_name;
     }
 
 #elif defined(_OS_WINDOWS_)
 
     wchar_t *pth16 = (wchar_t*)malloc(32768); // max long path length
-    DWORD n16 = GetModuleFileNameW(handle,pth16,32768);
+    DWORD n16 = GetModuleFileNameW((HMODULE)handle,pth16,32768);
     if (n16 <= 0) {
         free(pth16);
         return NULL;
@@ -657,6 +683,15 @@ DLLEXPORT const char *jl_pathname_for_handle(uv_lib_t *uv_lib)
 
     struct link_map *map;
     dlinfo(handle, RTLD_DI_LINKMAP, &map);
+#ifdef __has_feature
+#if __has_feature(memory_sanitizer)
+    __msan_unpoison(&map,sizeof(struct link_map*));
+    if (map) {
+      __msan_unpoison(map, sizeof(struct link_map));
+      __msan_unpoison_string(map->l_name);
+    }
+#endif
+#endif
     if (map)
         return map->l_name;
 
@@ -673,8 +708,8 @@ static BOOL CALLBACK jl_EnumerateLoadedModulesProc64(
   _In_opt_  PVOID a
 )
 {
-    jl_array_grow_end(a, 1);
-    //XXX: change to jl_arrayset if array storage allocation for Array{String,1} changes:
+    jl_array_grow_end((jl_array_t*)a, 1);
+    //XXX: change to jl_arrayset if array storage allocation for Array{ByteString,1} changes:
     jl_value_t *v = jl_cstr_to_string(ModuleName);
     jl_cellset(a, jl_array_dim0(a)-1, v);
     return TRUE;
@@ -696,6 +731,54 @@ DLLEXPORT void jl_raise_debugger(void)
 #endif // _OS_WINDOWS_
 }
 
+DLLEXPORT jl_sym_t* jl_get_OS_NAME()
+{
+#if defined(_OS_WINDOWS_)
+    return jl_symbol("Windows");
+#elif defined(_OS_LINUX_)
+    return jl_symbol("Linux");
+#elif defined(_OS_FREEBSD_)
+    return jl_symbol("FreeBSD");
+#elif defined(_OS_DARWIN_)
+    return jl_symbol("Darwin");
+#else
+#warning OS_NAME is Unknown
+    return jl_symbol("Unknown");
+#endif
+}
+
+DLLEXPORT jl_sym_t* jl_get_ARCH()
+{
+    static jl_sym_t* ARCH = NULL;
+    if (!ARCH)
+        ARCH = (jl_sym_t*) jl_get_global(jl_base_module, jl_symbol("ARCH"));
+    return ARCH;
+}
+
+DLLEXPORT size_t jl_maxrss()
+{
+#if defined(_OS_WINDOWS_)
+    PROCESS_MEMORY_COUNTERS counter;
+    GetProcessMemoryInfo( GetCurrentProcess( ), &counter, sizeof(counter) );
+    return (size_t)counter.PeakWorkingSetSize;
+
+#elif defined(_OS_LINUX_) || defined(_OS_DARWIN_) || defined (_OS_FREEBSD_)
+    struct rusage rusage;
+    getrusage( RUSAGE_SELF, &rusage );
+
+#if defined(_OS_LINUX_)
+    return (size_t)(rusage.ru_maxrss * 1024);
+#else
+    return (size_t)rusage.ru_maxrss;
+#endif
+
+#else
+    return (size_t)0;
+#endif
+}
+
+
+
 #ifdef __cplusplus
 }
 #endif
