@@ -1,3 +1,5 @@
+// This file is a part of Julia. License is MIT: http://julialang.org/license
+
 #include "uv.h"
 #define XX(uc,lc) :UV_##uc,
 #define YY(uc,lc) (:UV_##uc,UV__##uc),
@@ -5,8 +7,8 @@ const uv_handle_types = [UV_HANDLE_TYPE_MAP(XX) :UV_FILE]
 const uv_req_types = [UV_REQ_TYPE_MAP(XX)]
 const uv_err_vals = [UV_ERRNO_MAP(YY)]
 let
-    handles = [:UV_UNKNOWN_HANDLE, uv_handle_types, :UV_HANDLE_TYPE_MAX, :UV_RAW_FD, :UV_RAW_HANDLE]
-    reqs = [:UV_UNKNOWN_REQ, uv_req_types, :UV_REQ_TYPE_PRIVATE,:UV_REQ_TYPE_MAX]
+    handles = [:UV_UNKNOWN_HANDLE; uv_handle_types; :UV_HANDLE_TYPE_MAX; :UV_RAW_FD; :UV_RAW_HANDLE]
+    reqs = [:UV_UNKNOWN_REQ; uv_req_types; :UV_REQ_TYPE_PRIVATE; :UV_REQ_TYPE_MAX]
     for i=0:(length(handles)-1)
     @eval const $(handles[i+1]) = $i
     end
