@@ -1,3 +1,117 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
+@test reim(2 + 3im) == (2, 3)
+
+# Basic arithmetic
+for T in (Float16, Float32, Float64, BigFloat)
+    t = true
+    f = false
+
+    # Add and subtract
+    @test isequal(T(+0.0) + im, Complex(T(+0.0), T(+1.0)))
+    @test isequal(T(-0.0) + im, Complex(T(-0.0), T(+1.0)))
+    @test isequal(T(+0.0) - im, Complex(T(+0.0), T(-1.0)))
+    @test isequal(T(-0.0) - im, Complex(T(-0.0), T(-1.0)))
+    @test isequal(T(+1.0) + im, Complex(T(+1.0), T(+1.0)))
+    @test isequal(T(-1.0) + im, Complex(T(-1.0), T(+1.0)))
+    @test isequal(T(+1.0) - im, Complex(T(+1.0), T(-1.0)))
+    @test isequal(T(-1.0) - im, Complex(T(-1.0), T(-1.0)))
+    @test isequal(im + T(+0.0), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im + T(-0.0), Complex(T(-0.0), T(+1.0)))
+    @test isequal(im - T(+0.0), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im - T(-0.0), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im + T(+1.0), Complex(T(+1.0), T(+1.0)))
+    @test isequal(im + T(-1.0), Complex(T(-1.0), T(+1.0)))
+    @test isequal(im - T(+1.0), Complex(T(-1.0), T(+1.0)))
+    @test isequal(im - T(-1.0), Complex(T(+1.0), T(+1.0)))
+    @test isequal(T(f) + im, Complex(T(+0.0), T(+1.0)))
+    @test isequal(T(t) + im, Complex(T(+1.0), T(+1.0)))
+    @test isequal(T(f) - im, Complex(T(+0.0), T(-1.0)))
+    @test isequal(T(t) - im, Complex(T(+1.0), T(-1.0)))
+    @test isequal(im + T(f), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im + T(t), Complex(T(+1.0), T(+1.0)))
+    @test isequal(im - T(f), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im - T(t), Complex(T(-1.0), T(+1.0)))
+
+    # Multiply
+    @test isequal(T(+0.0) * im, Complex(T(+0.0), T(+0.0)))
+    @test isequal(T(-0.0) * im, Complex(T(-0.0), T(-0.0)))
+    @test isequal(T(+1.0) * im, Complex(T(+0.0), T(+1.0)))
+    @test isequal(T(-1.0) * im, Complex(T(-0.0), T(-1.0)))
+    @test isequal(im * T(+0.0), Complex(T(+0.0), T(+0.0)))
+    @test isequal(im * T(-0.0), Complex(T(-0.0), T(-0.0)))
+    @test isequal(im * T(+1.0), Complex(T(+0.0), T(+1.0)))
+    @test isequal(im * T(-1.0), Complex(T(-0.0), T(-1.0)))
+end
+
+# Test math functions. We compare to BigFloat instead of hard-coding
+# values, assuming that BigFloat has an independent and independently
+# tested implementation.
+for T in (Float32, Float64)
+    x = Complex{T}(1//3 + 1//4*im)
+    y = Complex{T}(1//2 + 1//5*im)
+    yi = 4
+    # Test random values
+    @test_approx_eq x^y big(x)^big(y)
+    @test_approx_eq x^yi big(x)^yi
+    @test_approx_eq abs(x) abs(big(x))
+    @test_approx_eq abs2(x) abs2(big(x))
+    @test_approx_eq acos(x) acos(big(x))
+    @test_approx_eq acosh(1+x) acosh(1+big(x))
+    @test_approx_eq angle(x) angle(big(x))
+    @test_approx_eq asin(x) asin(big(x))
+    @test_approx_eq asinh(x) asinh(big(x))
+    @test_approx_eq atan(x) atan(big(x))
+    @test_approx_eq atanh(x) atanh(big(x))
+    @test_approx_eq cis(real(x)) cis(real(big(x)))
+    @test_approx_eq cis(x) cis(big(x))
+    @test_approx_eq cos(x) cos(big(x))
+    @test_approx_eq cosh(x) cosh(big(x))
+    @test_approx_eq exp(x) exp(big(x))
+    @test_approx_eq exp10(x) exp10(big(x))
+    @test_approx_eq exp2(x) exp2(big(x))
+    @test_approx_eq_eps expm1(x) expm1(big(x)) eps(T)
+    @test_approx_eq log(x) log(big(x))
+    @test_approx_eq log10(x) log10(big(x))
+    @test_approx_eq log1p(x) log1p(big(x))
+    @test_approx_eq log2(x) log2(big(x))
+    @test_approx_eq sin(x) sin(big(x))
+    @test_approx_eq sinh(x) sinh(big(x))
+    @test_approx_eq sqrt(x) sqrt(big(x))
+    @test_approx_eq tan(x) tan(big(x))
+    @test_approx_eq tanh(x) tanh(big(x))
+    # Test inverses
+    @test_approx_eq acos(cos(x)) x
+    @test_approx_eq acosh(cosh(x)) x
+    @test_approx_eq asin(sin(x)) x
+    @test_approx_eq asinh(sinh(x)) x
+    @test_approx_eq atan(tan(x)) x
+    @test_approx_eq atanh(tanh(x)) x
+    @test_approx_eq cos(acos(x)) x
+    @test_approx_eq cosh(acosh(1+x)) 1+x
+    @test_approx_eq exp(log(x)) x
+    @test_approx_eq exp10(log10(x)) x
+    @test_approx_eq exp2(log2(x)) x
+    @test_approx_eq expm1(log1p(x)) x
+    @test_approx_eq log(exp(x)) x
+    @test_approx_eq log10(exp10(x)) x
+    @test_approx_eq log1p(expm1(x)) x
+    @test_approx_eq log2(exp2(x)) x
+    @test_approx_eq sin(asin(x)) x
+    @test_approx_eq sinh(asinh(x)) x
+    @test_approx_eq sqrt(x)^2 x
+    @test_approx_eq sqrt(x^2) x
+    @test_approx_eq tan(atan(x)) x
+    @test_approx_eq tanh(atanh(x)) x
+    # Test some properties
+    @test_approx_eq cosh(x) (exp(x)+exp(-x))/2
+    @test_approx_eq cosh(x)^2-sinh(x)^2 1
+    @test_approx_eq sin(x)^2+cos(x)^2 1
+    @test_approx_eq sinh(x) (exp(x)-exp(-x))/2
+    @test_approx_eq tan(x) sin(x)/cos(x)
+    @test_approx_eq tanh(x) sinh(x)/cosh(x)
+end
+
 # sqrt:
 # tests special values from csqrt man page
 # as well as conj(sqrt(z)) = sqrt(conj(z))
@@ -200,6 +314,9 @@ end
 @test_approx_eq complex(0.0,1.0)^complex(2.0,0) complex(-1.0, 0.0)
 @test_approx_eq complex(1.0,2.0)^complex(3.0,0) complex(-11.0, -2.0)
 
+@test isequal(complex(0.0,0.0)^false, complex(1.0,0.0))
+@test isequal(complex(0.0,0.0)^0, complex(1.0,0.0))
+
 # sinh: has properties
 #  sinh(conj(z)) = conj(sinh(z))
 #  sinh(-z) = -sinh(z)
@@ -415,7 +532,7 @@ end
 
 @test_throws DomainError  tan(complex( Inf, 5.0))
 @test isequal(tan(complex( Inf, Inf)),complex( 0.0, 1.0))
-@test isequal(tan(complex( Inf,-Inf)),complex (0.0,-1.0))
+@test isequal(tan(complex( Inf,-Inf)),complex( 0.0,-1.0))
 @test isequal(tan(complex(-Inf, Inf)),complex(-0.0, 1.0))
 @test isequal(tan(complex(-Inf,-Inf)),complex(-0.0,-1.0))
 
@@ -678,6 +795,8 @@ end
 @test_throws DomainError complex(2,2)^(-2)
 @test complex(2.0,2.0)^(-2) === complex(0.0, -0.125)
 
+@test complex(1.0,[1.0,1.0]) == [complex(1.0,1.0), complex(1.0,1.0)]
+@test complex([1.0,1.0],1.0) == [complex(1.0,1.0), complex(1.0,1.0)]
 # robust division of Float64
 # hard complex divisions from Fig 6 of arxiv.1210.4539
 z7 = Complex{Float64}(3.898125604559113300e289, 8.174961907852353577e295)
@@ -707,7 +826,7 @@ function logacc(x::Float64,expected::Float64)
   (x == Inf || x == -Inf) && (return 0)
   isnan(x) && (return 0)
   ra = relacc(BigFloat(x),BigFloat(expected))
-  max(ifloor(-log2(ra)),0)
+  max(floor(Int,-log2(ra)),0)
 end
 # the robust division algorithm should have 53 or 52
 # bits accuracy for each of the hard divisions
@@ -729,6 +848,26 @@ end
 @test log10(10+0im) === 1.0 + 0.0im
 @test log2(2+0im) === 1.0 + 0.0im
 
+# sign
+for T in (Float32, Float64)
+    z = Complex{T}(1)
+    @test typeof(sign(z)) == typeof(z)
+    z = Complex{T}(0)
+    @test typeof(sign(z)) == typeof(z)
+end
+for T in (Int32, Int64)
+    z = Complex{T}(1)
+    @test typeof(sign(z)) == typeof(float(z))
+    z = Complex{T}(0)
+    @test typeof(sign(z)) == typeof(float(z))
+end
+
+@test sign(0 + 0im) == 0
+@test sign(2 + 0im) == 1
+@test sign(-2 + 0im) == -1
+@test sign(1 + im) ≈ (1 + im) / sqrt(2)
+@test sign(1 - im) ≈ (1 - im) / sqrt(2)
+
 # cis
 @test_approx_eq cis(0.0+1.0im) 0.367879441171442321595523770161460867445811131031767834507836+0.0im
 @test_approx_eq cis(1.0+0.0im) 0.54030230586813971740093660744297660373231042061+0.84147098480789650665250232163029899962256306079im
@@ -746,3 +885,26 @@ end
 @test exp10(1.0+0.0im) == 10.0+0.0im
 #wolframalpha
 @test_approx_eq exp10(1.0+2.0im) -1.0701348355877020772086517528518239460495529361-9.9425756941378968736161937190915602112878340717im
+
+# round #8291
+@test round(Complex(1.125, 0.875), 2) == Complex(1.12, 0.88)
+@test round(Complex(1.5, 0.5), RoundDown, RoundUp) == Complex(1.0, 1.0)
+@test round([1:5;] + im) == [1:5;] + im
+@test round([1:5;] + 0.5im) == [1.0:5.0;]
+
+# float #8291
+@test float(Complex(1, 2)) == Complex(1.0, 2.0)
+@test round(float(Complex(π, e)),3) == Complex(3.142, 2.718)
+
+# Complex32 arithmetic #10003
+@test Float16(1)+Float16(1)im === Complex32(1, 1)
+@test Float16(1)-Float16(1)im === Float16(1)+Float16(-1)im === Complex32(1, -1)
+@test Float16(1)*im === Complex32(im)
+@test Float16(1)/im === 1.0f0/im === Complex(0.0, -1.0)
+@test Float16(1)^im === Complex32(1) === Float16(1)+Float16(0)im
+
+# issue/PR #10148
+@test typeof(Int8(1) - im) == Complex{Int8}
+
+# issue #10926
+@test typeof(π - 1im) == Complex{Float64}
