@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 function write_and_readchomp(data, cmd::Cmd)
     r, w, p = readandwrite(cmd)
     print(w,data); close(w)
@@ -9,7 +11,7 @@ end
 function mktree(d::Dict)
     lstree = ""
     for (name, data) in d
-        if isa(data, String)
+        if isa(data, AbstractString)
             sha1 = write_and_readchomp(data, `git hash-object -w --stdin`)
             lstree *= "100644 blob $sha1\t$name\n"
         elseif isa(data, Dict)
@@ -24,7 +26,7 @@ function mktree(d::Dict)
     write_and_readchomp(lstree, `git mktree`)
 end
 
-function verify_tree(d::Dict, tree::String)
+function verify_tree(d::Dict, tree::AbstractString)
     # check that tree matches d
     seen = Set()
     for line in eachline(`git ls-tree $tree`)
@@ -33,7 +35,7 @@ function verify_tree(d::Dict, tree::String)
         perm, kind, sha1, name = m.captures
         @test haskey(d,name)
         data = d[name]
-        if isa(data, String)
+        if isa(data, AbstractString)
             @test kind == "blob"
             @test data == readall(`git cat-file blob $sha1`)
         elseif isa(data, Dict)
@@ -58,7 +60,7 @@ function verify_work(d::Dict)
             continue
         end
         @test ispath(name)
-        if isa(data, String)
+        if isa(data, AbstractString)
             @test isfile(name)
             @test readall(name) == data
         elseif isa(data, Dict)
@@ -70,9 +72,12 @@ function verify_work(d::Dict)
         end
     end
     # check for anything that's not in d
-    for line in eachline(`ls -A`)
-        name = chomp(line)
-        @test name == ".git" || haskey(d,name)
+    extrapath = @windows? joinpath(JULIA_HOME,"..","Git","usr","bin")*";" : ""
+    withenv("PATH" => extrapath * ENV["PATH"]) do
+        for line in eachline(`ls -A`)
+            name = chomp(line)
+            @test name == ".git" || haskey(d,name)
+        end
     end
 end
 
@@ -82,16 +87,19 @@ function git_verify(h::Dict, i::Dict, w::Dict)
     verify_work(w)
 end
 
-function git_setup(h::Dict, i::Dict, w::Dict, parents::String...)
+function git_setup(h::Dict, i::Dict, w::Dict, parents::AbstractString...)
     # create tree objects
     headt = mktree(h)
     index = mktree(i)
     work  = mktree(w)
 
     # clear the repo
-    for line in eachline(`ls -A`)
-        name = chomp(line)
-        name == ".git" || rm(name, recursive=true)
+    extrapath = @windows? joinpath(JULIA_HOME,"..","Git","usr","bin")*";" : ""
+    withenv("PATH" => extrapath * ENV["PATH"]) do
+        for line in eachline(`ls -A`)
+            name = chomp(line)
+            name == ".git" || rm(name, recursive=true)
+        end
     end
 
     # create the head commit
