@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 include("../perfutil.jl")
 
 function cat2d_perf(n, iter)
@@ -45,7 +47,7 @@ function vcat_perf(n, iter)
     a = rand(n,n)
     b = rand(n,n)
     for i = 1:iter
-        c = [a, b, b, a]
+        c = [a; b; b; a]
     end
 end
 
