@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # https://github.com/JuliaLang/julia/issues/4707
 
 #----------------- Get the A matrix
@@ -27,18 +29,18 @@ function spdiags(B,d,m,n)
     d = d[:]
     p = length(d)
 
-    len = zeros(p+1,1)
+    len = zeros(Int, p+1, 1)
     for k = 1:p
-        len[k+1] = int(len[k]+length(max(1,1-d[k]):min(m,n-d[k])))
+        len[k+1] = len[k] + length(max(1,1-d[k]):min(m,n-d[k]))
     end
-    a = zeros(int(len[p+1]),3)
+    a = zeros(Int, len[p+1], 3)
     for k = 1:p
         # Append new d[k]-th diagonal to compact form
         i = max(1,1-d[k]):min(m,n-d[k])
-        a[(int(len[k])+1):int(len[k+1]),:] = [i i+d[k] B[i+(m>=n)*d[k],k]]
+        a[(len[k]+1):len[k+1],:] = [i i+d[k] B[i+(m>=n)*d[k],k]]
     end
 
-    A = sparse(int(a[:,1]),int(a[:,2]),a[:,3],m,n);
+    A = sparse(a[:,1],a[:,2],a[:,3],m,n)
 
     return A
 
