@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 function laplace_iter_devec(u, dx2, dy2, Niter, N)
     uout = copy(u)
     for iter = 1:Niter
@@ -35,3 +37,19 @@ function laplace_vec()
     dx2 = dy2 = 0.1*0.1
     u = laplace_iter_vec(u, dx2, dy2, Niter, N)
 end
+
+function laplace_iter_vec_sub(u, dx2, dy2, Niter, N)
+    for i = 1:Niter
+        u[2:N-1, 2:N-1] = ((sub(u, 1:N-2, 2:N-1) + sub(u,3:N, 2:N-1))*dy2 + (sub(u,2:N-1,1:N-2) + sub(u,2:N-1, 3:N))*dx2) * (1./ (2*(dx2+dy2)))
+    end
+    return u
+end
+
+function laplace_vec_sub()
+    N = 150
+    u = zeros(N,N)
+    u[1,:] = 1
+    Niter = 2^10
+    dx2 = dy2 = 0.1*0.1
+    u = laplace_iter_vec_sub(u, dx2, dy2, Niter, N)
+end
