@@ -1,7 +1,9 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 include("../perfutil.jl")
 
 #If running from .. directory, include "shootout/" path to data file
-rpath(filename::String) = string(contains(pwd(), "shootout") ? "" : "shootout/", filename)
+rpath(filename::AbstractString) = string(contains(pwd(), "shootout") ? "" : "shootout/", filename)
 
 include("binary_trees.jl")
 @timeit binary_trees(10) "binary_trees" "Allocate and deallocate many many binary trees"
@@ -41,3 +43,5 @@ include("revcomp.jl")
 
 include("spectralnorm.jl")
 @timeit spectralnorm() "spectralnorm" "Eigenvalue using the power method"
+
+maxrss("shootout")
