@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # Peter Norvig's Spelling Corrector
 # based off of the python implementation at http://norvig.com/spell-correct.html
 
@@ -11,28 +13,26 @@
 # @kmsquire, 2013-11-29
 #
 
-Pkg.add("DataStructures")
-
-using DataStructures
-using HTTPClient.HTTPC
-
 include("../perfutil.jl")
 
 words(text) = eachmatch(r"[a-z]+", lowercase(text))
 
 function train(features)
-    model = DefaultDict(String, Int, 1)
+    model = Dict{AbstractString, Int}()
     for f in features
-        model[f.match] += 1
+        model[f.match] = 2
     end
     return model
 end
 
-const NWORDS = train(words(bytestring(get("http://norvig.com/big.txt").body)))
+if !isfile("big.txt")
+    download("http://norvig.com/big.txt", "big.txt")
+end
+const NWORDS = train(words(readall("big.txt")))
 
 const alphabet = "abcdefghijklmnopqrstuvwxyz"
 
-function edits1(word::String)
+function edits1(word::AbstractString)
     splits     = [(word[1:i], word[i+1:end]) for i=0:length(word) ]
     deletes    = ["$a$(b[2:end])"               for (a,b) in splits[1:end-1]]
     transposes = ["$a$(b[2])$(b[1])$(b[3:end])" for (a,b) in splits[1:end-2]]
@@ -41,8 +41,8 @@ function edits1(word::String)
     return Set([deletes; transposes; replaces[:]; inserts[:]])
 end
 
-function known_edits2(word::String)
-    xs = Set{String}()
+function known_edits2(word::AbstractString)
+    xs = Set{AbstractString}()
     for e1 in edits1(word)
         for e2 in edits1(e1)
             haskey(NWORDS, e2) && push!(xs, e2)
@@ -52,14 +52,14 @@ function known_edits2(word::String)
 end
 
 function known(words)
-    xs = Set{String}()
+    xs = Set{AbstractString}()
     for word in words
         haskey(NWORDS, word) && push!(xs, word)
     end
     xs
 end
 
-function correct(word::String)
+function correct(word::AbstractString)
     candidates = known([word])
     length(candidates) == 0 && (candidates = known(edits1(word)))
     length(candidates) == 0 && (candidates = known_edits2(word) )
@@ -74,7 +74,7 @@ function spelltest(tests; bias=0, verbose=false)
     n, bad, unknown, start = 0, 0, 0, tic()
     if bias > 0
         for target in keys(tests)
-            NWORDS[target] += bias
+            NWORDS[target] = get(NWORDS, target, 1) + bias
         end
     end
     for (target,wrongs) in tests
@@ -83,7 +83,7 @@ function spelltest(tests; bias=0, verbose=false)
             w = correct(wrong)
             if w!=target
                 bad += 1
-                unknown += !(target in NWORDS)
+                unknown += !(target in keys(NWORDS))
                 if verbose
                     @printf("correct(%s) => %s (%d); expected %s (%d)\n",
                             wrong, w, NWORDS[w], target, NWORDS[target])
@@ -92,11 +92,11 @@ function spelltest(tests; bias=0, verbose=false)
         end
     end
 
-    return ["bad"=>bad, "n"=>n, "bias"=>bias, "pct"=>int(100. - 100.*bad/n),
-            "unknown"=>unknown, "secs"=>toc() ]
+    return Dict("bad"=>bad, "n"=>n, "bias"=>bias, "pct"=>round(Int, 100. - 100.*bad/n),
+                "unknown"=>unknown, "secs"=>toc())
 end
 
-const tests1 = [ "access"=> "acess", "accessing"=> "accesing", "accommodation"=>
+const tests1 = Dict("access"=> "acess", "accessing"=> "accesing", "accommodation"=>
 "accomodation acommodation acomodation", "account"=> "acount", "address"=>
 "adress adres", "addressable"=> "addresable", "arranged"=> "aranged arrainged",
 "arrangeing"=> "aranging", "arrangement"=> "arragment", "articles"=> "articals",
@@ -156,6 +156,6 @@ const tests1 = [ "access"=> "acess", "accessing"=> "accesing", "accommodation"=>
 "variable"=> "varable", "variant"=> "vairiant", "various"=> "vairious",
 "visited"=> "fisited viseted vistid vistied", "visitors"=> "vistors",
 "voluntary"=> "volantry", "voting"=> "voteing", "wanted"=> "wantid wonted",
-"whether"=> "wether", "wrote"=> "rote wote"]
+"whether"=> "wether", "wrote"=> "rote wote")
 
 @timeit(spelltest(tests1), "spell", "Peter Norvig's spell corrector")
