@@ -1,3 +1,4 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
 
 # fold(l|r) & mapfold(l|r)
 @test foldl(-, 1:5) == -13
@@ -22,9 +23,10 @@
 @test Base.mapfoldr(abs2, -, 10, 2:5) == -4
 
 # reduce & mapreduce
-@test reduce((x,y)->"($x+$y)", [9:11]) == "((9+10)+11)"
+@test reduce((x,y)->"($x+$y)", 9:11) == "((9+10)+11)"
 @test reduce(max, [8 6 7 5 3 0 9]) == 9
-@test reduce(+, 1000, [1:5]) == (1000 + 1 + 2 + 3 + 4 + 5)
+@test reduce(+, 1000, 1:5) == (1000 + 1 + 2 + 3 + 4 + 5)
+@test reduce(+,1) == 1
 
 @test mapreduce(-, +, [-10 -9 -3]) == ((10 + 9) + 3)
 @test mapreduce((x)->x[1:3], (x,y)->"($x+$y)", ["abcd", "efgh", "01234"]) == "((abc+efg)+012)"
@@ -32,14 +34,14 @@
 # sum
 
 @test sum(Int8[]) === 0
-@test sum(Int[]) === 0
+@test sum(Int[]) === Int(0)
 @test sum(Float64[]) === 0.0
 
-@test sum(int8(3)) === int8(3)
+@test sum(Int8(3)) === Int8(3)
 @test sum(3) === 3
 @test sum(3.0) === 3.0
 
-@test sum([int8(3)]) === 3
+@test sum([Int8(3)]) === 3
 @test sum([3]) === 3
 @test sum([3.0]) === 3.0
 
@@ -48,40 +50,42 @@ fz = float(z)
 @test sum(z) === 136
 @test sum(fz) === 136.0
 
-@test_throws ErrorException sum(sin, Int[])
+@test_throws ArgumentError sum(sin, Int[])
 @test sum(sin, 3) == sin(3.0)
 @test sum(sin, [3]) == sin(3.0)
-@test sum(sin, z) == sum(sin, fz) == sum(sin(fz))
+a = sum(sin, z)
+@test_approx_eq a sum(sin, fz)
+@test_approx_eq a sum(sin(fz))
 
 z = [-4, -3, 2, 5]
 fz = float(z)
 a = randn(32) # need >16 elements to trigger BLAS code path
 b = complex(randn(32), randn(32))
 @test sumabs(Float64[]) === 0.0
-@test sumabs([int8(-2)]) === 2
+@test sumabs([Int8(-2)]) === 2
 @test sumabs(z) === 14
 @test sumabs(fz) === 14.0
 @test_approx_eq sumabs(a) sum(abs(a))
 @test_approx_eq sumabs(b) sum(abs(b))
 
 @test sumabs2(Float64[]) === 0.0
-@test sumabs2([int8(-2)]) === 4
+@test sumabs2([Int8(-2)]) === 4
 @test sumabs2(z) === 54
 @test sumabs2(fz) === 54.0
 @test_approx_eq sumabs2(a) sum(abs2(a))
 @test_approx_eq sumabs2(b) sum(abs2(b))
 
 # check variants of summation for type-stability and other issues (#6069)
-sum2(itr) = invoke(sum, (Any,), itr)
+sum2(itr) = invoke(sum, Tuple{Any}, itr)
 plus(x,y) = x + y
 sum3(A) = reduce(plus, A)
-sum4(itr) = invoke(reduce, (Function, Any), plus, itr)
+sum4(itr) = invoke(reduce, Tuple{Function, Any}, plus, itr)
 sum5(A) = reduce(plus, 0, A)
-sum6(itr) = invoke(reduce, (Function, Int, Any), plus, 0, itr)
+sum6(itr) = invoke(reduce, Tuple{Function, Int, Any}, plus, 0, itr)
 sum7(A) = mapreduce(x->x, plus, A)
-sum8(itr) = invoke(mapreduce, (Function, Function, Any), x->x, plus, itr)
+sum8(itr) = invoke(mapreduce, Tuple{Function, Function, Any}, x->x, plus, itr)
 sum9(A) = mapreduce(x->x, plus, 0, A)
-sum10(itr) = invoke(mapreduce, (Function, Function, Int, Any), x->x,plus,0,itr)
+sum10(itr) = invoke(mapreduce, Tuple{Function, Function, Int, Any}, x->x,plus,0,itr)
 for f in (sum2, sum5, sum6, sum9, sum10)
     @test sum(z) == f(z)
     @test sum(Int[]) == f(Int[]) == 0
@@ -90,7 +94,7 @@ for f in (sum2, sum5, sum6, sum9, sum10)
 end
 for f in (sum3, sum4, sum7, sum8)
     @test sum(z) == f(z)
-    @test_throws ErrorException f(Int[])
+    @test_throws ArgumentError f(Int[])
     @test sum(Int[7]) == f(Int[7]) == 7
 end
 @test typeof(sum(Int8[])) == typeof(sum(Int8[1])) == typeof(sum(Int8[1 7]))
@@ -105,17 +109,19 @@ end
 @test prod(Float64[]) === 1.0
 
 @test prod([3]) === 3
-@test prod([int8(3)]) === 3
+@test prod([Int8(3)]) === 3
 @test prod([3.0]) === 3.0
 
 @test prod(z) === 120
 @test prod(fz) === 120.0
 
 @test prod(1:big(16)) == big(20922789888000)
-@test prod(big(typemax(Int64)):big(typemax(Int64))+16) == BigInt("25300281663413827620486300433089141956148633919452440329174083959168114253708467653081909888307573358090001734956158476311046124934597861626299416732205795533726326734482449215730132757595422510465791525610410023802664753402501982524443370512346073948799084936298007821432734720004795146875180123558814648586972474376192000")
+@test prod(big(typemax(Int64)):big(typemax(Int64))+16) == parse(BigInt,"25300281663413827620486300433089141956148633919452440329174083959168114253708467653081909888307573358090001734956158476311046124934597861626299416732205795533726326734482449215730132757595422510465791525610410023802664753402501982524443370512346073948799084936298007821432734720004795146875180123558814648586972474376192000")
+
+@test_throws ErrorException prod(bitunpack(trues(10)))
 
 # check type-stability
-prod2(itr) = invoke(prod, (Any,), itr)
+prod2(itr) = invoke(prod, Tuple{Any}, itr)
 @test prod(Int[]) === prod2(Int[]) === 1
 @test prod(Int[7]) === prod2(Int[7]) === 7
 @test typeof(prod(Int8[])) == typeof(prod(Int8[1])) == typeof(prod(Int8[1, 7])) == Int
@@ -123,8 +129,8 @@ prod2(itr) = invoke(prod, (Any,), itr)
 
 # maximum & minimum & extrema
 
-@test_throws ErrorException maximum(Int[])
-@test_throws ErrorException minimum(Int[])
+@test_throws ArgumentError maximum(Int[])
+@test_throws ArgumentError minimum(Int[])
 
 @test maximum(5) == 5
 @test minimum(5) == 5
@@ -147,7 +153,7 @@ prod2(itr) = invoke(prod, (Any,), itr)
 @test extrema([4., 3., NaN, 5., 2.]) == (2., 5.)
 
 @test maxabs(Int[]) == 0
-@test_throws ErrorException Base.minabs(Int[])
+@test_throws ArgumentError Base.minabs(Int[])
 
 @test maxabs(-2) == 2
 @test minabs(-2) == 2
@@ -157,6 +163,10 @@ prod2(itr) = invoke(prod, (Any,), itr)
 @test maximum(x->abs2(x), 3:7) == 49
 @test minimum(x->abs2(x), 3:7) == 9
 
+@test maximum(Int16[1]) === Int16(1)
+@test maximum(collect(Int16(1):Int16(100))) === Int16(100)
+@test maximum(Int32[1,2]) === Int32(2)
+
 # any & all
 
 @test any(Bool[]) == false
@@ -194,6 +204,27 @@ prod2(itr) = invoke(prod, (Any,), itr)
 @test reduce(&, fill(trues(5), 24))  == trues(5)
 @test reduce(&, fill(falses(5), 24)) == falses(5)
 
+@test_throws TypeError any(x->0, [false])
+@test_throws TypeError all(x->0, [false])
+
+# short-circuiting any and all
+
+let c = [0, 0], A = 1:1000
+    any(x->(c[1]=x; x==10), A)
+    all(x->(c[2]=x; x!=10), A)
+
+    @test c == [10,10]
+end
+
+# any and all with functors
+
+immutable SomeFunctor end
+Base.call(::SomeFunctor, x) = true
+
+@test any(SomeFunctor(), 1:10)
+@test all(SomeFunctor(), 1:10)
+
+
 # in
 
 @test in(1, Int[]) == false
@@ -237,3 +268,18 @@ end
 @test isequal(cummin([1 0; 0 1], 1), [1 0; 0 0])
 @test isequal(cummin([1 0; 0 1], 2), [1 0; 0 0])
 
+@test sum(collect(map(UInt8,0:255))) == 32640
+@test sum(collect(map(UInt8,254:255))) == 509
+
+A = reshape(map(UInt8, 101:109), (3,3))
+@test @inferred(sum(A)) == 945
+@test @inferred(sum(sub(A, 1:3, 1:3))) == 945
+
+A = reshape(map(UInt8, 1:100), (10,10))
+@test @inferred(sum(A)) == 5050
+@test @inferred(sum(sub(A, 1:10, 1:10))) == 5050
+
+# issue #11618
+@test sum([-0.0]) === -0.0
+@test sum([-0.0, -0.0]) === -0.0
+@test prod([-0.0, -0.0]) === 0.0
