@@ -1,3 +1,5 @@
+# This file is a part of Julia. License is MIT: http://julialang.org/license
+
 # main tests
 
 function safe_mapslices(op, A, region)
@@ -14,9 +16,9 @@ safe_maxabs{T}(A::Array{T}, region) = safe_mapslices(maximum, abs(A), region)
 safe_minabs{T}(A::Array{T}, region) = safe_mapslices(minimum, abs(A), region)
 
 Areduc = rand(3, 4, 5, 6)
-for region in {
+for region in Any[
     1, 2, 3, 4, 5, (1, 2), (1, 3), (1, 4), (2, 3), (2, 4), (3, 4),
-    (1, 2, 3), (1, 3, 4), (2, 3, 4), (1, 2, 3, 4)}
+    (1, 2, 3), (1, 3, 4), (2, 3, 4), (1, 2, 3, 4)]
     # println("region = $region")
     r = fill(NaN, Base.reduced_dims(size(Areduc), region))
     @test_approx_eq sum!(r, Areduc) safe_sum(Areduc, region)
@@ -87,7 +89,7 @@ A = reshape(1:6, 3, 2)
 
 # Small integers
 @test @inferred(sum(Int8[1], 1)) == [1]
-@test @inferred(sum(Uint8[1], 1)) == [1]
+@test @inferred(sum(UInt8[1], 1)) == [1]
 
 # Complex types
 @test typeof(@inferred(sum([1.0+1.0im], 1))) == Vector{Complex128}
@@ -98,8 +100,8 @@ A = reshape(1:6, 3, 2)
 @test typeof(@inferred(Base.prod(Base.Abs2Fun(), [1.0+1.0im], 1))) == Vector{Float64}
 
 # Heterogeneously typed arrays
-@test sum(Union(Float32, Float64)[1.0], 1) == [1.0]
-@test prod(Union(Float32, Float64)[1.0], 1) == [1.0]
+@test sum(Union{Float32, Float64}[1.0], 1) == [1.0]
+@test prod(Union{Float32, Float64}[1.0], 1) == [1.0]
 
 @test reducedim((a,b) -> a|b, [true false; false false], 1, false) == [true false]
 R = reducedim((a,b) -> a+b, [1 2; 3 4], 2, 0.0)
@@ -108,21 +110,43 @@ R = reducedim((a,b) -> a+b, [1 2; 3 4], 2, 0.0)
 @test reducedim((a,b) -> a+b, [1 2; 3 4], 1, 0) == [4 6]
 
 # inferred return types
-rt = Base.return_types(reducedim, (Function, Array{Float64, 3}, Int, Float64))
+rt = Base.return_types(reducedim, Tuple{Function, Array{Float64, 3}, Int, Float64})
 @test length(rt) == 1 && rt[1] == Array{Float64, 3}
 
+
 ## findmin/findmax
 A = [1.0 3.0 6.0;
      5.0 2.0 4.0]
-@test findmin(A, (1,)) == ([1.0 2.0 4.0], [1 4 6])
-@test findmin(A, (2,)) == (reshape([1.0,2.0], 2, 1), reshape([1,4], 2, 1))
-@test findmin(A, (1,2)) == (fill(1.0,1,1),fill(1,1,1))
-@test findmax(A, (1,)) == ([5.0 3.0 6.0], [2 3 5])
-@test findmax(A, (2,)) == (reshape([6.0,5.0], 2, 1), reshape([5,2], 2, 1))
-@test findmax(A, (1,2)) == (fill(6.0,1,1),fill(5,1,1))
+for (tup, rval, rind) in [((1,), [1.0 2.0 4.0], [1 4 6]),
+                          ((2,), reshape([1.0,2.0], 2, 1), reshape([1,4], 2, 1)),
+                          ((1,2), fill(1.0,1,1),fill(1,1,1))]
+    @test findmin(A, tup) == (rval, rind)
+    @test Base.findmin!(similar(rval), similar(rind), A) == (rval, rind)
+end
+
+for (tup, rval, rind) in [((1,), [5.0 3.0 6.0], [2 3 5]),
+                          ((2,), reshape([6.0,5.0], 2, 1), reshape([5,2], 2, 1)),
+                          ((1,2), fill(6.0,1,1),fill(5,1,1))]
+    @test findmax(A, tup) == (rval, rind)
+    @test Base.findmax!(similar(rval), similar(rind), A) == (rval, rind)
+end
 
 # issue #6672
 @test sum(Real[1 2 3; 4 5.3 7.1], 2) == reshape([6, 16.4], 2, 1)
-@test std(FloatingPoint[1,2,3], 1) == [1.0]
-@test sum({1 2;3 4},1) == [4 6]
+@test std(AbstractFloat[1,2,3], 1) == [1.0]
+@test sum(Any[1 2;3 4],1) == [4 6]
 @test sum(Vector{Int}[[1,2],[4,3]], 1)[1] == [5,5]
+
+# issue #10461
+Areduc = rand(3, 4, 5, 6)
+for region in Any[-1, 0, (-1, 2), [0, 1], (1,-2,3), [0 1;
+                                                     2 3], "hello"]
+    @test_throws ArgumentError sum(Areduc, region)
+    @test_throws ArgumentError prod(Areduc, region)
+    @test_throws ArgumentError maximum(Areduc, region)
+    @test_throws ArgumentError minimum(Areduc, region)
+    @test_throws ArgumentError sumabs(Areduc, region)
+    @test_throws ArgumentError sumabs2(Areduc, region)
+    @test_throws ArgumentError maxabs(Areduc, region)
+    @test_throws ArgumentError minabs(Areduc, region)
+end
